/* gvSIG. Geographic Information System of the Valencian Government
 *
 * Copyright (C) 2007-2008 Infrastructures and Transports Department
 * of the Valencian Government (CIT)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 */

/*
 * AUTHORS (In addition to CIT):
 * 2009 {Iver T.I.}   {Task}
 */

package org.gvsig.fmap.dal.store.gpe;

import java.util.ArrayList;

import org.gvsig.fmap.dal.DALLibrary;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.library.DALFileLibrary;
import org.gvsig.fmap.dal.library.DALFileLocator;
import org.gvsig.fmap.dal.library.FileHelper;
import org.gvsig.fmap.dal.spi.DataManagerProviderServices;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.metadata.exceptions.MetadataException;
import org.gvsig.tools.library.AbstractLibrary;
import org.gvsig.tools.library.Library;
import org.gvsig.tools.library.LibraryException;
import org.gvsig.xmlpull.lib.impl.DefaultXmlPullLibrary;
import org.gvsig.xmlpull.prov.kxml.KxmlLibrary;

/**
 * @author <a href="mailto:jpiera@gvsig.org">Jorge Piera</a>
 */
public class DALGPELibrary extends AbstractLibrary {

	public DALGPELibrary() {
		super(DALGPELibrary.class, Library.TYPE.IMPL);
		require(DALLibrary.class);
		require(DALFileLibrary.class);
		require(DefaultXmlPullLibrary.class);
		require(KxmlLibrary.class);
	}
	
	protected void doInitialize() throws LibraryException {
		// Do nothing
		new DefaultXmlPullLibrary().initialize();
	}

	protected void doPostInitialize() throws LibraryException {
		
		new KxmlLibrary().postInitialize();
		ArrayList exs = new ArrayList();

		GPEStoreParameters.registerParametersDefinition();
		
		try {
			FileHelper.registerMetadataDefinition(
					GPEStoreProvider.METADATA_DEFINITION_NAME,
					GPEStoreProvider.class, "GPEMetadata.xml");
		} catch (MetadataException e) {
			exs.add(e);
		}


		DataManagerProviderServices dataman = (DataManagerProviderServices) DALLocator
		.getDataManager();

		if (!dataman.getStoreProviders().contains(GPEStoreProvider.NAME)) {
			dataman.registerStoreProvider(GPEStoreProvider.NAME,
					GPEStoreProvider.class, GPEStoreParameters.class);
		}
		
		if(DALFileLocator.getFilesystemServerExplorerManager()!=null){
		DALFileLocator.getFilesystemServerExplorerManager().registerProvider(
				GPEStoreProvider.NAME, GPEStoreProvider.DESCRIPTION,
				GPEFileSystemServerProvider.class);
		}
		// Validate there is any implementation registered.
//		XMLSchemaManager xmlSchemaManager = XMLSchemaLocator.getXMLSchemaManager();
//		if (xmlSchemaManager == null) {
//			exs.add(new ReferenceNotRegisteredException(XMLSchemaLocator.XMLSCHEMA_MANAGER_NAME, XMLSchemaLocator.getInstance()));
//		}
//		
//		//Register the application types in gpe. GPE uses the DAL types like internal types
//		xmlSchemaManager.registerAppDataType(DataTypes.UNKNOWN, DataTypes.UNKNOWN);
//		xmlSchemaManager.registerAppDataType(DataTypes.BOOLEAN, DataTypes.BOOLEAN);
//		xmlSchemaManager.registerAppDataType(DataTypes.BYTE, DataTypes.BYTE);
//		xmlSchemaManager.registerAppDataType(DataTypes.CHAR, DataTypes.CHAR);
//		xmlSchemaManager.registerAppDataType(DataTypes.INT, DataTypes.INT);
//		xmlSchemaManager.registerAppDataType(DataTypes.LONG, DataTypes.LONG);
//		xmlSchemaManager.registerAppDataType(DataTypes.FLOAT, DataTypes.FLOAT);
//		xmlSchemaManager.registerAppDataType(DataTypes.DOUBLE, DataTypes.DOUBLE);
//		xmlSchemaManager.registerAppDataType(DataTypes.STRING, DataTypes.STRING);
//		xmlSchemaManager.registerAppDataType(DataTypes.DATE, DataTypes.DATE);
//		xmlSchemaManager.registerAppDataType(DataTypes.TIME, DataTypes.TIME);
//		xmlSchemaManager.registerAppDataType(DataTypes.TIMESTAMP, DataTypes.TIMESTAMP);
//		//xmlSchemaManager.registerAppDataType(DataTypes.GEOMETRY, DataTypes.GEOMETRY);
//		xmlSchemaManager.registerAppDataType(DataTypes.OBJECT, DataTypes.OBJECT);
//		xmlSchemaManager.registerAppDataType(DataTypes.DYNOBJECT, DataTypes.DYNOBJECT);
//		//xmlSchemaManager.registerAppDataType(DataTypes.CRS, DataTypes.CRS);		
//		xmlSchemaManager.registerAppDataType(DataTypes.FILE, DataTypes.FILE);
//		xmlSchemaManager.registerAppDataType(DataTypes.BYTEARRAY, DataTypes.BYTEARRAY);
//		
//		//Mapping between GPE types and the application types
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.GEOMETRY, XMLSchemaManager.GEOMETRY);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.POINT, XMLSchemaManager.POINT);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.CURVE, XMLSchemaManager.LINESTRING);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.SURFACE, XMLSchemaManager.POLYGON);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.SOLID, XMLSchemaManager.SOLID);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.AGGREGATE, XMLSchemaManager.MULTIGEOMETRY);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.MULTIPOINT, XMLSchemaManager.MULTIPOINT);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.MULTICURVE, XMLSchemaManager.MULTILINESTRING);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.MULTISURFACE, XMLSchemaManager.MULTIPOLYGON);
//		xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.MULTISOLID, XMLSchemaManager.MULTISOLID);
	      
        //Operations
		GeometryManager geometryManager = GeometryLocator.getGeometryManager();        
//        geometryManager.registerGeometryOperation(WriteGml2Operation.NAME, new PointWriteGml2Operation(), Geometry.TYPES.POINT);
//        geometryManager.registerGeometryOperation(WriteGml2Operation.NAME, new CurveWriteGml2Operation(), Geometry.TYPES.CURVE);
//        geometryManager.registerGeometryOperation(WriteGml2Operation.NAME, new SurfaceWriteGml2Operation(), Geometry.TYPES.SURFACE);
//        geometryManager.registerGeometryOperation(WriteGml2Operation.NAME, new MultiPointWriteGml2Operation(), Geometry.TYPES.MULTIPOINT);
//        geometryManager.registerGeometryOperation(WriteGml2Operation.NAME, new MultiCurveWriteGml2Operation(), Geometry.TYPES.MULTICURVE);
//        geometryManager.registerGeometryOperation(WriteGml2Operation.NAME, new MultiSurfaceWriteGml2Operation(), Geometry.TYPES.MULTISURFACE);
        
		if (exs.size() > 0) {
			throw new LibraryException(this.getClass(), exs);
		}
	}
//	protected void doInitialize() throws LibraryException {
//		
//		// from default gpe library:
//        GPELocator.registerGPEManager(DefaultGPEManager.class);
//        GPEProviderLocator.registerGPEProviderManager(DefaultGPEProviderManager.class); 
//		// new DefaultXMLSchemaLibrary().initialize();
//		new DefaultXmlPullLibrary().initialize();
//		new KxmlLibrary().initialize();
//	}
//
//	protected void doPostInitialize() throws LibraryException {
//		
//		// from default gpe library
//		GPEManager gpeManager = GPELocator.getGPEManager();		
//		gpeManager.setProperty(GPEProperties.DECIMAL_DIGITS, new Integer(20));
//		// new DefaultXMLSchemaLibrary().postInitialize();
//		new DefaultXmlPullLibrary().postInitialize();
//		new KxmlLibrary().postInitialize();
//		 // ==============================================================
//		
//		GPEStoreProvider.registerDynClass();
//		
//		 if(ToolsLocator.getPersistenceManager().getDefinition(GPENewStoreParameters.DYNCLASS_NAME)==null )
//			 GPEStoreParameters.registerPersistent();
//		 
//		 if(ToolsLocator.getPersistenceManager().getDefinition(GPENewStoreParameters.DYNCLASS_NAME)==null )
//			 GPENewStoreParameters.registerPersistent();
//
//		DataManagerProviderServices dataman = (DataManagerProviderServices) DALLocator
//		.getDataManager();
//
//		if (!dataman.getStoreProviders().contains(GPEStoreProvider.NAME)) {
//			dataman.registerStoreProvider(GPEStoreProvider.NAME,
//					GPEStoreProvider.class, GPEStoreParameters.class);
//		}
//
//		DALFileLocator.getFilesystemServerExplorerManager().registerProvider(
//				GPEStoreProvider.NAME, GPEStoreProvider.DESCRIPTION,
//				GPEFileSystemServerProvider.class);
//		
//		// Validate there is any implementation registered.
//		XMLSchemaManager xmlSchemaManager = XMLSchemaLocator.getXMLSchemaManager();
//		if (xmlSchemaManager != null) {
//
//			//Register the application types in gpe.
//			xmlSchemaManager.registerAppDataType(DataTypes.UNKNOWN, DataTypes.UNKNOWN);
//			xmlSchemaManager.registerAppDataType(DataTypes.BOOLEAN, DataTypes.BOOLEAN);
//			xmlSchemaManager.registerAppDataType(DataTypes.BYTE, DataTypes.BYTE);
//			xmlSchemaManager.registerAppDataType(DataTypes.CHAR, DataTypes.CHAR);
//			xmlSchemaManager.registerAppDataType(DataTypes.INT, DataTypes.INT);
//			xmlSchemaManager.registerAppDataType(DataTypes.LONG, DataTypes.LONG);
//			xmlSchemaManager.registerAppDataType(DataTypes.FLOAT, DataTypes.FLOAT);
//			xmlSchemaManager.registerAppDataType(DataTypes.DOUBLE, DataTypes.DOUBLE);
//			xmlSchemaManager.registerAppDataType(DataTypes.STRING, DataTypes.STRING);
//			xmlSchemaManager.registerAppDataType(DataTypes.DATE, DataTypes.DATE);
//			xmlSchemaManager.registerAppDataType(DataTypes.TIME, DataTypes.TIME);
//			xmlSchemaManager.registerAppDataType(DataTypes.TIMESTAMP, DataTypes.TIMESTAMP);
//			xmlSchemaManager.registerAppDataType(DataTypes.OBJECT, DataTypes.OBJECT);
//			xmlSchemaManager.registerAppDataType(DataTypes.FILE, DataTypes.FILE);
//			xmlSchemaManager.registerAppDataType(DataTypes.BYTEARRAY, DataTypes.BYTEARRAY);
//			//geometries
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.GEOMETRY, XMLSchemaManager.GEOMETRY);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.POINT, XMLSchemaManager.POINT);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.CURVE, XMLSchemaManager.LINESTRING);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.SURFACE, XMLSchemaManager.POLYGON);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.SOLID, XMLSchemaManager.SOLID);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.AGGREGATE, XMLSchemaManager.MULTIGEOMETRY);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.MULTIPOINT, XMLSchemaManager.MULTIPOINT);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.MULTICURVE, XMLSchemaManager.MULTILINESTRING);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.MULTISURFACE, XMLSchemaManager.MULTIPOLYGON);
//			xmlSchemaManager.registerAppGeometryType(Geometry.TYPES.MULTISOLID, XMLSchemaManager.MULTISOLID);
//		}
//	}
	
	
}


