package org.gvsig.mobile.animation;

import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

public class Animator {
	public static final int PROPERTY_OPACITY = 0;
	public static final int PROPERTY_X = 1;
	public static final int PROPERTY_Y = 2;
	public static final int PROPERTY_WIDTH = 3;
	public static final int PROPERTY_HEIGHT = 4;
	
	private static final int milisecondsStep  = 100;
	ArrayList subscribers;
	Timer timr;
	static Animator instance;
	
	public static Animator getInstance(){
		if (instance==null)
			instance=new Animator();
		return instance;
	} 
	
	/**
	 * Forces the complete cancellation of the refresh process. It is called when 
	 * the application terminates and this functionality is no longer needed.
	 *
	 */
	public void terminate(){
		if(timr != null)
			timr.cancel();		
	}
	
	/**
	 * Constructor. It initializes the components needed to perform a refresh
	 * of the tool group bar.
	 *
	 */
	public Animator(){
		 subscribers = new ArrayList();
		 timr = new Timer();
		 Temporizer executor = new Temporizer(this);
		 timr.scheduleAtFixedRate(executor,milisecondsStep,milisecondsStep);
		 
	}
	
	/**
	 * This method adds a new animable object to the list of objects that will receive
	 * an animation notification.
	 * 
	 * @param pobj the object that has to be animated
	 * @param pquantity offset in pixels between one position and the following.
	 * @param pmiliseconds duration of the animation in miliseconds. Each step
	 * of the animation will substract a certain amount of miliseconds. When
	 * it reaches zero, the naimation will end.
	 */
	public void subscribe(Animable pobj, int property, int pquantity, int pmiliseconds){		
		this.subscribe(pobj, property, pquantity, pmiliseconds, false);
	}
	
	/**
	 * This method adds a new animable object to the list of objects that will receive
	 * an animation notification.
	 * 
	 * @param pobj the object that has to be animated
	 * @param pquantity offset in pixels between one position and the following.
	 * @param pmiliseconds duration of the animation in miliseconds. Each step
	 * of the animation will substract a certain amount of miliseconds. When
	 * it reaches zero, the naimation will end.
	 * @param removeAll If true remove all the previous subscribers
	 */
	public void subscribe(Animable pobj, int property, int pquantity, int pmiliseconds, boolean removeAll){		
		Subscriptor subs = new Subscriptor(pobj, property, pquantity, pmiliseconds);
		//subs.doStep();
		if(subscribers.size() > 0 && removeAll) {
			subscribers.removeAll(subscribers);		
		}
		subscribers.add(subs);
	}
	
	/**
	 * This method notifies a new step in the animation to all the animable
	 * objects subscribed to it.
	 *
	 */
	public void doStep(){
		int i = subscribers.size() - 1;
		while (i >= 0){
			Subscriptor subs = (Subscriptor)subscribers.get(i);
			subs.doStep();
			if (subs.miliseconds <=0 || subs.quantity <=0) {
				subscribers.remove(i);
				//Automatic desubscription
			}
			i--;
		}
	}
	
	
	/**
	 * 
	 * This is a autility class used to perform the animation.
	 * 
	 * 
	 * @author jcarras
	 *
	 */
    public class Temporizer extends TimerTask{
    	Animator father;
        public void run(){
        	//logger.debug("thread: " + this.hashCode() + " milis: " + System.currentTimeMillis());
        	father.doStep();
        }
        public Temporizer(Animator father){
        	super();
        	this.father=father;
        }
    }
    
    /**
     * 
     * This is a autility class used to perform the animation.
     * 
     * @author jcarras
     *
     */
	public class Subscriptor{
		Animable subscriber;
		int quantity;
		int miliseconds;
		int nullsteps = 1;
		double inc;
		int property;
		boolean negative = false;
		
		public Subscriptor(Animable obj, int property, int quan, int mili){
			this.property=property;
			subscriber = obj;
			quantity = quan;
			if (quan<0){
				negative=true;
				quantity=-quan;
			}
			miliseconds = mili;
			double steps = ((double)miliseconds) / (double)milisecondsStep;
			inc = ((double)quantity / steps);
		}
		
		public void doStep(){
			// logger.debug("miliseconds: " + miliseconds);
			// logger.debug("quantity: " + quantity);			
			miliseconds -= milisecondsStep;
			
			int thisStepInt = (int)(inc * nullsteps + 0.5);
			if (thisStepInt == 0){
				nullsteps++;
			} else {
				nullsteps=1;
				if (negative){
					subscriber.set(property,Math.min(thisStepInt,-quantity) );
				} else {
					subscriber.set(property,Math.min(thisStepInt,quantity) );
				}
				quantity -= thisStepInt;
				double steps = ((double)miliseconds) / (double)milisecondsStep;
				inc = ((double)quantity / steps);
			}			
			// logger.debug("thisStepInt: " + thisStepInt);
		}		
	}
	
}
