package org.gvsig.mobile.app;

import java.util.HashSet;
import java.util.Hashtable;

import org.gvsig.mobile.core.auxfunctions.OSGiMethods;
import org.gvsig.mobile.core.auxfunctions.ShowMessage;
import org.gvsig.mobile.core.auxfunctions.StringConstructor;
import org.gvsig.mobile.core.concepts.GvSigMouseEvent;
import org.gvsig.mobile.core.constants.GvSigConstants;
import org.gvsig.mobile.core.exceptions.GvSigServiceNotFoundException;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.core.interfaces.IMouseEventProvider;
import org.gvsig.mobile.core.interfaces.MapEventsListener;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.wireadmin.Producer;
import org.osgi.service.wireadmin.Wire;
import org.osgi.service.wireadmin.WireAdmin;
import org.osgi.service.wireadmin.WireConstants;
import org.zaval.lw.event.LwMouseEvent;


public abstract class MouseEventProvider implements IMouseEventProvider, Producer, MapEventsListener {
	private BundleContext bc;
	private String PID;
	private Wire[] myWires;
	private Object objectChanged;
	public ServiceRegistration reg;
	public GvSigMouseEvent myEvent;
	public MouseEventProvider(BundleContext context, String pid) {
		setBundleContext(context);
		this.setPID(pid);
		openConnection();

		
	}
	
//	@SuppressWarnings("unchecked")
	public void openConnection(){
		Hashtable props = new Hashtable();
		Class[] flavors=new Class[]{IGvSigMouseEvent.class};
		props.put(WireConstants.WIREADMIN_PRODUCER_FLAVORS, flavors);
		props.put(GvSigConstants.SERVICE_PID, getPID());
		
		reg=bc.registerService(Producer.class.getName(), this,props);
		
	}
	
	public void closeConection(){ 
		WireAdmin wa = null;
		try {
			wa = OSGiMethods.getWireAdmin(bc);
		} catch (GvSigServiceNotFoundException e) {
			ShowMessage.Show(e.toString());
		}
		if (myWires!=null){
			if(wa!=null){
				for (int i = 0; i < myWires.length; i++) {
					wa.deleteWire(myWires[i]);
				}
			}
		}
		reg.unregister();
		bc = null;
	}
	
	
	
	public BundleContext getBundlecontext() {
		return this.bc;
	}
	
	public void setBundleContext(BundleContext context) {
		this.bc=context;
		
	}
	
	public String getPID() {
		return this.PID;
	}
	
	public void setPID(String pid) {
		this.PID=pid;
	}
	
	public Wire[] getWires() {
		return this.myWires;
	}
	
	public boolean subscribed(Wire myWire){
		HashSet subscriptions=(HashSet)myWire.getProperties().get(GvSigConstants.MOUSE_SUBSCRIPTIONS);
		if (subscriptions.contains(new Integer(this.myEvent.getID()))){
			return true;
		}
		
		return false;
	}
	
	
	public void consumersConnected(Wire[] wires) {
		if (wires != null) {
		      for (int i = 0; i < wires.length; i++) {
		    	  if(subscribed(wires[i])){
		    		  wires[i].update(polled(wires[i]));
		    	  }
		      }
		 }
	}
	
	public Object polled(Wire wire) {
		return this.myEvent;	
			
	}
	
	public String getFilter(){
		
		return StringConstructor.Construct(new String[]{"(",WireConstants.WIREADMIN_PRODUCER_PID,"=",this.getPID(),")"});
	}
	
	public GvSigMouseEvent getLastMouseEvent(){
		return myEvent;
	}
	public void newMouseEvent(LwMouseEvent e) {
		WireAdmin wa = null;
		try {
			wa = OSGiMethods.getWireAdmin(bc);
		} catch (GvSigServiceNotFoundException e1) {
			ShowMessage.Show(e1.toString());
		}
		if(wa!=null){
		if (e instanceof LwMouseEvent) {
			LwMouseEvent m = (LwMouseEvent) e;
			switch(m.getID()){
			case LwMouseEvent.MOUSE_MOVED:
				//ShowMessage.Show("MOUSE MOVED");
				break;
			default:
				this.myEvent=new GvSigMouseEvent(m.getID(),m.getAbsX(),m.getAbsY(),m.getClickCount(),m.getMask(),m.getX(),m.getY());				
				try {
					this.consumersConnected(wa.getWires(getFilter()));
				} catch (InvalidSyntaxException e1) {
					e1.printStackTrace();
				}
				break;
			}
			
			
		}
		}
		
	}

}
