package org.gvsig.mobile.core.auxfunctions;

import java.awt.Dimension;
import java.awt.geom.Rectangle2D;

public class GeoUtils {
	
    /**
    *
    * Computes the minimum rectangle with the width/height ratio that contains
    * the provided extent.
    *
    * @param fext
    *            the extent to be covered
    * @param imgdim
    *            the size of the screen
    * @return the minimum rectangle with the width/height ratio that contains
    *         the provided extent.
    */
   public static Rectangle2D getExtentFromFullExtentAndImageSize(Rectangle2D fext,
       Dimension imgdim) {
       double ext_ratio = fext.getWidth() / fext.getHeight();
       double img_ratio = (1.0 * imgdim.width) / (1.0 * imgdim.height);

       if (ext_ratio == img_ratio) {
           // same ratio
           return fext;
       }
       else {
           if (ext_ratio > img_ratio) {
               // cinemascope
               double newexth = fext.getWidth() / img_ratio;
               double yoff = 0.5 * (newexth - fext.getHeight());

               return new Rectangle2D.Double(fext.getMinX(),
                   fext.getMinY() - yoff, fext.getWidth(), newexth);
           }
           else {
               // canadian flag
               double newextw = fext.getHeight() * img_ratio;
               double xoff = 0.5 * (newextw - fext.getWidth());

               return new Rectangle2D.Double(fext.getMinX() - xoff,
                   fext.getMinY(), newextw, fext.getHeight());
           }
       }
   }

}
