package org.gvsig.mobile.core.concepts;


import java.util.HashSet;
import java.util.Iterator;

import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;

/**
 * It does the work of adding and removing observers
 * and broadcasting it's initialization and termination
 * 
 * @author jcarras
 *
 */
public abstract class DefaultTask implements ITask {
	
	HashSet observers = new HashSet();
	boolean active;
	
	public TaskStatusListener addObserver(TaskStatusListener observer){
		observers.add(observer);
		return observer;
	}
	public TaskStatusListener removeObserver(TaskStatusListener observer){
		observers.remove(observer);
		return observer;
	}
	
	public void removeAllObservers(){
		observers.clear();		
	}
	
	public void pause(int milisecs) {
		// nothing, subclasses may override
	}

	public void run() {
		active=true;
		Iterator it = observers.iterator();
		while (it.hasNext()){
			TaskStatusListener obs = (TaskStatusListener)it.next();
//			if(obs != null)
				obs.taskInited(this);
		}		
	}

	public void stop() {
		active=false;
		Iterator it = observers.iterator();
		while (it.hasNext()){
			TaskStatusListener obs = (TaskStatusListener)it.next();
//			if(obs != null)
				obs.taskFinished(this);
		}
	}
	public boolean isActive() {
		return active;
	}
	
	public void notifyProgress(int p) {
		
		Iterator it = observers.iterator();
		while (it.hasNext()){
			TaskStatusListener obs = (TaskStatusListener)it.next();
				obs.taskProgress(this, p);
		}		

		
	}

}
