package org.gvsig.mobile.core.concepts;

import java.util.HashSet;
import java.util.Hashtable;

import org.gvsig.mobile.core.auxfunctions.FunctionalityResolutor;
import org.gvsig.mobile.core.auxfunctions.OSGiMethods;
import org.gvsig.mobile.core.auxfunctions.ShowMessage;
import org.gvsig.mobile.core.auxfunctions.StringConstructor;
import org.gvsig.mobile.core.constants.GvSigConstants;
import org.gvsig.mobile.core.exceptions.GvSigServiceNotFoundException;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.wireadmin.Consumer;
import org.osgi.service.wireadmin.Wire;
import org.osgi.service.wireadmin.WireAdmin;
import org.osgi.service.wireadmin.WireConstants;

public abstract class Functionality extends DefaultTask implements
		IFunctionality, Consumer {

	public BundleContext bc = null;

	public ServiceReference gvSigMobileRef;

	public Hashtable props = new Hashtable();

	private ServiceRegistration reg;	

	public HashSet events;

	private MapControl mapControl=null;

	public Functionality() {
		this.setListenToMouseEvents(false);
		configure();		
	}
	
	public Functionality(String ID, String name, String version,
			String provider, String extensionID, String interfaceID,
			String interfaceName, String interfaceVersion, String description,
			String comments, BundleContext bc) {
		this(ID, bc);
		this.setName(name);
		this.setVersion(version);
		this.setProvider(provider);
		this.setExtensionID(extensionID);
		this.setInterfaceID(interfaceID);
		this.setInterfaceName(interfaceName);
		this.setInterfaceVersion(interfaceVersion);
		this.setDescription(description);
		this.setComments(comments);
		this.setListenToMouseEvents(false);
		this.configure();
	}

	public Functionality(String ID, BundleContext bc) {
		this.setBundleContext(bc);
		this.setID(ID);
		this.setListenToMouseEvents(false);
		this.configure();		
	}

	public void setID(String ID) {
		this.props.put(GvSigConstants.FUNCTIONALITY_ID, ID);
	}

	public void setName(String name) {
		this.props.put(GvSigConstants.FUNCTIONALITY_NAME, name);
	}

	public void setVersion(String version) {
		this.props.put(GvSigConstants.FUNCTIONALITY_VERSION, version);
	}

	public void setProvider(String provider) {
		this.props.put(GvSigConstants.FUNCTIONALITY_PROVIDER, provider);
	}

	public void setExtensionID(String extensionID) {
		this.props.put(GvSigConstants.EXTENSION_ID, extensionID);
	}

	public void setInterfaceID(String interfaceID) {
		this.props.put(GvSigConstants.FUNCTIONALITY_INTERFACE_ID, interfaceID);
	}

	public void setInterfaceName(String interfaceName) {
		this.props.put(GvSigConstants.FUNCTIONALITY_INTERFACE_NAME,
				interfaceName);
	}

	public void setInterfaceVersion(String interfaceVersion) {
		this.props.put(GvSigConstants.FUNCTIONALITY_INTERFACE_VERSION,
				interfaceVersion);
	}

	public void setDescription(String description) {
		this.props.put(GvSigConstants.FUNCTIONALITY_DESCRIPTION, description);
	}

	public void setComments(String comments) {
		this.props.put(GvSigConstants.FUNCTIONALITY_COMMENTS, comments);
	}

	public void setBundleContext(BundleContext bc) {
		this.bc = bc;
	}
	
	public MapControl getMapControl(){
		
		if (this.mapControl == null) {
			final BundleContext bc = getBundleContext();
			IGvSigContext gv = (IGvSigContext)bc.getService(gvSigMobileRef);
			this.mapControl = (MapControl)gv.getMapControl();
		}
		return mapControl;
	}

	public String getID() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_ID).toString();
	}

	public String getName() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_NAME).toString();
	}

	public String getVersion() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_VERSION).toString();
	}

	public String getProvider() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_PROVIDER).toString();
	}

	public String getExtensionID() {
		return this.props.get(GvSigConstants.EXTENSION_ID).toString();
	}

	public String getFunctionalityID() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_ID).toString();
	}

	public String getFunctionalityName() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_NAME).toString();
	}

	public String getFunctionalityVersion() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_VERSION).toString();
	}

	public String getDescription() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_DESCRIPTION)
				.toString();
	}

	public String getComments() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_COMMENTS).toString();
	}

	public Hashtable getProps() {
		return this.props;
	}

	public String getInterfaceID() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_INTERFACE_ID)
				.toString();
	}

	public String getInterfaceName() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_INTERFACE_NAME)
				.toString();
	}

	public String getInterfaceVersion() {
		return this.props.get(GvSigConstants.FUNCTIONALITY_INTERFACE_VERSION)
				.toString();
	}

	public BundleContext getBundleContext() {
		return this.bc;
	}

	public boolean getListenToMouseEvents() {
		return Boolean
				.valueOf(this.props.get(
						GvSigConstants.FUNCTIONALITY_LISTEN_TO_MOUSE_EVENTS)
						.toString()).booleanValue();
	}

	private void setListenToMouseEvents(boolean value) {
		this.props.put(GvSigConstants.FUNCTIONALITY_LISTEN_TO_MOUSE_EVENTS,
				String.valueOf(value));
	}

	public void addMouseEventSubscription(int gvSigMouseEventType) {
		if (gvSigMouseEventType > 0) {
			if (this.events == null) {
				this.events = new HashSet();
			}
			this.events.add(new Integer(gvSigMouseEventType));
			this.props.put(
					GvSigConstants.FUNCTIONALITY_MOUSE_EVENTS_SUBSCRIBED,
					this.events);
			this.setListenToMouseEvents(true);
		}
	}

	public HashSet getMouseEventsSubscriptions() {
		return (HashSet) this.props
				.get(GvSigConstants.FUNCTIONALITY_MOUSE_EVENTS_SUBSCRIBED);
	}

	public void producersConnected(Wire[] wires) {
		if (wires != null) {			
			ShowMessage.Show(StringConstructor.Construct(new String[] {
					"Connected to ", String.valueOf(wires.length), " wires" }),
					true);
		} else {
			ShowMessage.Show("Not connected to any wire", true);
		}
	}

	public boolean execute(ServiceReference gvSIGMobileRef) {	
		this.gvSigMobileRef = gvSIGMobileRef;
		return execution(gvSIGMobileRef);
	}

	public void updated(Wire wire, Object value) {
		GvSigMouseEvent e = (GvSigMouseEvent) value;
		onNewMouseEventListened(e);
	}
	

	public ServiceRegistration getServiceRegistration() {
		return this.reg;
	}

	public void setServiceRegistration(ServiceRegistration sr) {
		this.reg = sr;
	}

	public void registerAsEventConsumer(Object[] classes) {

		Hashtable props = new Hashtable();
		props.put(WireConstants.WIREADMIN_CONSUMER_FLAVORS, classes);
		props.put(GvSigConstants.SERVICE_PID, getID());
		setServiceRegistration(bc.registerService(Consumer.class.getName(),
				this, props));
	}	

	public void run() {
		super.run();
		runFunctionality();

	}

	public boolean runFunctionality() /* throws GvSigException */{
		/*
		 * IFunctionalityResolutor funcRes =
		 * OSGiMethods.getFunctionalityResolutor(bc); if ( funcRes == null ) {
		 * ShowMessage
		 * .Show("(gvSIGMobile) Functionality Resolutor bundle not found!",
		 * true); //throw new
		 * GvSigException("Functionality Resolutor Not Found"); return false; }
		 */

		createWires();
		try {
			return FunctionalityResolutor.execute(this, this.bc);
		} catch (GvSigServiceNotFoundException e) {
			ShowMessage.Show(e.toString());
			return false;
		}

	}

	private void createWires() {
		WireAdmin wa = null;
		try {
			wa = OSGiMethods.getWireAdmin(bc);
		} catch (GvSigServiceNotFoundException e) {
			e.toString();
		}
		if (wa != null) {
			// TODO: AFEGIR LA RESTA DE EVENTS QUE ESCOLTAR
			if (getListenToMouseEvents()) {
				Hashtable props = new Hashtable();
				props.put(GvSigConstants.MOUSE_SUBSCRIPTIONS,
						getMouseEventsSubscriptions());
				wa.createWire(GvSigConstants.GvSIGMobile_MOUSE_EVENT_PROVIDER,
						getID(), props);
			}
		}
	}

	public void stop() {
		super.stop();

		if (getListenToMouseEvents()) {
			WireAdmin wa = null;
			try {
				wa = OSGiMethods.getWireAdmin(bc);
			} catch (GvSigServiceNotFoundException e1) {
				ShowMessage.Show(e1.toString());
			}
			if (wa != null) {
				try {
					Wire[] myWires = wa.getWires(StringConstructor
							.Construct(new String[] { "(",
									WireConstants.WIREADMIN_CONSUMER_PID, "=",
									getID(), ")" }));
					if (myWires != null) {
						for (int i = 0; i < myWires.length; i++) {
							wa.deleteWire(myWires[i]);
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	
	public boolean isApplicable(){
		return true;
	}
}
