package org.gvsig.mobile.core.concepts;

import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.gvsig.mobile.core.extensions.interfaces.IToolEnvelope;

/**
 * An Envelope to send service events and instances of services (ITool) that have changed
 * over wires. A Producer must instantiate a ToolEnvelope and a Consumer
 * should take an action depending on the attributes of the ToolEnvelope
 * @author aromeu
 *
 */
public class ToolEnvelope implements IToolEnvelope {

	private Object ID;
	private String scope;
	private ITool value;
	private int serviceEvent;
	
	/**
	 * The constructor
	 * @param ID
	 * @param scope
	 * @param value The service (ITool) changed
	 * @param serviceEvent The event (ServiceEvent.REGISTERED, ServiceEvent.UNREGISTERING, ServiceEvent.MODIFIED)
	 */
	public ToolEnvelope(Object ID, String scope, ITool value, int serviceEvent) {
		this.ID = ID;
		this.scope = scope;
		this.value = value;
		this.serviceEvent = serviceEvent;
	}
	
	public Object getIdentification() {
		return ID;
	}

	public String getScope() {
		return scope;
	}

	public Object getValue() {
		return value;
	}	
	
	public ITool getTool() {
		return value;
	}
	
	public int getServiceEvent() {
		return serviceEvent;
	}

}
