package org.gvsig.mobile.core.extensions.interfaces;

import java.util.ArrayList;
import java.util.Hashtable;

import org.osgi.framework.BundleContext;
/**
 * This Interface provides the methods that must implement every Extension of GvSIG Mobile
 * @author ProS
 */
public interface IExtension {
	
	/**
	 * Called by the constructor to initialize collections containing
	 * tools, functionalities and so.
	 */
	public void initializeCollections();
	
	/**
	 * 
	 * @return The ID of the extension (This parameter is mandatory)
	 */
	public String getID();
	
	/**
	 * 
	 * @return The Name of the extension (if it has been attached)
	 */	
	public String getName();
	
	/**
	 * 
	 * @return The Version of the extension (if it has been attached)
	 */	
	public String getVersion();
	
	/**
	 * 
	 * @return The Provider of the extension (if it has been attached)
	 */	
	public String getProvider();
	
	/**
	 * 
	 * @return The Date of Publication of the extension (if it has been attached)
	 */	
	public String getPublicationDate();
	
	/**
	 * 
	 * @return The Description of the extension (if it has been attached)
	 */	
	public String getDescription();
	
	/**
	 * 
	 * @return The Comments associated to the extension (if it has been attached)
	 */	
	public String getComments();

	
	/**
	 * 
	 * @return The BundleContext of the extension (if it has been attached)
	 */	
	public BundleContext getBundleContext();

	/**
	 * 	Set the ID of the extension (this parameter is mandatory)
	 * @param ID ID of the extension
	 * 
	 */
	public void setID(String ID);
	
	/**
	 * Set the NAME of the extension
	 * 
	 * @param name Name of the extension
	 */
	public void setName(String name);
	
	/**
	 * Set the VERSION of the extension
	 * @param version Version of the extension
	 */
	public void setVersion(String version);
	
	/**
	 * Set the PROVIDER of the extension 
	 * @param provider Provider of the extension
	 */
	public void setProvider(String provider);
	
	/**
	 * Set the DATE OF PUBLICATION of the extension
	 * @param publicationDate Date of publication of the extension
	 */
	public void setPublicationDate(String publicationDate);
	
	/**
	 * Set the DESCRIPTION of the extension
	 * @param description Description of the extension
	 */
	public void setDescription(String description);
	
	/**
	 * Set the COMMENTS of the extension
	 * @param comments of the extension
	 */
	public void setComments(String comments);
	
	/**
	 * Set the BundleContext to the extension
	 * @param bc System BundleContext
	 */
	public void setBundleContext(BundleContext bc);
	
	/**
	 * Returns The Extension OSGi Properties HashTable &ltKEY:String,VALUE:String&gt
	 * @return The OSGi Properties of the extension 
	 */
	public Hashtable getProps();
	
	/**
	 * 
	 * @param tool Add a tool to the Tool Extension Collection
	 * @return true if correct, false if there was an error
	 */
	public boolean addTool(ITool tool);
	
	/**
	 * 
	 * @param tool Remove a tool to the Tool Extension Collection
	 * @return true if correct, false if there was an error
	 */
	public boolean removeTool(ITool tool);
	
	/**
	 * 
	 * @param functionality Add a functionality to the functionality Extension Collection
	 * @return true if correct, false if there was an error
	 */
	public boolean addFunctionality(IFunctionality functionality);
	
	/**
	 * 
	 * @param functionality Remove a functionality to the functionality Extension Collection
	 * @return true if correct, false if there was an error
	 */
	public boolean removeFunctionality(IFunctionality functionality);
	
	/**
	 * 
	 * @return The Tool Collection of the extension
	 */
	public ArrayList getTools();
	
	/**
	 * 
	 * @return The Functionality Collection of the extension
	 */
	public ArrayList getFunctionalities();
	
	/**
	 * Register in OSGi all the plugins that has been attached to the extension
	 * @return true if correct, false if there was an error
	 */
	public boolean registerAll();

}
