/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.extension.base.forms.view;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;

import org.gvsig.compat.CompatLocator;
import org.gvsig.fmap.dal.DataTypes;
import org.gvsig.mobile.extension.base.forms.model.FeatureTemp;
import org.gvsig.mobile.extension.base.layer.raster.Utils;
import org.gvsig.mobile.extension.base.util.DataUtils;
import org.gvsig.mobile.extension.base.util.LwKeyEventFwTextField;
import org.gvsig.mobile.extension.base.util.TaskPerformer;
import org.gvsig.mobile.ui.awt.IWindowInfo;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.gvsig.mobile.ui.lw.ISelfCloseListener;
import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwWindow;
import org.zaval.lw.event.LwKeyEvent;
import org.zaval.lw.event.LwKeyListener;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.event.LwMouseListener;


/**
 * This form is created automatically specifically for a feature
 * Using the information of the feature attribute types and the
 * thinlet template used for generic forms and custom forms
 *  
 * @author jcarras
 *
 */
public class GenericForm extends LwPanel implements IWindowInfo, FeatureForm,
LwMouseListener, LwKeyListener, ISelfCloseListener { 
// extends Dialog implements FeatureForm, WindowListener, ComponentListener{
	
	private static Logger logger = LoggerFactory.getLogger(GenericForm.class);
	private transient Image doublebuffer;
	
	public static String TEMPLATE_FILE_NAME = ""; // init in Activator
	
	private boolean opened = false;
	protected FeatureTemp originalFeature;
	// protected Frame parent_frame;
	
	protected ThinletForm frm = null; // new ThinletForm(this); 
	protected BundleContext bc = null;
	
	public static int FORM_HEADER_HEIGHT = 26;
	
	static {
		if (Utils.USING_PDA) {
			FORM_HEADER_HEIGHT = 26;
		} else {
			FORM_HEADER_HEIGHT = 40;
		}
	}
	
	public static int FORM_MARGIN_WIDTH = 4;
	
	protected TaskPerformer taskPerformer;
	
	protected LwKeyEventFwTextField auxTextField = null;
	

	private void setSize(int x, int y, int w, int h){
		
		logger.debug("Setting size: " + x + ", "+ y + ", "+ w + ", "+ h);
		// pack();
		this.setPSSize(w,h);
		this.setSize(w,h);
		
		//26 pixels is the size from the accept/cancel buttons 
		//frm.setBounds(x,y-26,w,h);
		Insets i = this.getInsets();
		frm.setBounds(
				i.left+10,i.top+10,w-i.left-i.right-FORM_MARGIN_WIDTH,h-i.top-i.bottom-FORM_HEADER_HEIGHT);
		
		//frm.setBounds(0,0,w,h);
		frm.doLayout();
	}
	
	public GenericForm(BundleContext _bc, Rectangle bounds) {
		frm = new ThinletForm();
		bc = _bc;
		setSize(bounds.x, bounds.y, bounds.width, bounds.height);
		this.setLwLayout(new LwBorderLayout());
		
		auxTextField = new LwKeyEventFwTextField();
		auxTextField.setSize(150, 30);
		auxTextField.setPSSize(150, 30);
		auxTextField.setText("150 x 30");
		auxTextField.addKeyListener(this);

		add(LwBorderLayout.CENTER, auxTextField);
		// LwToolkit.getEventManager().addKeyListener(this);
		// lwem.addKeyListener(this);
	}
	
	public void updateListeners() {
		LwPanel parent_ = (LwPanel) this.getLwParent();
		LwWindow gparent_ = (LwWindow) parent_.getLwParent();
		
		gparent_.canHaveFocus();
		
	}
	
	
	
	protected void addCustomFormLines(StringBuffer buff){
		if (originalFeature==null) return;
		buff.append("\n<panel weightx=\"1\" columns=\"2\" gap=\"2\">\n");
		Object[] vals = originalFeature.getValues();
		String[] names = originalFeature.getFieldNames();
		for (int i=0; i<names.length; i++){ 
			
			String geomfld = originalFeature.getGeometryFieldName();
			// skip geom field
			if (names[i].compareToIgnoreCase(geomfld) != 0) {
				//TODO: Adapt every field appearance to the value type 
				buff.append("<label text=\"" + names[i].toString() + "\"/>");
				buff.append("<textfield weightx=\"1\" property=\"featureAttribute=" + names[i].toString() + "\" />\n");
			}
		}
		buff.append("</panel>\n");
		//System.out.print(buff);
	}
	
	protected InputStream getMixedTemplate(){
		StringBuffer result = new StringBuffer();
		try{
			BufferedReader tin = new BufferedReader(
					new FileReader(TEMPLATE_FILE_NAME)
					);
			String line;
			while ( (line=tin.readLine()) != null ) {
				if (line.trim().equals("<label name=\"content\" text=\"Content\"/>")){
					addCustomFormLines(result);
				} else {
					result.append(line);
				}
            }
			
			
			
		} catch (FileNotFoundException e){
			e.printStackTrace();
		} catch (IOException e1){
			e1.printStackTrace();
		}
		
		//TODO: please improve me
		byte[] bArray = result.toString().getBytes();
		ByteArrayInputStream bais = new ByteArrayInputStream(bArray);
		
		return bais;
	}

	public void cancelEdition() {
		//Sets the original values in every field and sets fields to not enabled
		// TODO Auto-generated method stub
	}

	public void close() {
		//Terminates the form
		// WindowManager.getInstance().closeActivePanel();
		this.opened=false;
		this.frm.destroy();
	}



	/*
	public void hide() {
		//Hides the form to the user
		//setVisible(false);
		this.opened=false;
		super.hide();
	}
	*/

	public boolean isCanceled() {
		//True if user closed the form without having clicked save
		return frm.cancelled;
	}

	public boolean isModified() {
		//True if user has modified any value
		return frm.modified;
	}

	/**
	 * @param component
	 * @param value
	 * @param type
	 */
	protected void fillField(Object component, Object value, int type){
		Object valType = frm.getProperty(component, "valueType");
		Object defaultValue = frm.getProperty(component, "defaultValue");
		String fieldType = frm.getClass(component);
		////System.out.println(value.getClass().getName() + " " + type);
		
		if (value==null || value.toString()==null){
			if (!"checkbox".equals(fieldType))
				frm.setString(component, "text", "");
			return;
		}
		
		if ("checkbox".equals(fieldType)){
			Object chkValue = frm.getProperty(component, "chkValue");
			
			if (value!=null && chkValue!=null && chkValue.toString().trim().equals(
					DataUtils.fieldValueToString(value)
					)){
				frm.setBoolean(component,"selected",true);
				return;
			}
			frm.setBoolean(component,"selected",false);
			return;
		}
		
		if ("combobox".equals(fieldType)){
			Object[] options = frm.getItems(component);
			for (int i =0; i<options.length;i++){
				if (DataUtils.fieldValueToString(value).equals(frm.getString(options[i], "name").trim())){
					frm.setInteger(component,"selected",i);
					frm.setString(component, "text", frm.getString(options[i], "text"));
					return;
				}
			}
			frm.setInteger(component,"selected",-1);
			frm.setString(component, "text", "");
			return;
		}
		
		//If the field is telling us the format we rely on it
		Object valueType = frm.getProperty(component, "valueType");
		if ("INTEGER".equals(valueType)){
			try{
				// int val = (int)(Double.parseDouble(value.toString()));
				frm.setString(component, "text", DataUtils.fieldValueToString(value));
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
		if ("NUMERIC".equals(valueType)){
			try{
				// double val = Double.parseDouble(value.toString());
				frm.setString(component, "text", DataUtils.fieldValueToString(value));
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
		if ("DATE".equals(valueType)){
			try{
				// Date val = new Date(value.toString());
				frm.setString(component, "text", DataUtils.fieldValueToString(value));
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
//		if ("TIME".equals(valueType)){
//			try{
//				String date_time= "00/00/0000" + (String)frm.getString(component, "text");
//				Date value = new Date(Date.parse(date_time));
//				return ValueFactory.createValue(value);
//			} catch (Exception e){ e.printStackTrace(); }
//		}
		
		//If the field is not telling us the format,
		// we rely on the feature attribute type
		if (value instanceof Integer || value instanceof Long)
			try{
				// int val = (int)(Double.parseDouble(value.toString()));
				frm.setString(component, "text", DataUtils.fieldValueToString(value));
				return;
			} catch (Exception e){ e.printStackTrace(); }
			
		if (value instanceof Double || value instanceof Float ){
			try{
				double val = Double.parseDouble(DataUtils.fieldValueToString(value));
					DecimalFormat frmt = new DecimalFormat("#########.##");
					frm.setString(component, "text", frmt.format(val));			
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
		
		if (value instanceof Date){
			try{
				// Date val = new Date(value.toString());
				frm.setString(component, "text", DataUtils.fieldValueToString(value));
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
		
		if (value!=null)
			frm.setString(component, "text", value.toString().trim());
		else 
			frm.setString(component, "text", String.valueOf(value));
	}
	protected void fillFields(){
		if (originalFeature==null)return;
		//Do the binding
		HashMap comps = getBindedComponents();
		String[] names = originalFeature.getFieldNames();
		Object[] vals = originalFeature.getValues();
		int[] types = originalFeature.getFieldTypes();
		for (int i=0;i<names.length;i++){
			if (comps.containsKey(names[i])){
				Object comp = comps.get(names[i]);
				fillField(comp, vals[i], types[i]);
				frm.setBoolean(comp, "enabled", false);
			}
		}
	}
	
	private void debugThinlet(Object component, int nTabs){
		for (int i=0;i<frm.getCount(component);i++){
			for (int t=0;t<nTabs;t++)
				System.out.print("\t");
			Object c2 = frm.getItem(component, i);
			System.out.println(frm.getClass(c2));
			debugThinlet(c2, nTabs+1);
		}
	}
	
	public void paintOnTop(Graphics g) {
		
		super.paintOnTop(g);
		
		// on top of the top
		if (frm != null) {
			frm.paint(g);
		}
		
	}
	
	public void setFeature(FeatureTemp feature) {
		//Fills all the binded components with the values in the feature
		originalFeature = feature;

		try{
			
			frm.add(frm.parse(getMixedTemplate()));
			//debugThinlet(frm.getDesktop(),0);
			// add(frm); // , BorderLayout.CENTER);
			//pack();
			// frm.requestFocus();
			// addWindowListener(this);
			// parent_frame.addComponentListener(this);
			//System.out.println("ADDED. there are " + this.getComponentCount() + " components");
			Image img = ImageUtils.getResourceImage(bc.getBundle(), "ok_2_new"); 
			frm.setIcon(
					frm.find("btnsave"),
					"icon",
					img);
			img = ImageUtils.getResourceImage(bc.getBundle(), "cancel_2_new");
			frm.setIcon(
					frm.find("btncancel"),
					"icon",
					img);
			//pack();

		} catch (IOException e){
			e.printStackTrace();
		}
		fillFields();
	}
	
	
	private Object createValue(String val, int type){
		try{
			if (type == DataTypes.INT) {
				if (val==null) return new Integer(0);
				int value = 0;
				;
				try{
					value = Integer.parseInt(val);
				} catch (Exception e) {
					logger.debug("Ignored bad input string: " + val);
				}
				return new Integer(value); //  ValueFactory.createValue(value);
			}
			
			if (type == DataTypes.DOUBLE || type == DataTypes.FLOAT) {
				if (val==null) return new Double(0);
				double value = 0;
				try{
					value = Double.parseDouble(CompatLocator.getStringUtils().replaceAll(val, ",", "."));
				} catch (Exception e){ e.printStackTrace(); }
				return new Double(value); // ValueFactory.createValue(value);
			}
			
			if (type == DataTypes.LONG || type == DataTypes.TIMESTAMP) {
				
				if (val==null) return new Long(0);

				long value = 0;
				try{
					value = Long.parseLong(val);
				} catch (Exception e){ e.printStackTrace(); }
				return new Long(value); // ValueFactory.createValue(value);
			}
			
			if (type == DataTypes.DATE){
				Date value = new Date();
				
				if (val==null) return value;

				try{
					value = Utils.dateFromString(val);
				} catch (Exception e){ e.printStackTrace(); }
				return value; // ValueFactory.createValue(value);
			}
			
			if (type == DataTypes.BOOLEAN){
				Boolean value = new Boolean(false);
				
				if (val==null) return value;

				try{
					value = Utils.booleanFromString(val);
				} catch (Exception e){ e.printStackTrace(); }
				return value; // ValueFactory.createValue(value);
			}
			
			return val == null ? "" : val; // ValueFactory.createValue(val);
		} catch (Exception e){
			e.printStackTrace();
			return ""; // ValueFactory.createNullValue();
		}
	}
	
	public Object getValue(Object component, int type){
		
		String fieldType = frm.getClass(component);
		String value = (String)frm.getString(component, "text");
		
		if ("checkbox".equals(fieldType)){
			Object chkValue = frm.getProperty(component, "chkValue");
			Object nonChkValue = frm.getProperty(component, "nonChkValue");
			
			if (frm.getBoolean(component,"selected")){
				value = (String)chkValue;
			}else{
				value = (String)nonChkValue;

			}
		}
		
		if ("combobox".equals(fieldType)){
			//TODO: support text writting on combos
			Object option = frm.getSelectedItem(component);
			if (option==null){
				value = null;
			} else{
				value = (String)frm.getString(option, "name");
			}
		}
		
	
		return createValue(value, type);

	}
	
	public FeatureTemp getFeature() {
		//Returns the feature with the new values updated by the user
		FeatureTemp fea = (FeatureTemp)originalFeature.clone();
		HashMap comps = getBindedComponents();
		String[] names = fea.getFieldNames();
		int[] types = fea.getFieldTypes();
		for (int i=0;i<names.length;i++){
			if (comps.containsKey(names[i])){
				try{
					fea.getValues()[i]=getValue(comps.get(names[i]),types[i]);
				} catch (Exception e){e.printStackTrace();}
			}
		}
		return fea;
	}

	public void setVisible(boolean value){
		//super.setVisible(true);
		
		//System.out.println("SETTING VISIBLE");

		this.opened=value;

		super.setVisible(value);
	}

	public void startEdition() {
		//Puts the fields with modifiable=true to enabled
			if (originalFeature==null)return;
			frm.setBoolean(frm.find("btnsave"), "enabled", true);
			HashMap comps = getBindedComponents();
			String[] names = originalFeature.getFieldNames();
			for (int i=0;i<names.length;i++){
				if (comps.containsKey(names[i])){
					Object comp = comps.get(names[i]);
					String mod = (String)this.frm.getProperty(comp,"modifiable");
					if (mod==null || "TRUE".equalsIgnoreCase(mod)){
						frm.setBoolean(comp, "enabled", true);
					}
				}
			}
		
	}

	public boolean isValid() {
		//Checks the restrictions of fields and alerts user if there are incorrections
		// TODO Auto-generated method stub
		return true;
	}
	public boolean isOpen() {
		return opened;
	}	
	

	
	public HashMap getBindedComponents(){
		HashMap result = new HashMap();
		getBindedComponents(result, frm.getDesktop());
		return result;
	}
	
	private void getBindedComponents(HashMap result, Object component){
		Object[] comps = this.frm.getItems(component);
		for (int i=0;i<comps.length;i++){
			Object comp = comps[i];
			Object att = frm.getProperty(comp, "featureAttribute");
			if (att!=null){
				result.put(att,comp);
			}
			getBindedComponents(result,comp);
		}
	}
	
//	public void windowClosing(WindowEvent e) {
//		opened=false;
//		
//		if (frm.cancelled || !frm.modified) {
//			Object[] obj = new Object[1];
//			obj[0] = this;
//			taskPerformer.start(obj);
//		} else {
//			if (frm.modified) {
//				Object[] obj = new Object[2];
//				obj[0] = this;
//				obj[1] = new Boolean(true);
//				taskPerformer.start(obj);
//			}
//		}
//		
//		setVisible(false);
//		
//	}
//	public void windowOpened(WindowEvent e) {}
//	public void windowClosed(WindowEvent e) {}
//	public void windowIconified(WindowEvent e) {}
//	public void windowDeiconified(WindowEvent e) {}
//	public void windowActivated(WindowEvent e) {}
//	public void windowDeactivated(WindowEvent e) {}
	
	/**
	 * Call the paint method to redraw this component without painting a
	 * background rectangle
	 */
	public void update(Graphics g) {
		paint(g);
	}

	public void componentResized(ComponentEvent e) {

		try {
			int w = this.getWidth();
			int h = this.getHeight();
			Insets i = this.getInsets();
			frm.setBounds(
					i.left+10,i.top+10,w-i.left-i.right-FORM_MARGIN_WIDTH,h-i.top-i.bottom-FORM_HEADER_HEIGHT);
			frm.doLayout();
		} catch (Exception ex) {
			logger.error("While resizing comp: " + ex.getMessage());
		}

	}

	
	public Dimension getDesiredSize() {
		return null;
	}

	public boolean isMaximized() {
		return true;
	}

	public void mouseClicked(LwMouseEvent ev) {
		// frm.mouseClicked(ev);
	}
	public void mouseEntered(LwMouseEvent ev) { }
	public void mouseExited(LwMouseEvent ev) { }

	public void mousePressed(LwMouseEvent ev) {
		frm.mousePressed(ev);
		auxTextField.requestFocus();
		this.repaint();
	}

	public void mouseReleased(LwMouseEvent ev) {
		frm.mouseReleased(ev);
		this.repaint();
	}

	public void keyPressed(LwKeyEvent ev) {
		frm.keyPressed(ev);
		this.repaint();
	}

	public void keyReleased(LwKeyEvent ev) {
		frm.keyReleased(ev);
		this.repaint();
	}
	
	public void keyTyped(LwKeyEvent ev) {
		frm.keyTyped(ev);
		this.repaint();
	}

	public void setTaskPerformer(TaskPerformer performer) {
		taskPerformer = performer;
	}

	public void closeEventFollows() {
		
		if (isModified()) {
			Object[] params = new Object[2];
			params[0] = this;
			params[1] = new Boolean(true);
			taskPerformer.start(params);
		}
		
	}





}
