/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.base.info.dialog;

import java.util.ArrayList;

import org.gvsig.fmap.dal.DataQuery;
import org.gvsig.fmap.dal.DataSet;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureQuery;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.gvsig.tools.dataTypes.DataTypes;
import org.gvsig.tools.evaluator.gt2.GT2Evaluator;
import org.gvsig.tools.evaluator.gt2.filter.FilterFactory2Impl;
import org.opengis.filter.Filter;
import org.opengis.filter.expression.Expression;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;


public class SearchDialog extends OkCancelDialog {	

	public static final String OP_EQ_STRING = Messages.getText("SearchDialog.0"); //$NON-NLS-1$
	public static final String OP_GT_STRING = Messages.getText("SearchDialog.1"); //$NON-NLS-1$
	public static final String OP_LT_STRING = Messages.getText("SearchDialog.2"); //$NON-NLS-1$
	public static final String OP_GET_STRING = Messages.getText("SearchDialog.3"); //$NON-NLS-1$
	public static final String OP_LET_STRING = Messages.getText("SearchDialog.4"); //$NON-NLS-1$
	public static final String OP_NEQ_STRING = Messages.getText("SearchDialog.5"); //$NON-NLS-1$
	public static final String OP_CONTAINS_STRING = Messages.getText("SearchDialog.6"); //$NON-NLS-1$
	public static final String OP_BEGINS_WITH_STRING = Messages.getText("SearchDialog.7"); //$NON-NLS-1$
	public static final String OP_ENDS_WITH_STRING = Messages.getText("SearchDialog.8"); //$NON-NLS-1$
	
	
	
	private LwCombo lyrCombo;
	private LwCombo colCombo;
	private LwCombo opCombo;
	private LwTextField valueText;
	
//	private LayersTable toc;
	
	public LwPanel buildMainPanel() {
		// TODO Auto-generated method stub
//		return null;
		LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwBorderLayout());
		
		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(4,1));
		
		LwPanel center = new LwPanel();
		center.setLwLayout(new LwGridLayout(4, 1, LwToolkit.HORIZONTAL));
		
		LwConstraints l = DialogConstants.getTextFieldConstraints();		
		
		LwConstraints l1 = DialogConstants.getLabelConstraints();		
		
//		LwLabel titleLabel = new LwLabel(Messages.getText("SearchDialog.10")); //$NON-NLS-1$
		
		LwLabel lyrLabel = DialogConstants.getDefaultLabel(Messages.getText("SearchDialog.11"));		
		LwLabel attrLabel = DialogConstants.getDefaultLabel(Messages.getText("SearchDialog.12"));		
		LwLabel opLabel = DialogConstants.getDefaultLabel(Messages.getText("SearchDialog.13"));		
		LwLabel valLabel = DialogConstants.getDefaultLabel(Messages.getText("SearchDialog.14"));		

		lyrCombo = getLyrCombo();
		colCombo = reloadColCombo();
		opCombo= getOpCombo();
		valueText = DialogConstants.getDefaultTextField();
		
		left.add(l1, lyrLabel);
		center.add(l, lyrCombo);
		
		left.add(l1, attrLabel);
		center.add(l, colCombo);
		
		left.add(l1, opLabel);
		center.add(l, opCombo);
		
		left.add(l1, valLabel);
		center.add(l, valueText);
		
		resp.add(LwBorderLayout.WEST, left);
		resp.add(LwBorderLayout.CENTER, center);
		
		updateOkButton();
		
		return resp;
	}
	
	public void actionPerformed(Object src, Object data) {
		if (src == this.getCancelButton()) {
			WindowManager.getInstance().closeActivePanel();
		} else if (src == this.getOkButton()) {
//			toc.getMapControl().repaint();
			executeSearch();
			WindowManager.getInstance().closeActivePanel();
		} else if (src instanceof LwAList) {

			this.updateOkButton();
			if (src  == lyrCombo.getList())  {
				reloadColCombo();
			}
		}
		
		getOkButton().repaint();
	}
	
	/**
	 * Constructor. Needs the TOC to list all the available vector layers.
	 * 
	 * @param _toc the current toc, where the dialog will search for vector layers.
	 * @param p parent frame, needed for graphical purposes
	 */
	public SearchDialog(BundleContext bc) {
		super(false, bc, true);
	}

	private LwCombo getLyrCombo() {
		
		ArrayList lyrs = mapControl.getLayersForReading();
		LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
		
		// l.addItem(DialogConstants.getDefaultLabel(""));
		
		int sz = lyrs.size();
		for (int i=0; i<sz; i++) {
			FLayer lyr = (FLayer) lyrs.get(i); 
			if (lyr instanceof FLyrVect) {
				FLyrVect lyrv = (FLyrVect) lyr;
				l.addItem(lyrv.getName());
			}
		}
		l.addSelectionListener(this);
		resp.setList(l);
		return resp;
	}	

	private LwCombo getOpCombo() {
		LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
		
		// =============================================
		// do not change order... it's index-based !!
		l.addItem(DialogConstants.getDefaultLabel("=")); // OP_EQ_STRING));
		l.addItem(DialogConstants.getDefaultLabel(">")); // OP_GT_STRING));
		l.addItem(DialogConstants.getDefaultLabel("<")); // OP_LT_STRING));
		l.addItem(DialogConstants.getDefaultLabel(">=")); // OP_GET_STRING));
		l.addItem(DialogConstants.getDefaultLabel("<=")); // OP_LET_STRING));
		l.addItem(DialogConstants.getDefaultLabel("!=")); // OP_NEQ_STRING));
		l.addItem(DialogConstants.getDefaultLabel(OP_CONTAINS_STRING));
		l.addItem(DialogConstants.getDefaultLabel(OP_BEGINS_WITH_STRING));
		l.addItem(DialogConstants.getDefaultLabel(OP_ENDS_WITH_STRING));
		// =============================================
		l.addSelectionListener(this);
		resp.setList(l);
		return resp;
	}

	private LwCombo reloadColCombo() {
		
		if (colCombo == null) {
			colCombo = new LwCombo();
		} else {
			colCombo.getList().removeAll();
		}

		LwCompList l = new LwCompList();		
		
		FLyrVect lyrv = this.getSelectedLayer();
		if (lyrv == null) {
			return colCombo;
		}
		
		FeatureAttributeDescriptor[] atts = null;
		try {
			atts = lyrv.getFeatureStore().getDefaultFeatureType().getAttributeDescriptors();
		} catch (Exception e) {
			logger.error("While getting atts: " + e.getMessage());
			return colCombo;
		}
		
		int sz = atts.length;
		for (int i=0; i<sz; i++) {
			l.addItem(atts[i].getName());
		}
		l.addSelectionListener(this);
		colCombo.setList(l);
		return colCombo;
	}

//	private SelectableDataSource getSDS() throws FMapDriverException {
//		FLyrVect layer = getSelectedLayer();
//		ReadableVectorial rv = layer.getSource();
//		return rv.getRecordset();
//	}
//
	private void executeSearch() {
		
		boolean enab = getSelectedLayer() != null &&
	    getSelectedAttIndex() != -1 &&
		getSelectedOpIndex() != -1;
		
		if (!enab) {
			return;
		}

		try {
			// int att_ind =
			// getAttributeIndex(sds.getFieldNames(), colCombo.getList().getSelected().toString());
			
			String att_name = ((LwLabel) colCombo.getList().getSelected()).getText();
			FLyrVect lyrv = getSelectedLayer();
			FeatureAttributeDescriptor[] atts = null;
			FeatureType ftype = lyrv.getFeatureStore().getDefaultFeatureType(); 
			atts = ftype.getAttributeDescriptors();
			int sz = atts.length;
			int att_indx = -1;
			for (int i=0; i<sz; i++) {
				if (att_name.compareTo(atts[i].getName()) == 0) {
					att_indx = i;
					break;
				}
			}
			
			if (att_indx == -1) {
				throw new Exception("Att not found (?!) in selected layer. Layer, att = " + lyrv.getName() + ", " + att_name);
			}

			// boolean is_numeric = isNumericType(atts[att_indx].getType());
			
			FilterFactory2Impl filfact = new FilterFactory2Impl();
			Expression exp1 = null;
			Expression exp2 = null;
			String str_e2 = "";
			
			double literalv = 0;
			boolean numeric_comp = false;
			try {
				literalv = Double.parseDouble(valueText.getText());
				numeric_comp = true;
			} catch (Exception ex) { }
			
			String literalvstr = valueText.getText();
			
			if (numeric_comp) {
				exp2 = filfact.literal(literalv);
			} else {
				exp2 = filfact.literal(literalvstr);
			}
			
			exp1 = filfact.property(att_name);
			Filter filter = getFilterForSelectedOp(
					getSelectedOpIndex(), filfact,
					exp1, exp2,
					literalvstr);
			
			GT2Evaluator ev = new GT2Evaluator(filter);
			DataQuery dq = new DefaultFeatureQuery(ftype, ev); // null; // ew DataQ
			DataSet queryres = lyrv.getDataStore().getDataSet(dq);
			lyrv.getDataStore().setSelection(queryres);
			mapControl.setDirty(); // .paintLayers(); // .re.repaint(true);

		} catch (Exception ex) {
			logger.error("while executing search: " + ex.getMessage()); //$NON-NLS-1$
		}
	}
	

	private String putBrackets(String str) {
		
		String resp = "";
		int len = str.length();
		for (int i=0; i<len; i++) {
			resp = resp + "[" + str.substring(i, i+1) + "]";
		}
		return resp;
	}

	private Filter getFilterForSelectedOp(
			int ind,
			FilterFactory2Impl ffact,
			Expression e1,
			Expression e2,
			String e2_str) {
		switch (ind) {
		case 0: // ==
			return ffact.equals(e1, e2);
		case 1: // >
			return ffact.greater(e1, e2);
		case 2: // <
			return ffact.less(e1, e2);
		case 3: // >=
			return ffact.greaterOrEqual(e1, e2);
		case 4: // <=
			return ffact.lessOrEqual(e1, e2);
		case 5: // !=
			return ffact.notEqual(e1, e2);
		case 6: // contains
			return ffact.like(e1, "%" + e2_str + "%");
		case 7: // begins with
			return ffact.like(e1, e2_str + "%");
		case 8: // ends with
			return ffact.like(e1, "%" + e2_str);
		default:
			logger.error("Unexpected operation index: " + ind);
		 	return ffact.equals(e1, e2);
		}
	}

	private boolean isNumericType(int dt) {
	
		if ((dt == DataTypes.BYTE)
			|| (dt == DataTypes.DOUBLE)
			|| (dt == DataTypes.FLOAT)
			|| (dt == DataTypes.INT)
			|| (dt == DataTypes.LONG)
			) {
		
			return true;
		} else {
			return false;
		}
	}

//	private boolean contains(Value v_big, Value v_small) {
//		String v_big_lower = v_big.toString().toLowerCase();
//		String v_small_lower = v_small.toString().toLowerCase();
//		return v_big_lower.indexOf(v_small_lower) != -1;
//	}
//	
//	private boolean beginsWith(Value v_big, Value v_small) {
//		String v_big_lower = v_big.toString().toLowerCase();
//		String v_small_lower = v_small.toString().toLowerCase();
//		return v_big_lower.indexOf(v_small_lower) == 0;
//	}
//
//	private boolean endsWith(Value v_big, Value v_small) {
//		String str_big = v_big.toString().toLowerCase();
//		String str_sma = v_small.toString().toLowerCase();
//		int l = str_big.length() - str_sma.length();
//		if (l < 0) return false;
//		return str_big.indexOf(str_sma) == l;
//	}
	

	private int getAttributeIndex(String[] ff, String f) {
		for (int i=0; i<ff.length; i++) {
			if (ff[i].compareToIgnoreCase(f) == 0) return i; 
		}
		return -1;
	}

	private FLyrVect getSelectedLayer() {
		
		if (lyrCombo == null || lyrCombo.getList() == null || lyrCombo.getList().getSelected() == null) {
			return null;
		}
		
		String lyrname = ((LwLabel) lyrCombo.getList().getSelected()).getText();
		
		if (lyrname == null) return null;

		ArrayList lyrs = mapControl.getLayersForReading(); // FLayer[] ll = toc.getLayers();
		int sz = lyrs.size();
		for (int i=0; i<sz; i++) {
			FLayer lyr = (FLayer) lyrs.get(i); // ll[i];
			if (lyr instanceof FLyrVect) {
				if (lyr.getName().compareToIgnoreCase(lyrname) == 0) {
					return (FLyrVect) lyr;
				}
			}
		}
		return null;
	}
	
	private int getSelectedAttIndex() {
		if (colCombo == null || colCombo.getList() == null) {
			return -1;
		} else {
			return colCombo.getList().getSelectedIndex();
		}
	}

	
	private int getSelectedOpIndex() {
		if (opCombo == null || opCombo.getList() == null) {
			return -1;
		} else {
			return opCombo.getList().getSelectedIndex();
		}
	}

	public String getTitle() {
		return Messages.getText("SearchDialog.10");
	}	
	
	private void updateOkButton() {
		boolean enab = getSelectedLayer() != null &&
	    getSelectedAttIndex() != -1 &&
		getSelectedOpIndex() != -1; 
		this.getOkButton().setEnabled(enab);
	}



}
