package org.gvsig.mobile.extension.base.info.functionality;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.io.File;
import java.util.ArrayList;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.tools.dispose.DisposableIterator; 
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureReference;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.mapcontext.layers.FLyrDefault;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.app.MainFrame;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.extension.base.forms.model.FeatureTemp;
import org.gvsig.mobile.extension.base.forms.view.AlphanumericalData;
import org.gvsig.mobile.extension.base.forms.view.CustomForm;
import org.gvsig.mobile.extension.base.forms.view.FeatureForm;
import org.gvsig.mobile.extension.base.forms.view.GenericForm;
import org.gvsig.mobile.extension.base.util.DataUtils;
import org.gvsig.mobile.extension.base.util.TaskPerformer;
import org.gvsig.mobile.graphicLayer.IntersectsGeometryEvaluatorCdc;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.lw.MessageBox;
import org.gvsig.tools.evaluator.Evaluator;
import org.gvsig.tools.task.Cancellable;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class InfoFunc extends Functionality implements TaskPerformer {
	
	private static final Logger logger = LoggerFactory.getLogger(PointSelectionFunc.class);
	public static int CLICK_TOLERANCE = 10;
	
	private MapControl mapCtrl = null;
	private IGvSigContext gvc;
	private FLyrDefault layer;

	
	public InfoFunc() {
		super();
	}

	public void configure() {
		// TODO Auto-generated method stub
		
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {

		gvc = (IGvSigContext)bc.getService(gvSIGMobileRef);
		mapCtrl = (MapControl) gvc.getMapControl();
		
		ArrayList infolyrs = mapCtrl.getLayersForInfo();
		
		if (infolyrs == null || infolyrs.size() != 1) {
			logger.warn("There must be exactly one layer for selection.");
			stop();
			mapCtrl = null;
			layer = null;
			return false;
		} else {
			layer = (FLyrDefault) infolyrs.get(0);
			return true;
		}

		
	}

	/**
	 * Point event that indicates the screen coordinates where the user has clicked
	 * to query for info. This method will show a dialog with the information available
	 * from each layer.
	 * 
	 * @param event the point event that indicates the point of interest.
	 */
	public void onNewMouseEventListened(IGvSigMouseEvent event) {


		if (mapCtrl == null || layer == null) {
			logger.error("While using info tool: MAPCONTROL = " + mapCtrl);
			logger.error("(nothing done)         LAYER      = " + layer);
			return;
		}

		// -------------------------- starts working
		/*
		if (mapCtrl.isProcessing()) {
			return;
		} else {
			mapCtrl.setProcessing(true);
		}
		*/

		int x = event.getX();
		int y = event.getY();

		// Point2D wp = mapCtrl.getViewPort().toMapPoint(x, y);
		FeatureTemp fea =null;

		double tol = mapCtrl.getViewPort().toMapDistance(CLICK_TOLERANCE);
		
		
		try {
			fea = infoByPoint(
					layer,
					new Point(x,y),
					tol, null, mapCtrl.getMapContext().getProjection().getAbrev());
		} catch (Exception e) {
			logger.error("While getting info by point: " + e.getMessage());
			return;
		}
		

		if (fea==null) {
			logger.warn("Info returned null info (?) - functionality cancelled");
			return;
		}
		
		FeatureForm form;
		String pathForm = null;
		
		Boolean aux = new Boolean(false);
		if (layer.hasDynValue(CustomForm.PROPERTY_KEY_USES_CUSTOM_FORM)) {
			aux = (Boolean) layer.getDynValue(CustomForm.PROPERTY_KEY_USES_CUSTOM_FORM);
			if (aux.booleanValue()) {
				if (layer.hasDynValue(CustomForm.PROPERTY_KEY_CUSTOM_FORM_PATH)) {
					pathForm = (String) layer.getDynValue(CustomForm.PROPERTY_KEY_CUSTOM_FORM_PATH);
				}
			}
		}
		
		MainFrame mf = (MainFrame) gvc.getMainFrame();
		Rectangle mfbb = mf.getBounds();
		
		if (pathForm==null || pathForm.equals("")){
			form = new GenericForm(bc, mfbb);
		} else {
			File fForm = new File(pathForm);
			if (!fForm.exists()){
				form = new GenericForm(bc, mfbb);
			} else {
				try{
					form = new CustomForm(pathForm, bc, mfbb);
				} catch (Exception frmex){
					
					MessageBox.showDialog(true, bc,
							Messages.getText("custom_form_has_errors") + ": " + frmex.getMessage(),
							null, MessageBox.MESSAGE_TYPE_ERROR, MessageBox.MESSAGE_ONLY_OK);
					mapCtrl.repaint();
					return;
				}
			}
		}

		if (layer.isEditing()) {
			form.setTaskPerformer(this);
			AlphanumericalData.update(form, fea);
		} else {
			AlphanumericalData.visualize(form, fea);
		}
		
		form.updateListeners();

		mapCtrl.repaint();		
	}
	

	
	private EditableFeature getEditableFeature(FLyrDefault lyr, FeatureTemp ftmp) throws Exception {
		EditableFeature resp = null;
		FeatureReference fr = ftmp.getFeatureReference();
		if(layer instanceof FLyrVect){
			FLyrVect layer = (FLyrVect)lyr;
			Feature fe = layer.getFeatureStore().getFeatureByReference(fr);
			resp = fe.getEditable();
		
			String[] fldnames = ftmp.getFieldNames();
			Object[] fldvv = ftmp.getValues();
			int len = fldnames.length;
			String itemname = "";
			for (int i=0; i<len; i++) {
				itemname = fldnames[i];
				resp.set(itemname, fldvv[i]);
			}	
		}
		return resp;
	}

	public FeatureTemp infoByPoint(FLyrDefault layer, Point p, double tolerance, Cancellable cancel, String srs) throws Exception {
		
		Point2D mapp = mapCtrl.getViewPort().toMapPoint(p.x, p.y);
		Geometry surr = DataUtils.getSurroundingRect(mapp.getX(), mapp.getY(), tolerance);
		if(layer instanceof FLyrVect){
			FLyrVect lyr = (FLyrVect)layer;
			FeatureStore featureStore = lyr.getFeatureStore();
		
			FeatureType ft = null;
			try {
				ft = featureStore.getDefaultFeatureType();
			} catch (DataException e1) {
				logger.error("While getting featuretype to get geometry name: " + e1.getMessage());
				return null;
			}
		
			String geomname = ft.getDefaultGeometryAttributeName();
		
			FeatureQuery fq = featureStore.createFeatureQuery();

			Evaluator evaluator=new IntersectsGeometryEvaluatorCdc(
					surr, mapCtrl.getViewPort().getProjection(),
					ft, geomname);
		
			fq.setFilter(evaluator);
			FeatureSet set = null;
			try {
				set = featureStore.getFeatureSet(fq);
			} catch (DataException e) {
				logger.error("While setting selection: " + e.getMessage());
			}
		
			if (set.getSize() == 0) {
				return null;
			}
		
			DisposableIterator diter = set.iterator();
			Feature featu = (Feature) diter.next();
			diter.dispose();

			FeatureTemp ftmp = featureToFeatureTemp(featu, featureStore.getDefaultFeatureType());
			return ftmp;
		}
		return null;
	}

	private FeatureTemp featureToFeatureTemp(Feature ff, FeatureType fety) {
		
		FeatureTemp resp = new FeatureTemp(fety.getDefaultGeometryAttributeName());
		FeatureAttributeDescriptor[] attdescs = fety.getAttributeDescriptors();
		int len = attdescs.length;
		
		int[] att_types = new int[len];
		Object[] att_vals = new Object[len];
		String[] att_names = new String[len];
		
		for (int i=0; i<len; i++) {
			att_names[i] = attdescs[i].getName();
			att_types[i] = attdescs[i].getType();
			att_vals[i] = ff.get(i);
		}
		
		resp.setFieldNames(att_names);
		resp.setFieldTypes(att_types);
		resp.setValues(att_vals);
		resp.setFeatureReference(ff.getReference());
		return resp;
	}

	/**
	 * Edition task
	 */
	public boolean start(Object[] objs) {

		if (objs == null) {
			logger.error("Unexpected values: NULL");
			return false;
		}
		
		if (objs.length < 1) {
			logger.error("Unexpected empty array.");
			return false;
		}

		if (!(objs[0] instanceof GenericForm)) {
			logger.error("Unexpected object: " + objs[0].getClass().getName());
			return false;
		}
		
		GenericForm gf = (GenericForm) objs[0]; 
		
		if (gf.isCanceled()) {
			gf.close();
			return false;
		} else {
			
			if (objs.length < 2) {
				logger.error("Unexpected array with 1 element.");
				gf.close();
				return false;
			}

			if (!(objs[1] instanceof Boolean)) {
				logger.error("Unexpected object: " + objs[1].getClass().getName());
				gf.close();
				return false;
			}

			FeatureTemp fea = gf.getFeature();
			if(layer instanceof FLyrVect){
				FLyrVect lyr = (FLyrVect)layer;
				try{
					EditableFeature efe = getEditableFeature(layer, fea);
					FeatureStore featureStore = lyr.getFeatureStore();
					lyr.update(null, efe);	
				} catch(Exception e) {
					gf.close();
					MessageBox.showDialog(true, bc,
						Messages.getText("error_while_updating") + ": " + e.getMessage(),
						null, MessageBox.MESSAGE_TYPE_ERROR, MessageBox.MESSAGE_ONLY_OK);
					mapCtrl.repaint();
					return false;
				}
				gf.close();
				return true;
			}
		}
		return false;
	}

}
