package org.gvsig.mobile.extension.base.info.functionality;

import java.awt.geom.Point2D;
import java.util.ArrayList;

import org.cresques.cts.IProjection;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.Geometry.SUBTYPES;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.mobile.extension.base.layer.raster.Utils;
import org.gvsig.mobile.libProjection.cartoutil.GeoUtils;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarDescription;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarLabel;

public class MeasureAreaFunc extends MeasureDistanceFunc {

	
	protected static final String BAR_ID = "MEASURE_AREA_BAR_ID";

	
	public MeasureAreaFunc() {
		super();
	}


	
	protected Geometry getGeometry(ArrayList vv) {
		
		int sz = vv.size();
		Geometry resp = null;
		if (sz < 2) {
			try {
				resp = GeometryLocator.getGeometryManager().createNullGeometry(SUBTYPES.GEOM2D);
			} catch (Exception e) {
				logger.error("While creating null geom: " + e.getMessage());
			}
			return resp;
		}
		
		Point2D fromp = (Point2D) vv.get(0);
		Point2D top = null;
		
		GeneralPathX gpx = new GeneralPathX();
		gpx.moveTo(fromp.getX(), fromp.getY());
		for (int i=1; i<sz; i++) {
			top = (Point2D) vv.get(i);
			gpx.lineTo(top.getX(), top.getY());
		}
		gpx.lineTo(fromp.getX(), fromp.getY());
		
		try {
			resp = GeometryLocator.getGeometryManager().createSurface(gpx, SUBTYPES.GEOM2D);
		} catch (Exception e) {
			logger.error("While creating null geom: " + e.getMessage());
		}
		return resp;
	}
	
	protected void setStatusBar() {
		
		double area = getArea(verticesCoords, mc.getMapContext().getProjection());
		String unit = " m2";
		int leaveround = 4;
		int rounded = -((int) Math.round(Math.log(area) / Math.log(10)) - leaveround);
		if (area >= 5000000) {
			
			if (area >= 5000000000000d) {
				area = area / 1000000000000d;
				unit = " Mill. Km2";
				rounded = -((int) Math.round(Math.log(area) / Math.log(10)) - leaveround);
			} else {
				area = area / 1000000;
				unit = " Km2";
				rounded = -((int) Math.round(Math.log(area) / Math.log(10)) - leaveround);
			}
		}
		
		String[] str = new String[1];
		str[0] = Utils.getFormattedDouble(area, rounded, false) + unit;
		
		if (totLabel == null) {
			totLabel = new StatusBarLabel(str[0]);
		} else {
			totLabel.setText(str[0]);
		}
		
		barDescription = new StatusBarDescription(StatusBarDescription.PREFERENCE_HIGH, BAR_ID);
		barDescription.add(new Integer(100), totLabel);
	}



	private double getArea(ArrayList vv, IProjection p) {

		int sz = vv.size();
		GeneralPathX gpx = new GeneralPathX();
		Point2D po = (Point2D) vv.get(0);
		gpx.moveTo(po.getX(), po.getY());
		for (int i=1; i<sz; i++) {
			po = (Point2D) vv.get(i);
			gpx.lineTo(po.getX(), po.getY());
		}
		po = (Point2D) vv.get(0);
		gpx.lineTo(po.getX(), po.getY());
		double resp = GeoUtils.getAreaInSquareMeters(gpx.getPathIterator(null), p.isProjected());
		return resp;
	}
	

}
