package org.gvsig.mobile.extension.base.info.functionality;

import java.awt.geom.Point2D;

import org.cresques.cts.IProjection;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.extension.base.layer.raster.Utils;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarDescription;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarLabel;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarManager;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MeasureCoordinateFunc extends Functionality {
	
	private static final Logger logger = LoggerFactory.getLogger(MeasureCoordinateFunc.class);
	protected static final String BAR_ID = "MEASURE_COORD_BAR_ID";

	private IGvSigContext gvc;
	private MapControl mc;
	
	private StatusBarDescription barDescription;
	private StatusBarLabel xLabel;
	private StatusBarLabel yLabel;
	
	public MeasureCoordinateFunc() {
		super();
	}

	public void configure() {
		// TODO Auto-generated method stub
		
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {

		try {
			gvc = (IGvSigContext)bc.getService(gvSIGMobileRef);
			mc = (MapControl) gvc.getMapControl();
			
			return true;
		} catch (Exception e) {
			logger.error("While executing MeasureCoordinateFunc: " + e.getMessage());
			return false;
		}
	}
	
	public void stop() {
		super.stop();

		if (gvc != null) {
			if (! ((StatusBarManager) gvc.getStatusBarManager()).removeStatusBar(barDescription)) {
				logger.error("Could not remove status bar MeasureCoordinateFunc (not found?)");
			}
		} else {
			logger.error("Could not remove status bar MeasureCoordinateFunc: IGvSigContext is NULL");
		}
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		
		Point2D mp = mc.getViewPort().toMapPoint(event.getX(), event.getY());
		
		String[] str = getCoordinatesString(
				mp.getX(), mp.getY(),
				mc.getViewPort().getProjection());
		
		if (xLabel == null) {
			xLabel = new StatusBarLabel(str[0]);
		} else {
			xLabel.setText(str[0]);
		}
		
		if (yLabel == null) {
			yLabel = new StatusBarLabel(str[1]);
		} else {
			yLabel.setText(str[1]);
		}

		if (barDescription == null) {
			barDescription = new StatusBarDescription(StatusBarDescription.PREFERENCE_HIGH, BAR_ID);
			barDescription.add(new Integer(50), xLabel);
			barDescription.add(new Integer(50), yLabel);
		}
		
		StatusBarManager sbm = (StatusBarManager) gvc.getStatusBarManager(); 
		sbm.addStatusBar(barDescription);
		// sbm.getStatusBar().setVisible(true);
		// ((StatusBarManager) gvc.getStatusBarManager()).refresh();
	}
	
	private String[] getCoordinatesString(double x, double y, IProjection p) {
		
		String units = "m";
		int decimals = 2;
		if (!p.isProjected()) {
			units = "";
			decimals = 7;
		}

		String[] resp = new String[2];
		resp[0] = Utils.getFormattedDouble(x, decimals, false) + " " + units;
		resp[1] = Utils.getFormattedDouble(y, decimals, false) + " " + units;
		return resp;
	}

}
