package org.gvsig.mobile.extension.base.info.functionality;

import java.awt.geom.Point2D;
import java.util.ArrayList;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.primitive.Point;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.graphicLayer.IntersectsGeometryEvaluatorCdc;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.tools.evaluator.Evaluator;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class PointSelectionFunc extends Functionality {
	
	private static final Logger logger = LoggerFactory.getLogger(PointSelectionFunc.class);
	
	private IGvSigContext gvc;
	private MapControl mc;
	private FLyrVect layer;

	
	public PointSelectionFunc() {
		super();
	}

	public void configure() {
		// TODO Auto-generated method stub
		
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {

			gvc = (IGvSigContext)bc.getService(gvSIGMobileRef);
			mc = (MapControl) gvc.getMapControl();
			
			ArrayList infolyrs = mc.getLayersForInfo();
			
			if (infolyrs == null || infolyrs.size() != 1) {
				logger.warn("There must be exactly one layer for selection.");
				stop();
				return false;
			} else {
				return true;
			}
	}
	
	public void onNewMouseEventListened(IGvSigMouseEvent event) {

		ArrayList infolyrs = mc.getLayersForInfo();
		
		if (infolyrs == null || infolyrs.size() != 1) {
			logger.error("There must be exactly one layer for selection.");
			return;
		}
		
		layer = (FLyrVect) infolyrs.get(0);

		Point2D mapp = mc.getViewPort().toMapPoint(event.getX(), event.getY());
		Point p = new org.gvsig.fmap.geom.primitive.impl.Point2D(mapp);
		FeatureStore featureStore = layer.getFeatureStore();
		
		FeatureType ft = null;
		try {
			ft = featureStore.getDefaultFeatureType();
		} catch (DataException e1) {
			logger.error("While getting featuretype to get  geometry name: " + e1.getMessage());
			return;
		}
		
		String geomname = ft.getDefaultGeometryAttributeName();
		
		FeatureQuery fq = featureStore.createFeatureQuery();

		Evaluator evaluator=new IntersectsGeometryEvaluatorCdc(
				p, mc.getViewPort().getProjection(),
				ft, geomname);
		
		fq.setFilter(evaluator);
		FeatureSet set;
		try {
			set = featureStore.getFeatureSet(fq);
			featureStore.getFeatureSelection().deselectAll();
			featureStore.getFeatureSelection().select(set);		
		} catch (DataException e) {
			logger.error("While setting selection: " + e.getMessage());
		}
		mc.setDirty();
	}

}
