package org.gvsig.mobile.extension.base.info.functionality;

import java.util.ArrayList;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureSelection;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.tools.dispose.DisposableIterator;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ZoomSelectionFunc extends Functionality {
	
	private static final Logger logger = LoggerFactory.getLogger(PointSelectionFunc.class);
	
	private IGvSigContext gvc;
	private MapControl mc;
	private FLyrVect layer;

	public ZoomSelectionFunc() {
		super();
	}

	public void configure() {
		// TODO Auto-generated method stub
		
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {
		gvc = (IGvSigContext)bc.getService(gvSIGMobileRef);
		mc = (MapControl) gvc.getMapControl();
		
		ArrayList infolyrs = mc.getLayersForInfo();
		
		if (infolyrs == null || infolyrs.size() != 1) {
			logger.warn("There must be exactly one layer for selection.");
			stop();
			return false;
		} else {
			layer = (FLyrVect) infolyrs.get(0);
			try {
				FeatureStore fs = layer.getFeatureStore();
				DefaultFeatureSelection dfs = (DefaultFeatureSelection) fs.getSelection();
				DisposableIterator iter = fs.getFeatureSet().fastIterator();
				Envelope ext = null;
				Envelope inc = null;
				Feature feat = null;
				while (iter.hasNext()) {
					feat = (Feature) iter.next();
					if (!dfs.isSelected(feat)) {
						continue;
					}
					
					if (ext == null) {
						ext = feat.getDefaultEnvelope();
						if (ext == null) {
							ext = feat.getDefaultGeometry().getEnvelope();
						}
					} else {
						inc = feat.getDefaultEnvelope();
						if (inc == null) {
							inc = feat.getDefaultGeometry().getEnvelope();
						}
						if (inc != null) {
							ext.add(inc);
						}
					}
				}
				iter.dispose();
				
				if (ext != null) {
					mc.getViewPort().setEnvelope(ext);
				}
				
			} catch (DataException e) {
				logger.error("Unable to clear selection: " + e.getMessage());
				stop();
				return false;
			}
			stop();
			mc.setDirty();
			return true;
		}
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		// TODO Auto-generated method stub
		
	}

}
