/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.base.layer.dialog;

import java.awt.Color;
import java.awt.Point;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.gvsig.metadata.exceptions.MetadataException;
import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.fmap.mapcontext.layers.CancelationException;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontext.layers.LayerCollectionEvent;
import org.gvsig.fmap.mapcontext.layers.LayerCollectionListener;
import org.gvsig.fmap.mapcontext.layers.LayerPositionEvent;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.auxfunctions.FunctionalityResolutor;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.exceptions.GvSigServiceNotFoundException;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.extension.base.navigation.functionality.ZoomNextEnvelopeHistory;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.MessageBox;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.gvsig.mobile.ui.lw.UIConstants;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.data.Item;
import org.zaval.data.MatrixModel;
import org.zaval.data.Tree;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCanvas;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwCompRender;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwImgRender;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.LwSwitchManImpl;
import org.zaval.lw.LwTextRender;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.LwView;
import org.zaval.lw.LwViewProvider;
import org.zaval.lw.event.LwActionListener;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.grid.LwTreeGrid;
import org.zaval.lw.misc.LwCustomTreeView;
import org.zaval.lw.tree.LwTree;

public class LayerManagerDialog extends OkCancelDialog implements
		TaskStatusListener, LayerCollectionListener {

	// private LwButton addWFSButton;
	private LwButton addFileLayerButton;
	private LwButton addWMSButton;
	private LwButton deleteButton;
	private LwButton upButton;
	private LwButton downButton;
	private LwButton zoomButton;
	private LwButton propButton;
	private LwButton symButton;

	public ServiceReference gvSigMobileRef;
	private LwPanel toc;
	private LwTreeGrid treeGrid;
	int selected = -1;
	private ArrayList layers = null;
	private String previousTocDesc;

	private static final int VISIBLE = 0;
	private static final int INFOABLE = 1;
	private static final int EDITABLE = 2;

	private int rowHeight = 0;
	
	private static Logger logger = LoggerFactory.getLogger(LayerManagerDialog.class);

	public LayerManagerDialog(BundleContext bc, boolean scrollable) {
		super(scrollable, bc);
		this.getCancelButton().setVisible(false);
		
		mapControl.getMapContext().getLayers().addLayerCollectionListener(this);
	}

	/**
	 * Handle button actions
	 */
	public void actionPerformed(Object src, Object data) {
		if (false) { // src == addWFSButton) {

		} else if (src == addFileLayerButton) {
			execute("es.prodevelop.gvsig.extension.base.layer.AddFileLayerFunc.1.0.0.functionality");
		} else if (src == addWMSButton) {			
			execute("es.prodevelop.gvsig.extension.wms.layer.AddWMSFunc.1.0.0.functionality");			
		} else if (src == deleteButton) {
			deleteLayer();
		} else if (src == upButton) {
			if (upSelectedLayer()) {
				// mapControl.setDirty();
			}
		} else if (src == downButton) {
			if (downSelectedLayer()) {
				// mapControl.setDirty();
			}
		} else if (src == zoomButton) {
			FLayer layer = this.getSelectedLayer();
			if (layer != null) {
				try {
					ZoomNextEnvelopeHistory.clearNextEnvelopes();
					this.mapControl.getViewPort().setEnvelope(
							layer.getFullEnvelope());
				} catch (ReadException e) {
					// TODO Auto-generated catch block
					logger.error(e.getMessage());
				}
			}
			close();

		} else if (src == propButton) {
			FLayer sellyr = getSelectedLayer();
			if (sellyr == null) {
				this.showSelectLayerWarning();
				return;
			}
			LayerPropertiesDialog l = new LayerPropertiesDialog(false, this
					.getBundleContext(), sellyr, this);
			l.showDialog();
		} else if (src == symButton) {
			FLayer sellyr = getSelectedLayer();
			if (sellyr == null) {
				this.showSelectLayerWarning();
				return;
			}
			if (sellyr instanceof FLyrVect) {
				FLyrVect vect = (FLyrVect) sellyr;
				LayerSymbologyDialog l = new LayerSymbologyDialog(false, this
						.getBundleContext(), vect);
				l.showDialog();
			} else {
				MessageBox.showDialog(
						true, bc,
						Messages.getText("symbology_not_available_for_this_layer"),
						null, MessageBox.MESSAGE_TYPE_INFO, MessageBox.MESSAGE_ONLY_OK);
			}
		} else if (src == this.getOkButton()) {
			// try {
			// if (((LwCheckbox)toc.get(4)).getState()){
			// ((FLyrVect)mapControl.getLayers().get(0)).setEditing(true);
			// }else{
			// ((FLyrVect)mapControl.getLayers().get(0)).setEditing(false);
			// }
			// } catch (StartEditionLayerException e) {
			// e.printStackTrace();
			// }
			// previousTocDesc = getLayersDesc(layers);
			
			String curr_toc = getLayersDesc(getLayers(true));
			
			if (curr_toc.compareTo(previousTocDesc) != 0) {
				mapControl.setDirty();
			} else {
			}
			close();
		} else if (((Integer) data).intValue() == MessageBox.MESSAGE_OK) {
			// System.out.println("OK");
		} else if (((Integer) data).intValue() == MessageBox.MESSAGE_CANCEL) {
			// System.out.println("Cancel");
		}
	}

	private void updateToc() {
		getLayers(true);
		toc.remove(treeGrid);
		this.addTreeGrid();
	}

	private void showSelectLayerWarning() {
		MessageBox
				.showDialog(true, bc, Messages
						.getText("LayersManagerDialog.16"), this,
						MessageBox.MESSAGE_TYPE_WARNING,
						MessageBox.MESSAGE_ONLY_CANCEL);
	}

	/**
	 * 
	 * @return done or not
	 */
	private boolean upSelectedLayer() {

		FLayer l = this.getSelectedLayer();

		if (l == null)
			return false;

		int pos = selected - 1 - 1;

		if (pos < 0)
			return false;

		FLayers lyrs = mapControl.getMapContext().getLayers(); // .updateLayerPosition(pos, l);
		lyrs.moveTo(pos+1, pos);
		
		selected = pos + 1;
		return true;
	}

	/**
	 * 
	 * @return done or not
	 */
	private boolean downSelectedLayer() {
		FLayer l = this.getSelectedLayer();

		if (l == null)
			return false;

		int pos = selected - 1 + 1;

		if (pos > layers.size() - 1)
			return false;

		// mapControl.updateLayerPosition(pos, l);
		FLayers lyrs = mapControl.getMapContext().getLayers(); // .updateLayerPosition(pos, l);
		lyrs.moveTo(pos-1, pos);

		selected = pos + 1;
		return true;
	}

	private void deleteLayer() {

		if (selected > 0) {
			FLayers lyss = mapControl.getMapContext().getLayers(); 
			int cnt = lyss.getLayersCount();
			
			if (cnt == 0) {
				return;
			}
			// toc and layers have inverted order:
			lyss.removeLayer( (cnt-1) - (selected-1) );
		}
		updateToc();

	}

	private FLayer getSelectedLayer() {
		final ArrayList layers = getLayers(false);
		if (selected > 0) {
			return (FLayer) layers.get(selected - 1);
		}
		return null;
	}

	private void execute(String functionalityID) {
		try {
			IFunctionality func = FunctionalityResolutor.solve(functionalityID,
					"", "", "", "", "", "", getBundleContext());
			func.addObserver(this);
			((Functionality) func).runFunctionality();
			// FunctionalityResolutor.execute(func, getBundleContext());
		} catch (GvSigServiceNotFoundException e) {
			// TODO Auto-generated catch block
			logger.error(e.getMessage());
		}
	}

	public LwPanel buildMainPanel() {

		LwContainer toolbar1 = new LwPanel();
		toolbar1.setBackground(Color.white);
		toolbar1.setLocation(20, 50);
		LwFlowLayout toolbar1Layout = new LwFlowLayout(LwToolkit.LEFT,
				LwToolkit.TOP, LwToolkit.VERTICAL, 1);
		toolbar1.setLwLayout(toolbar1Layout);

		// Setting Menu Buttons
		// addVectButton = DialogConstants.getButton(getBundle(),
		// 		"capa_vectorial_mini", "capa_vectorial_mini", this);
		addFileLayerButton = DialogConstants.getButton(getBundle(),
				"capa_mini", "capa_mini", this);
		addWMSButton = DialogConstants.getButton(getBundle(),
				"capa_wms_mini", "capa_wms_mini", this);
		deleteButton = DialogConstants.getButton(getBundle(), "capa_borrar_mini",
				"capa_borrar_mini", this);
		upButton = DialogConstants.getButton(getBundle(), "flecha_arriba_mini",
				"flecha_arriba_mini", this);
		downButton = DialogConstants.getButton(getBundle(), "flecha_abajo_mini",
				"flecha_abajo_mini", this);
		zoomButton = DialogConstants.getButton(getBundle(),
				"capa_buscar_mini", "capa_buscar_mini", this);
		propButton = DialogConstants.getButton(getBundle(),
				"capa_opciones_mini", "capa_opciones_mini", this);
		symButton = DialogConstants.getButton(getBundle(), "colores_mini",
				"colores_mini", this);

		// toolbar1.add(this.addVectButton);
		toolbar1.add(this.addFileLayerButton);
		toolbar1.add(this.addWMSButton);
		toolbar1.add(this.deleteButton);
		toolbar1.add(this.upButton);
		toolbar1.add(this.downButton);
		toolbar1.add(this.zoomButton);
		toolbar1.add(this.propButton);
		toolbar1.add(this.symButton);

		LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwBorderLayout());

		// Defining Root Panel
		toc = new LwPanel();
		toc.setBackground(Color.white);
		toc.setLwLayout(new LwBorderLayout());
		addTreeGrid();
		
		previousTocDesc = getLayersDesc(getLayers(true));

		// LwGridCaption ver = new LwGridCaption(g, LwToolkit.VERTICAL);
		// g.add(LwGrid.LEFT_CAPTION_EL, ver);
		LwCanvas stub = new LwCanvas();
		stub.getViewMan(true).setBorder("br.raised");
		// g.add(LwGrid.STUB_EL, stub);
		// for (int i = 0; i < g.getModel().getRows(); i++)
		// ver.putTitle(i, "" + i);

		LwScrollPan sc1 = new LwScrollPan(toc);
		sc1.setIncrements(30, 60, 30, 60);
		sc1.setBackground(Color.white);

		resp.add(LwBorderLayout.CENTER, sc1);
		resp.add(LwBorderLayout.EAST, toolbar1);
		return resp;
	}

	private String getLayersDesc(ArrayList list) {
		
		PersistenceManager pm = ToolsLocator.getPersistenceManager(); 
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		for (int i=0; i<list.size(); i++) {
			PersistentState ps = null;
			try {
				Object o = null;
				FLayer lyr = (FLayer) list.get(i);
				if (lyr instanceof FLyrVect) {
					o = ((FLyrVect) lyr).getLegend();
					ps = pm.getState(o);
					baos.write("LEGEND:".getBytes());
					pm.saveState(ps, baos);
					baos.write("-LABELS:".getBytes());
					o = ((FLyrVect) lyr).getLabelingStrategy();
					
					if (o == null) {
						baos.write("NULL".getBytes());
					} else {
						ps = pm.getState(o);
						pm.saveState(ps, baos);
					}

				} else {
					String str = lyr.toString();
					baos.write(str.getBytes());
				}
				baos.write(("-VISIBLE:" + lyr.isVisible()).getBytes());
				baos.write(("-ACTIVE:" + lyr.isActive()).getBytes());
			} catch (Exception e) {
				e.printStackTrace();
				logger.error("While getting state to compare tocs: " + e.getMessage());
			}
		}
		try {
			baos.flush();
		} catch (IOException e) { }
		return baos.toString();
	}

	private ArrayList getLayers(boolean update) {
		if (update) {
			this.layers = inverted(mapControl.getLayersForReading());
			return layers;
		}

		if (layers == null) {
			layers = inverted(mapControl.getLayersForReading());
		}
		return layers;
	}

	private ArrayList inverted(ArrayList list) {
		ArrayList resp = new ArrayList();
		for (int i=(list.size()-1); i>=0; i--) {
			resp.add(list.get(i));
		}
		return resp;
	}

	/**
	 * 
	 * @return string description of toc
	 */
	private void addTreeGrid() {
		Tree model = createTreeTablePanel(getLayers(true));

		treeGrid = new LwTreeGrid(model) {
			
			public void mouseReleased(LwMouseEvent event) { }
			public void mouseClicked(LwMouseEvent event) { }
			
			public void mousePressed(LwMouseEvent event) {
				Point p = treeGrid.cellByLocation(event.getX(), event.getY());
				
				if (p == null) { return; }
				if (p.x == 0) { return; } // because x is vertical (?), this is click on top bar
				
				super.mousePressed(event);

				Object o = treeGrid.get(p.x, p.y);

				selected = p.x;
				if (o instanceof LwCompRender) {
					// I know there is a checkbox
					LwCheckbox c = (LwCheckbox) ((LwPanel) ((LwCompRender) o)
							.getTarget()).get(0);
					c.setState(!c.getState());
					switch (p.y) {
					case 1:
						updateLayer(c.getState(), VISIBLE);
						break;
					case 3:
						updateLayer(c.getState(), INFOABLE);
						break;
					case 4:
						updateLayer(c.getState(), EDITABLE);
						break;
					}
				}
				repaint();
			}

			public void actionPerformed(Object source, Object data) {
				super.actionPerformed(source, data);
			}

			public void removeRows(int row, int count) {
				this.rowHeights = new int[] { rowHeight, rowHeight };

				this.remove(row);

			}
		};

		// treeGrid.
		treeGrid.setBackground(Color.white);
		LwCustomTreeView view = new LwCustomTreeView();
		view.setDefFont(DialogConstants.getDefaultFont());
		// view.
		// view.
		treeGrid.setTreeViewProvider(view);

		treeGrid.setTreeElView(LwTree.OPENED_VIEW, null);
		treeGrid.setTreeElView(LwTree.CLOSED_VIEW, null);
		treeGrid.setTreeElView(LwTree.LEAST_VIEW, null);
		treeGrid.setTreeElView(LwTree.TOGGLE_ON_VIEW, null);
		treeGrid.setTreeElView(LwTree.TOGGLE_OFF_VIEW, null);
		
		
		fillMatrixModel(model);

		toc.setBackground(Color.white);

		toc.add(LwBorderLayout.CENTER, treeGrid);
		this.laidout();
	}

	public void fillMatrixModel(Tree model) {
		// Defining Images
		LwImgRender image1 = new LwImgRender(ImageUtils.getResourceImage(
				getBundle(), "toc_ojo_abierto"), LwView.ORIGINAL);
		LwImgRender image2 = new LwImgRender(ImageUtils.getResourceImage(
				getBundle(), "toc_dlg_heading_eye_scale"), LwView.ORIGINAL);
		LwImgRender image3 = new LwImgRender(ImageUtils.getResourceImage(
				getBundle(), "toc_info_icon"), LwView.ORIGINAL);
		LwImgRender image4 = new LwImgRender(ImageUtils.getResourceImage(
				getBundle(), "toc_lapiz"), LwView.ORIGINAL);

		MatrixModel matrixModel = treeGrid.getModel();

		LwSwitchManImpl tt = new LwSwitchManImpl();
		tt.addActionListener(treeGrid);

		// Defining images
		matrixModel.put(0, 1, image1);
		matrixModel.put(0, 2, image2);
		matrixModel.put(0, 3, image3);
		matrixModel.put(0, 4, image4);

		FLayer l;

		for (int i = 1; i < model.getItemsCount(); i++) {
			l = (FLayer) layers.get(i - 1);

			this.addCheckToGrid(l.isVisible(), matrixModel, i, 1);
			
			Boolean infoed = new Boolean(false);
			
			if (l.hasDynValue(MapControl.INFO_ACTIVATED_DYN_PROPERTY_NAME)) {
				infoed = (Boolean) l.getDynValue(MapControl.INFO_ACTIVATED_DYN_PROPERTY_NAME);
			}
			
			this.addCheckToGrid(infoed.booleanValue(), matrixModel, i, 3);
			this.addCheckToGrid(l.isEditing(), matrixModel, i, 4);
		}

		// Definig Grid Title Component
		// LwGridCaption caption = new LwGridCaption(treeGrid);
		// caption.setBackground(Color.white);
		// caption.setBorderView(null);
		// caption.putTitle (0, title);
		// caption.putTitle (1, image1);
		// caption.putTitle (2, image2);
		// caption.putTitle (3, image3);
		// caption.putTitle (4, image4);

		// treeGrid.add (LwGrid.TOP_CAPTION_EL, caption);

		// Adapt the table size to the content
		// treeGrid.usePsMetric(true);
		treeGrid.setEditorProvider(new org.zaval.lw.grid.LwDefEditors());
		int scr_w = Math.min(480, UIConstants.SCREEN_SIZE.width);
		treeGrid.setColWidth(0, scr_w*2/5); // treeGrid.getColPSWidth(0));
		treeGrid.setColWidth(1, image1.getPreferredSize().width + 10);
		treeGrid.setColWidth(2, image2.getPreferredSize().width + 10);
		treeGrid.setColWidth(3, image3.getPreferredSize().width + 10);
		treeGrid.setColWidth(4, image4.getPreferredSize().width + 10);
	}

	public Tree createTreeTablePanel(ArrayList layers) {
		try{
		// Defining Layer Table Title
		final LwTextRender title = new LwTextRender("");
		title.setFont(DialogConstants.getDefaultTitleFont());
		title.getTextModel().setText(Messages.getText("LayersTableItem.0"));


		// if((layers!=null)&&(layers.size()>0)){

		Item treeRoot = new Item(title);
//		
//				" " + 
//				(Messages.getText("LayersTableItem.0")) + " ");

		Tree model = new Tree(treeRoot);

		FLayer temp;
		
		for (int numLayer = 0; numLayer < layers.size(); numLayer++) {
			Item lyrData = null;
			temp = ((FLayer) layers.get(numLayer));
			if (temp.getName() == null) {
				int index = (temp.getMetadataID()).toString().lastIndexOf(
						File.separator);
				String n = temp.getMetadataID().toString().substring(index + 1);
				LwTextRender lyrlbl = new LwTextRender(n);
				lyrlbl.setFont(DialogConstants.getDefaultFont());
				lyrData = new Item(lyrlbl);
				temp.setName(n);
			} else {
				LwTextRender lyrlbl = new LwTextRender(temp.getName());
				lyrlbl.setFont(DialogConstants.getDefaultFont());
				lyrData = new Item(lyrlbl);
				// lyrData = new Item();
			}
			model.add(treeRoot, lyrData);
		}
		
		return model;
		}catch(MetadataException e){
			logger.error(e.fillInStackTrace().toString());
		}
		return null;
		/**
		 * GRID NORMAL... POR SI ACASO
		 */
		// }else{
		//		
		// Matrix m = (layers != null) ? new Matrix(0, 5): new Matrix(1, 5);
		//			
		// LwGrid grid = null;
		//			
		// if (grid==null){
		// grid = new LwGrid(m);
		// grid.setBackground(Color.white);
		// }
		//			
		// LwGridCaption caption = new LwGridCaption(grid);
		// caption.setBackground(Color.white);
		// caption.setBorderView(null);
		// caption.putTitle (0, title);
		// caption.putTitle (1, image1);
		// caption.putTitle (2, image2);
		// caption.putTitle (3, image3);
		// caption.putTitle (4, image4);
		// grid.add (LwGrid.TOP_CAPTION_EL, caption);
		// int gridWidth =
		// grid.getColWidth(0)+grid.getColWidth(1)+grid.getColWidth(2)+grid.getColWidth(3)+grid.getColWidth(4);
		// int titleTextWidth = grid.getColWidth(0);
		// int image1Width = image1.getPreferredSize().width;
		// int image2Width = image2.getPreferredSize().width;
		// int image3Width = image3.getPreferredSize().width;
		// int image4Width = image4.getPreferredSize().width;
		// int imagesSize = image1Width +image2Width +image3Width
		// +image4Width+titleTextWidth+25;
		// int estimatedTitleSize = gridWidth-imagesSize;
		// grid.setColWidth(1, image1Width + 10);
		// grid.setColWidth(2, image2Width + 10);
		// grid.setColWidth(3, image3Width + 10);
		// grid.setColWidth(4, image4Width + 10);
		//		    
		// //Calcule icons size
		// if (title.getPreferredSize().width > estimatedTitleSize)
		// grid.setColWidth(0, title.getPreferredSize().width + 10);
		// else
		// grid.setColWidth(0, estimatedTitleSize);
		//		    
		// root.add (LwBorderLayout.CENTER, grid);
		// return root;
		// }
	}

	private void updateLayer(boolean checked, int type) {

		if (layers == null) {
			getLayers(true);
		}

		FLayer f = this.getSelectedLayer();
		if (f == null)
			return;

		switch (type) {
		case VISIBLE:
			f.setVisible(checked);
			break;
		case INFOABLE:
			f.setDynValue(MapControl.INFO_ACTIVATED_DYN_PROPERTY_NAME, new Boolean(checked));
			break;
		case EDITABLE:
			try {
				if (!checked) {
					
					MessageBox.showDialog(
							true,
							bc,
							Messages.getText("save_edits"),
							new EditionEndListener((FLyrVect) f),
							MessageBox.MESSAGE_TYPE_INFO,
							MessageBox.MESSAGE_OK_CANCEL);
				} else {
					layers.add(f);
					((FLyrVect) f).getMapContext().setLayersToSnap(layers);
				}
				f.setEditing(checked);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
			break;
		}
	}

	private void addCheckToGrid(boolean checked, MatrixModel matrixModel,
			int row, int column) {
		LwPanel checkCell = new LwPanel();
		checkCell.setLwLayout(new LwFlowLayout(LwToolkit.LEFT,
				LwToolkit.CENTER, LwToolkit.HORIZONTAL, 2));
		LwCheckbox check = new LwCheckbox();
		check.setState(checked);
		checkCell.add(check);
		LwCompRender render = new LwCompRender(checkCell);
		matrixModel.put(row, column, render);
	}

	public String getTitle() {
		return Messages.getText("LayersManagerDialog.0");
	}

	class LwCustomViewProvider implements LwViewProvider {
		public LwView getView(LwComponent d, Object obj) {
			LwPanel panel = new LwPanel();
			panel.setLwLayout(new LwFlowLayout());
			// LwImage icon = new LwImage(getImage("lw.gif"));
			String value = (String) ((Item) obj).getValue();
			LwLabel title = new LwLabel(value);
			// panel.add (icon);
			panel.add(title);
			return new LwCompRender(panel);
		}
	}

	public void taskFinished(ITask task) {
		rebuildTOC();
	}

	public void rebuildTOC() {
		toc.remove(treeGrid);
		addTreeGrid();
	}

	public void taskInited(ITask task) { }
	public void taskProgress(ITask task, int progress) { }
	
	private class EditionEndListener implements LwActionListener {

		private FLyrVect _lyr_ = null;
		
		public EditionEndListener(FLyrVect llyr) {
			_lyr_ = llyr;
		}
		
		public void actionPerformed(Object src, Object action) {
			
			if (action instanceof Integer) {
				Integer acti = (Integer) action;
				if (acti.intValue() == MessageBox.MESSAGE_OK) {
					try {
						_lyr_.getFeatureStore().finishEditing();
						_lyr_.reload();
						_lyr_.getMapContext().setLayersToSnap(new ArrayList());
						_lyr_.setEditing(false);
					} catch (Exception ex) {
						logger.error("While term. editing: " + ex.getMessage());
					}
				} else {
					try {
						_lyr_.getFeatureStore().cancelEditing();
						_lyr_.getMapContext().setLayersToSnap(new ArrayList());
						_lyr_.setEditing(false);
					} catch (Exception ex) {
						logger.error("While term. editing: " + ex.getMessage());
					}
				}
			}
		}
	}

	public void layerAdded(LayerCollectionEvent e) {
		updateToc();
	}

	public void layerMoved(LayerPositionEvent e) {
		updateToc();
	}

	public void layerRemoved(LayerCollectionEvent e) {
		updateToc();
	}

	public void layerMoving(LayerPositionEvent e) throws CancelationException { }
	public void layerAdding(LayerCollectionEvent e) throws CancelationException { }
	public void layerRemoving(LayerCollectionEvent e)
			throws CancelationException { }
	public void visibilityChanged(LayerCollectionEvent e)
			throws CancelationException { }
	
	
}