/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.base.layer.dialog;

import java.awt.Color;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.tools.dispose.DisposableIterator; 
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry.TYPES;
import org.gvsig.fmap.mapcontext.exceptions.LegendLayerException;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.fmap.mapcontext.rendering.legend.IVectorLegend;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.ILabelingStrategy;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.IZoomConstraints;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.extension.base.util.DataUtils;
import org.gvsig.mobile.symbology.basic.DefaultSymbol;
import org.gvsig.mobile.symbology.labeling.SimplePlacementConstraints;
import org.gvsig.mobile.symbology.labeling.SingleFieldLabelingStrategy;
import org.gvsig.mobile.symbology.labeling.constraints.SimpleZoomConstraints;
import org.gvsig.mobile.symbology.labeling.fonts.FontFamilyManager;
import org.gvsig.mobile.symbology.legend.AbstractVectorLegend;
import org.gvsig.mobile.symbology.legend.RangeVectorLegend;
import org.gvsig.mobile.symbology.legend.SingleSymbolVectorLegend;
import org.gvsig.mobile.symbology.legend.UniqueValueLegend;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.ColorChooserDialog;
import org.gvsig.mobile.ui.lw.ColorChooserListener;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.gvsig.mobile.ui.lw.UIConstants;
import org.gvsig.tools.dataTypes.DataTypes;
import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwGroup;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwList;
import org.zaval.lw.LwNotebook;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwSlider;
import org.zaval.lw.LwTextRender;
import org.zaval.lw.LwToolkit;

public class LayerSymbologyDialog extends OkCancelDialog implements
		ColorChooserListener {

	public static final String LABEL_COVERS_LINE = "covering_line";
	public static final String LABEL_ABOVE_LINE = "above_line";
	public static final String LABEL_UNDER_LINE = "under_line";
	private static final String LABEL_POS_ABOVE = "above";
	private static final String LABEL_POS_UNDER = "below";
	private static final String LABEL_POS_RIGHT = "right";
	private static final String LABEL_POS_LEFT = "left";
	
	
	final static private Logger logger =
		LoggerFactory.getLogger(LayerSymbologyDialog.class);
	
	// =============================
	private LwNotebook mainTabbedPane;
	private LwComponent[] tab;
	// labeling
	private LwCheckbox activeLabeling;
	
	private LwCheckbox scaleLimitLabelcheck;
	private LwCombo scaleLimitCombo;

	private LwCombo labelFontCombo;
	private LwButton labelColorButton;
	private LwSlider labelSizeSlider;
	private LwLabel pointsLabel;
	
	private LwCombo labelAttCombo;
	private LwCombo labelPosCombo;
	// private LwCheckbox activeRot;
	private LwCombo rotAttCombo;
	private LwCombo overLinePosCombo;
	
	private LwCheckbox pointPosRadioOffset, pointPosRadioAtt, curvePosRadioParalel;
	private LwCheckbox curvePosRadioAtt, surfacePosRadioCentered, surfacePosRadioAtt;
	
	// private LwCheckbox lineRotRadioNone;
	// private LwCheckbox lineRotRadioOverLine;
	// private LwCheckbox lineRotRadioAtt;

	// symbology
	// common
	private LwLabel lineWLabel;
	private LwSlider lineWidthSlider;
	private LwLabel lineWidthValueLabel;
	private LwCheckbox fillCheck;
	// single
	private LwButton fillButton;
	private LwButton borderButton;
	// range
	private LwCombo rangeSymAttCombo;
	private LwButton minButton;
	private LwButton maxButton;
	// ===============================
	
	private FLyrVect layer;
	private ILabelingStrategy labelStrategy;
	private int symbolType = TYPES.SURFACE;
	// private FeatureType featType;
	
	// unique
	private LwCombo allAttCombo;
	
	// aux
	private LwConstraints leftNoFillConstraints;
	private LwConstraints leftFillConstraints;
	private LwConstraints centerFillConstraints;
	private LwCheckbox[] exclusiveLegendType;
	
	public LayerSymbologyDialog(boolean scrollable, BundleContext bc, FLyrVect lyr) {
		super(scrollable, bc, false);
		
		DialogConstants.COLOR_BUTTON_PREF_SIZE = (8 * new LwCheckbox("", LwCheckbox.RADIO).getPreferredSize().width) / 10; 
		layer = lyr;
		
		if (lyr.isLabeled()) {
			labelStrategy = lyr.getLabelingStrategy();
		}

		try {
			symbolType = lyr.getShapeType();
		} catch (Exception ex) {
			logger.error("While getting shapetype: " + ex.getMessage());
			symbolType = TYPES.SURFACE;
		}
		initialize(scrollable);
		loadControls((AbstractVectorLegend) lyr.getLegend(), labelStrategy);
		actionPerformed(mainTabbedPane, new Integer(0));
	}

	private void loadControls(AbstractVectorLegend lgn, ILabelingStrategy lblstrat) {
		
		this.lineWidthSlider.setValue(lgn.getBorderWidth());
		
		this.fillCheck.setState( ((DefaultSymbol) lgn.getDefaultSymbol()).isFilled());
		
		// Legend ------------------------------------
		// init colors
		colorSelected(minButton, new Color(150, 150, 255));
		colorSelected(maxButton, Color.RED.darker());
		colorSelected(labelColorButton, Color.BLACK);

		borderButton.setBackground(Color.BLACK);
		fillButton.setBackground(new Color(230,85,13));
		
		if (lgn instanceof SingleSymbolVectorLegend) {
			
			exclusiveLegendType[0].setState(true);
			SingleSymbolVectorLegend xleg = (SingleSymbolVectorLegend) lgn;
			DefaultSymbol dsym = (DefaultSymbol) xleg.getDefaultSymbol();
			
			this.fillButton.setBackground(dsym.getColor());
			if ((symbolType == TYPES.SURFACE) || (symbolType == TYPES.MULTISURFACE)) {
				this.borderButton.setBackground(dsym.getBorderColor());
			} else {
				this.borderButton.setBackground(dsym.getColor());
			}
			
		} else {
			
			if (lgn instanceof RangeVectorLegend) {

				exclusiveLegendType[1].setState(true);
				RangeVectorLegend xleg = (RangeVectorLegend) lgn;
				setAsSelected(rangeSymAttCombo.getList(), xleg.getFieldName());
				maxButton.setBackground(xleg.getMaxColor());
				minButton.setBackground(xleg.getMinColor());

			} else {
				
				if (lgn instanceof UniqueValueLegend) {

					exclusiveLegendType[2].setState(true);
					UniqueValueLegend xleg = (UniqueValueLegend) lgn;
					setAsSelected(allAttCombo.getList(), xleg.getAttName());

				} else {
					
				}
			}
		}

		// Labeling ------------------------------------
		ILabelingStrategy strat = layer.getLabelingStrategy();
		if (strat == null) {
			activeLabeling.setState(false);
		} else {
			activeLabeling.setState(true);
			
			IZoomConstraints zoomc = strat.getZoomConstraints();
			if (zoomc == null) {
				this.scaleLimitLabelcheck.setState(false);
				// scaleLimitCombo.setEnabled(false);
			} else {
				this.scaleLimitLabelcheck.setState(true);
				// scaleLimitCombo.setEnabled(true);
				this.setSelectedScale(getScalesCombo().getList(), zoomc.getMaxScale());
			}
			
			if (strat instanceof SingleFieldLabelingStrategy) {
				SingleFieldLabelingStrategy sstrat = (SingleFieldLabelingStrategy) strat;
				DataUtils.setSelected(labelAttCombo.getList(), sstrat.getUsedFields()[0]);
				int fh = sstrat.getFont().getHeight();
				fh = Math.min(fh, labelSizeSlider.getMax());
				fh = Math.max(fh, labelSizeSlider.getMin());
				labelSizeSlider.setValue(fh);
				DataUtils.setSelected(labelFontCombo.getList(), sstrat.getFont().getName());
				labelColorButton.setBackground(sstrat.getFontColor());
				
				int lalo = sstrat.getLabelLocation();
				
				switch (sstrat.getShapeType()) {
				case TYPES.POINT:
				case TYPES.MULTIPOINT:
					switch (lalo) {
					case SimplePlacementConstraints.LABEL_WITH_ROTATION_ATTRIBUTE:
						pointPosRadioAtt.setState(true);
						String rot_att_name = DataUtils.getNameOfNthAttribute(
								layer,
								sstrat.getRotationAttributeIndex());
						DataUtils.setSelected(rotAttCombo.getList(), rot_att_name);
						break;
						
						/*
						 * listt.addItem(LABEL_POS_ABOVE);
						 * listt.addItem(LABEL_POS_UNDER);
						 * listt.addItem(LABEL_POS_RIGHT);
						 * listt.addItem(LABEL_POS_LEFT);
						 */
					case SimplePlacementConstraints.LABEL_NORTH_OF_POINT:
						pointPosRadioOffset.setState(true);
						getLabelPosCombo().getList().select(0);
						break;
						
					case SimplePlacementConstraints.LABEL_SOUTH_OF_POINT:
						pointPosRadioOffset.setState(true);
						getLabelPosCombo().getList().select(1);
						break;
						
					case SimplePlacementConstraints.LABEL_EAST_OF_POINT:
						pointPosRadioOffset.setState(true);
						getLabelPosCombo().getList().select(2);
						break;
						
					case SimplePlacementConstraints.LABEL_WEST_OF_POINT:
						pointPosRadioOffset.setState(true);
						getLabelPosCombo().getList().select(3);
						break;
						
					default:
						logger.error("Unexpected location: " + lalo);
						break;
					}
					break;
					
				case TYPES.CURVE:
				case TYPES.MULTICURVE:
				case TYPES.GEOMETRY:
					
					switch (lalo) {
					case SimplePlacementConstraints.LABEL_WITH_ROTATION_ATTRIBUTE:
						curvePosRadioAtt.setState(true);
						String rot_att_name = DataUtils.getNameOfNthAttribute(
								layer,
								sstrat.getRotationAttributeIndex());
						DataUtils.setSelected(rotAttCombo.getList(), rot_att_name);
						
						break;
						
					/*
					 * listt.addItem(LABEL_COVERS_LINE);
					 * listt.addItem(LABEL_ABOVE_LINE);
					 * listt.addItem(LABEL_UNDER_LINE);
					 */
					case SimplePlacementConstraints.LABEL_ABOVE_LINE:
						curvePosRadioParalel.setState(true);
						getOverLineCombo().getList().select(0);
						break;
						
					case SimplePlacementConstraints.LABEL_OVER_LINE:
						curvePosRadioParalel.setState(true);
						getOverLineCombo().getList().select(1);
						break;
						
					case SimplePlacementConstraints.LABEL_BELOW_LINE:
						curvePosRadioParalel.setState(true);
						getOverLineCombo().getList().select(2);
						break;
						
					default:
						logger.error("Unexpected location: " + lalo);
						break;
					}
					break;
					
				case TYPES.SURFACE:
				case TYPES.MULTISURFACE:
					switch (lalo) {
					case SimplePlacementConstraints.LABEL_WITH_ROTATION_ATTRIBUTE:
						surfacePosRadioAtt.setState(true);
						String rot_att_name = DataUtils.getNameOfNthAttribute(
								layer,
								sstrat.getRotationAttributeIndex());
						DataUtils.setSelected(rotAttCombo.getList(), rot_att_name);
						break;
					case SimplePlacementConstraints.LABEL_NOT_CHANGED:
						surfacePosRadioCentered.setState(true);
						break;
					default:
						logger.error("Unexpected location: " + lalo);
						break;
					}
					break;
					
				default:
					logger.error("Unexpected shape type: " + sstrat.getShapeType());
					break;
				}
				
			} else {
				activeLabeling.setState(false);
				logger.error("Unexpected stategy class: " + strat.getClass().getName());
			}
		}
		
		// init slider labels
		actionPerformed(lineWidthSlider, null);
		actionPerformed(labelSizeSlider, null);

	}

	private void setAsSelected(LwAList list, String str) {
		
		int cnt = list.count();
		for (int i=0; i<cnt; i++) {
			if (((LwLabel) list.getItem(i)).getText().compareToIgnoreCase(str) == 0) {
				list.select(i);
				return;
			}
		}
		logger.error("Did not find item " + str + "in the combo.");
	}

	public void actionPerformed(Object arg0, Object arg1) {
		if (arg0 == lineWidthSlider) {
			lineWidthValueLabel.setText("  " + lineWidthSlider.getValue() + " pt ");
		} else if (arg0 == this.getCancelButton()) {
			WindowManager.getInstance().closeActivePanel();
		} else if (arg0 == this.getOkButton()) {
			setLegend();
			setLabeling();
			WindowManager.getInstance().closeActivePanel();
		} else if (arg0 instanceof LwButton) {
			if ((arg0 == fillButton) || (arg0 == borderButton) || (arg0 == minButton)
					|| (arg0 == maxButton) || (arg0 == labelColorButton)) {
				ColorChooserDialog d = new ColorChooserDialog((LwButton) arg0, this);
				d.showDialog();
			}
		} else if (arg0 == labelSizeSlider) {
			this.pointsLabel.setText(labelSizeSlider.getValue() + " pt ");
		} else if (arg0 == this.mainTabbedPane) {
			// this.pointsLabel.setText("  " + labelSizeSlider.getValue() + " pt");
			// int indx = ((Integer) arg1).intValue();
			// tab[indx].l
			// mainTabbedPane.layout(mainTabbedPane); // .getView(indx).getTop();
			// mainTabbedPane.getView(indx).getPreferredSize();
			// arg0.getClass();
			// System.out.println("W/H = " + getSize().width + " , " +  getSize().height);
			// tab[indx].setSize(getSize().width, getSize().height);
			// this.setSize(0,0);
		}
		
		// 
//			LwCheckbox cb = (LwCheckbox) arg0;
//			cb.
//			if ((arg0 == fillButton) || (arg0 == borderButton)) {
//				ColorChooserDialog d = new ColorChooserDialog((LwButton) arg0, this);
//				d.showDialog();
//			}
	}

	private void setLabeling() {
		
		if (activeLabeling.getState()) {
			
			// SimplePlacementConstraints simcon = new SimplePlacementConstraints();
			// SimpleLabelLocationMetrics simmet = new SimpleLabelLocationMetrics();
			
			float fontsize = labelSizeSlider.getValue();
			
			String labelAtt_name = ((LwLabel) labelAttCombo.getList().getSelected()).getText();
			
			
			int rotAtt_type = -500;
			int rotAtt_ind = -500;
			
			if (rotAttCombo.isEnabled() && (rotAttCombo.getList().count() > 0)) {
				String rotAtt_name = ((LwLabel) rotAttCombo.getList().getSelected()).getText();
				
				FeatureType ft = null;
				FeatureStore fs = null;
				try {
					fs = layer.getFeatureStore();
					ft = fs.getDefaultFeatureType();
				} catch (DataException e) {
					logger.error("While getting FeatureType: " + e.getMessage());
					return;
				}
				rotAtt_type = ft.getAttributeDescriptor(rotAtt_name).getType();
				rotAtt_ind = ft.getAttributeDescriptor(rotAtt_name).getIndex();
			}

			String fontn = ((LwLabel) labelFontCombo.getList().getSelected()).getText();
			// bc.
			// File ff = bc.getDataFile(".");
			// Bundle bu = bc.getBundle();
			
			SingleFieldLabelingStrategy stra = new SingleFieldLabelingStrategy(
					layer,
					labelAtt_name,
					fontn,
					fontsize, // 0.4,
					labelColorButton.getBackground(),
					rotAtt_ind,
					rotAtt_type,
					getLabelLocation());
			
			if (!scaleLimitLabelcheck.getState()) {
				stra.setZoomConstraints(null);
			} else {
				long v = getSelectedScale(this.scaleLimitCombo.getList());
				IZoomConstraints zc = new SimpleZoomConstraints(v);
				stra.setZoomConstraints(zc);
			}
			// SimplePlacementConstraints.LABEL_WITH_ROTATION_ATTRIBUTE);
			
			layer.setLabelingStrategy(stra);
			layer.setIsLabeled(true);
		} else {
			layer.setLabelingStrategy(null);
			layer.setIsLabeled(false);
		}
		// TODO Auto-generated method stub
		
	}

	private int getLabelLocation() {
		
		switch (symbolType) {

		case TYPES.POINT:
		case TYPES.MULTIPOINT:
			if (pointPosRadioAtt.getState()) {
				return SimplePlacementConstraints.LABEL_WITH_ROTATION_ATTRIBUTE;
			} else {
				int selind = getLabelPosCombo().getList().getSelectedIndex();
				switch (selind) {
				case 0:
					return SimplePlacementConstraints.LABEL_NORTH_OF_POINT;
				case 1:
					return SimplePlacementConstraints.LABEL_SOUTH_OF_POINT;
				case 2:
					return SimplePlacementConstraints.LABEL_EAST_OF_POINT;
				case 3:
					return SimplePlacementConstraints.LABEL_WEST_OF_POINT;
				default:
					return SimplePlacementConstraints.LABEL_EAST_OF_POINT;
				}
			}
		case TYPES.CURVE:
		case TYPES.MULTICURVE:
			if (curvePosRadioAtt.getState()) {
				return SimplePlacementConstraints.LABEL_WITH_ROTATION_ATTRIBUTE;
			} else {
				int selind = getOverLineCombo().getList().getSelectedIndex();
				switch (selind) {
				case 0:
					return SimplePlacementConstraints.LABEL_ABOVE_LINE;
				case 1:
					return SimplePlacementConstraints.LABEL_OVER_LINE;
				case 2:
					return SimplePlacementConstraints.LABEL_BELOW_LINE;
				default:
					return SimplePlacementConstraints.LABEL_ABOVE_LINE;
				}
			}

		case TYPES.SURFACE:
		case TYPES.MULTISURFACE:
			if (surfacePosRadioAtt.getState()) {
				return SimplePlacementConstraints.LABEL_WITH_ROTATION_ATTRIBUTE;
			} else {
				return SimplePlacementConstraints.LABEL_NOT_CHANGED;
			}

		default:
			return SimplePlacementConstraints.LABEL_NOT_CHANGED;
		}

	}

	private void setLegend() {
		
		IVectorLegend newlegend = null;
		
		if (exclusiveLegendType[0].getState()) {
			
			SingleSymbolVectorLegend ssleg = new SingleSymbolVectorLegend();
			((DefaultSymbol) ssleg.getDefaultSymbol()).setFilled(fillCheck.getState());
			ssleg.setBorderWidth(lineWidthSlider.getValue());
			
			DefaultSymbol dss = new DefaultSymbol();

			switch (symbolType) {

			case TYPES.POINT:
			case TYPES.MULTIPOINT:
			case TYPES.CURVE:
			case TYPES.MULTICURVE:
				dss.setColor(borderButton.getBackground());
				dss.setBorderColor(borderButton.getBackground());
				break;
			default:
				dss.setColor(fillButton.getBackground());
				dss.setBorderColor(borderButton.getBackground());
				break;
			}

			dss.setWidth(lineWidthSlider.getValue());
			dss.setFilled(fillCheck.getState());
			ssleg.setDefaultSymbol(dss);
			newlegend = ssleg;

		} else {
			if (exclusiveLegendType[1].getState()) {
				
				String att_name = ((LwLabel) rangeSymAttCombo.getList().getSelected()).getText();
				
				FeatureType ft = null;
				FeatureStore fs = null;
				try {
					fs = layer.getFeatureStore();
					ft = fs.getDefaultFeatureType();
				} catch (DataException e) {
					logger.error("While getting FeatureType: " + e.getMessage());
					return;
				}
				int att_type = ft.getAttributeDescriptor(att_name).getType();
				int att_ind = ft.getAttributeDescriptor(att_name).getIndex();
				RangeVectorLegend lege = new RangeVectorLegend();
				lege.setFieldName(att_name);
				lege.setFieldInd(att_ind);
				lege.setFieldType(att_type);
				lege.setBorderWidth(lineWidthSlider.getValue());
				lege.setMinColor(minButton.getBackground());
				lege.setMaxColor(maxButton.getBackground());
				double[] minmax = getMinMaxForAtt(att_name, fs, att_type);
				lege.setMinVal(minmax[0]);
				lege.setMaxVal(minmax[1]);
				((DefaultSymbol) lege.getDefaultSymbol()).setFilled(fillCheck.getState());
				newlegend = lege;
				
			} else {
				if (exclusiveLegendType[2].getState()) {
					
					String att_name = ((LwLabel) allAttCombo.getList().getSelected()).getText();
					UniqueValueLegend uvl = new UniqueValueLegend();
					// att_name, symbolType, slider.getValue(), 
					uvl.setAttName(att_name);
					uvl.setBorderWidth(lineWidthSlider.getValue());
					((DefaultSymbol) uvl.getDefaultSymbol()).setFilled(fillCheck.getState());
					newlegend = uvl;
					
				} else {
					
					
					
				}
			}
		}
		try {
			layer.setLegend(newlegend);
		} catch (LegendLayerException e) {
			logger.error("While setting legend: " + e.getMessage());
		}
	}

	private double[] getMinMaxForAtt(String fname, FeatureStore fs, int attType) {
		
		double[] resp = new double[2];
		resp[0] = Double.MAX_VALUE;
		resp[1] = Double.MIN_VALUE;
		Feature feat;
		
		try {
			DisposableIterator diter = fs.getFeatureSet().fastIterator();
			
			if (attType == DataTypes.DOUBLE) {
				
				double aux = 0;
				while (diter.hasNext()) {
					feat = (Feature) diter.next();
					aux = feat.getDouble(fname);
					if (aux > resp[1]) resp[1] = aux;
					if (aux < resp[0]) resp[0] = aux;
				}
			} else {
				if (attType == DataTypes.INT) {
					int aux = 0;
					while (diter.hasNext()) {
						feat = (Feature) diter.next();
						aux = feat.getInt(fname);
						if (aux > resp[1]) resp[1] = aux;
						if (aux < resp[0]) resp[0] = aux;
					}
				} else {
					if (attType == DataTypes.FLOAT) {
						float aux = 0;
						while (diter.hasNext()) {
							feat = (Feature) diter.next();
							aux = feat.getFloat(fname);
							if (aux > resp[1]) resp[1] = aux;
							if (aux < resp[0]) resp[0] = aux;
						}
					} else {
							if (attType == DataTypes.LONG) {
								long aux = 0;
								while (diter.hasNext()) {
									feat = (Feature) diter.next();
									aux = feat.getLong(fname);
									if (aux > resp[1]) resp[1] = aux;
									if (aux < resp[0]) resp[0] = aux;
								}
							} else {
								if (attType == DataTypes.BYTE) {
									byte aux = 0;
									while (diter.hasNext()) {
										feat = (Feature) diter.next();
										aux = feat.getByte(fname);
										if (aux > resp[1]) resp[1] = aux;
										if (aux < resp[0]) resp[0] = aux;
									}
								} else {
									double aux = 0;
									while (diter.hasNext()) {
										feat = (Feature) diter.next();
										String auxstr = feat.get(fname).toString();
										
										try {
											aux = Double.parseDouble(auxstr);
										} catch (Exception ex) {
											logger.error("While parsing string for uncommon numeric type: " + ex.getMessage());
											aux = 0;
										}
										if (aux > resp[1]) resp[1] = aux;
										if (aux < resp[0]) resp[0] = aux;
									}
								}
							}
					}
				}
			}
			diter.dispose();
		} catch (DataException e) {
			logger.error("While getting feat iter: " + e.getMessage());
		}
		return resp;
		
	}

	public LwPanel buildMainPanel() {
		leftNoFillConstraints = new LwConstraints();
		leftNoFillConstraints.ax = LwToolkit.LEFT;
		leftNoFillConstraints.fill = LwToolkit.NONE;
		leftFillConstraints = new LwConstraints();
		leftFillConstraints.ax = LwToolkit.LEFT;
		leftFillConstraints.fill = LwToolkit.HORIZONTAL;
		centerFillConstraints = new LwConstraints();
		centerFillConstraints.ax = LwToolkit.CENTER;
		centerFillConstraints.fill = LwToolkit.HORIZONTAL;
		
		exclusiveLegendType = new LwCheckbox[3];
		LwGroup switchMan = new LwGroup();
		exclusiveLegendType[0] = new LwCheckbox(Messages.getText("Single_symbol") + "  ", LwCheckbox.RADIO);
		exclusiveLegendType[0].setSwitchManager(switchMan);
		exclusiveLegendType[1] = new LwCheckbox(Messages.getText("Color_range") + "  ", LwCheckbox.RADIO);
		exclusiveLegendType[1].setSwitchManager(switchMan);
		exclusiveLegendType[2] = new LwCheckbox(Messages.getText("Single_value") + "  ", LwCheckbox.RADIO);
		exclusiveLegendType[2].setSwitchManager(switchMan);
		// switchMan.register(exclusiveLegendType[1]);

		tab = new LwComponent[2];
		tab[0] = getPanelForType(symbolType); 
		// symtab.setLwLayout(new LwBorderLayout());
		// setSize(w, h);

		mainTabbedPane = new LwNotebook(LwToolkit.TOP);
		// mainTabbedPane.setLwLayout(new )
		LwTextRender tabSymb = new LwTextRender("");
		tabSymb.setFont(DialogConstants.getDefaultFont());
		tabSymb.getTextModel().setText(Messages.getText("_Symbology"));
		
		LwTextRender tabEtiq = new LwTextRender("");
		tabEtiq.setFont(DialogConstants.getDefaultFont());
		tabEtiq.getTextModel().setText(Messages.getText("_Labeling"));
		
		tab[1] = getLabelingTab(symbolType);
		mainTabbedPane.addPage(tabSymb, tab[0]);
		mainTabbedPane.addPage(tabEtiq, tab[1]);
		mainTabbedPane.setBackground(Color.white);
		mainTabbedPane.addSelectionListener(this);
		// mainTabbedPane.
		// mainTabbedPane.layout(mainTabbedPane);
		// add(LwBorderLayout.CENTER, mainTabbedPane);
		return mainTabbedPane;
	}

	private LwComponent getLabelingTab(int shptype) {
		
		LwComponent resp = null;
		LwPanel respPanel = new LwPanel();
		
		activeLabeling = new LwCheckbox();
		scaleLimitLabelcheck = new LwCheckbox();
		// scaleLimitLabelcheck.set

		// labelColorButton = new 
		
		LwLabel nameSizeColorLabel = DialogConstants.getDefaultTitleLabel(Messages.getText("Font_name_size_color"));
		// LwLabel fontNameLabel = DialogConstants.getDefaultLabel(Messages.getText("font_name"));
		
		labelSizeSlider = new LwSlider();
		labelSizeSlider.setValues(10, 60, new int[0], 10, 2);
		labelSizeSlider.setScaleStep(2);
		int w = Math.min(160, 3 * UIConstants.SCREEN_SIZE.width / 10);
		labelSizeSlider.setPSSize(w, -1);
		labelSizeSlider.addActionListener(this);
		pointsLabel = DialogConstants.getDefaultLabel("");
		// labelSizeSlider
		LwPanel fontNameSizeColorPanel = getFloPa();
		fontNameSizeColorPanel.add(getLabelFontCombo());
		fontNameSizeColorPanel.add(new LwLabel(" "));
		fontNameSizeColorPanel.add(centerFillConstraints, labelSizeSlider);
		fontNameSizeColorPanel.add(new LwLabel(" "));
		fontNameSizeColorPanel.add(pointsLabel);
		fontNameSizeColorPanel.add(new LwLabel(" "));

		// LwLabel colorLab = DialogConstants.getDefaultLabel(Messages.getText("font_color"));
		labelColorButton = DialogConstants.getButton("", "", this);
		labelColorButton.getViewMan(true).setBorder("br.plain");
		labelColorButton.setOpaque(true);		
		labelColorButton.setPSSize(DialogConstants.COLOR_BUTTON_PREF_SIZE,
				DialogConstants.COLOR_BUTTON_PREF_SIZE);
		labelColorButton.addActionListener(this);
		fontNameSizeColorPanel.add(labelColorButton);
		// activeRot = new LwCheckbox();
		
		LwGroup switchMan = new LwGroup();
		// LwPanel auxPanel2 = null;
		LwCombo rotcom = (LwCombo) getRotAttCombo(); 
		
		switch (shptype) {
		
		case TYPES.POINT:
		case TYPES.MULTIPOINT:
			respPanel.setLwLayout(new LwGridLayout(12, 1));

			addRowSep(respPanel);

			LwPanel au1 = getFloPa();
			au1.add(activeLabeling);
			au1.add(DialogConstants.getDefaultTitleLabel(" " + Messages.getText("Use_labels")));
			respPanel.add(centerFillConstraints, au1);
			
			addRowSep(respPanel);
			
			LwPanel au2 = getFloPa();
			au2.add(scaleLimitLabelcheck);
			au2.add(DialogConstants.getDefaultLabel(" " + Messages.getText("Scale_limit") + "  1 : "));
			au2.add(getScalesCombo());
			respPanel.add(au2);
			
			LwPanel au3 = getFloPa();
			au3.add(DialogConstants.getDefaultLabel(Messages.getText("Label_fieldname") + " "));
			au3.add(getLabelAttCombo());
			respPanel.add(au3);

			addRowSep(respPanel);

			respPanel.add(nameSizeColorLabel);
			respPanel.add(fontNameSizeColorPanel);
			
			addRowSep(respPanel);
			
			respPanel.add(DialogConstants.getDefaultTitleLabel(Messages.getText("Label_position")));

			pointPosRadioOffset = new LwCheckbox(Messages.getText("Label_offset") + " ", LwCheckbox.RADIO);
			pointPosRadioOffset.setSwitchManager(switchMan);
			
			LwPanel au5 = getFloPa();
			au5.add(pointPosRadioOffset);
			au5.add(getLabelPosCombo());
			respPanel.add(au5);

			pointPosRadioAtt = new LwCheckbox(Messages.getText("Rotation_from_att") + " ", LwCheckbox.RADIO);
			pointPosRadioAtt.setSwitchManager(switchMan);
			
			pointPosRadioOffset.setState(true);

			LwPanel au6 = getFloPa();
			au6.add(pointPosRadioAtt);
			au6.add(rotcom);
			respPanel.add(au6);

			resp = respPanel;
			if (rotcom.getList().count() == 0) {
				LwAList rotlist = rotcom.getList();
				rotlist.addItem("         ");
				rotcom.setEnabled(false);
				pointPosRadioAtt.setState(false);
				pointPosRadioAtt.setEnabled(false);
			}
			break;
			
		case TYPES.CURVE:
		case TYPES.MULTICURVE:
		case TYPES.GEOMETRY:

			respPanel.setLwLayout(new LwGridLayout(12, 1));
			
			addRowSep(respPanel);

			LwPanel au7 = getFloPa();
			au7.add(activeLabeling);
			au7.add(DialogConstants.getDefaultTitleLabel(" " + Messages.getText("Use_labels")));
			respPanel.add(centerFillConstraints, au7);
			
			addRowSep(respPanel);
			
			LwPanel au8 = getFloPa();
			au8.add(scaleLimitLabelcheck);
			au8.add(DialogConstants.getDefaultLabel(" " + Messages.getText("Scale_limit") + "  1 : "));
			au8.add(getScalesCombo());
			respPanel.add(au8);
			
			LwPanel au9 = getFloPa();
			au9.add(DialogConstants.getDefaultLabel(Messages.getText("Label_fieldname") + " "));
			au9.add(getLabelAttCombo());
			respPanel.add(au9);

			addRowSep(respPanel);

			respPanel.add(nameSizeColorLabel);
			respPanel.add(fontNameSizeColorPanel);
			
			addRowSep(respPanel);
			
			respPanel.add(DialogConstants.getDefaultTitleLabel(Messages.getText("Label_position")));

			curvePosRadioParalel = new LwCheckbox(Messages.getText("Paralel"), LwCheckbox.RADIO);
			curvePosRadioParalel.setSwitchManager(switchMan);
			
			LwPanel au10 = getFloPa();
			au10.add(leftNoFillConstraints, curvePosRadioParalel);
			au10.add(getOverLineCombo());
			respPanel.add(au10);

			// addGap(respPanel, 2, 10);

			curvePosRadioAtt = new LwCheckbox(Messages.getText("Rotation_from_att"), LwCheckbox.RADIO);
			curvePosRadioAtt.setSwitchManager(switchMan);
			
			curvePosRadioParalel.setState(true);
			
			LwPanel au11 = getFloPa();
			au11.add(leftNoFillConstraints, curvePosRadioAtt);
			au11.add(rotcom);
			respPanel.add(au11);

			resp = respPanel;
			
			if (rotcom.getList().count() == 0) {
				LwAList rotlist = rotcom.getList();
				rotlist.addItem("         ");
				rotcom.setEnabled(false);
				curvePosRadioAtt.setState(false);
				curvePosRadioAtt.setEnabled(false);
			}
			break;

		case TYPES.SURFACE:
		case TYPES.MULTISURFACE:

			respPanel.setLwLayout(new LwGridLayout(12, 1));
			
			addRowSep(respPanel);

			LwPanel au12 = getFloPa();
			au12.add(activeLabeling);
			au12.add(DialogConstants.getDefaultTitleLabel(" " + Messages.getText("Use_labels")));
			respPanel.add(centerFillConstraints, au12);
			
			addRowSep(respPanel);
			
			LwPanel au13 = getFloPa();
			au13.add(scaleLimitLabelcheck);
			au13.add(DialogConstants.getDefaultLabel(" " + Messages.getText("Scale_limit") + "  1 : "));
			au13.add(getScalesCombo());
			respPanel.add(au13);
			
			LwPanel au14 = getFloPa();
			au14.add(DialogConstants.getDefaultLabel(Messages.getText("Label_fieldname") + " "));
			au14.add(getLabelAttCombo());
			respPanel.add(au14);

			addRowSep(respPanel);

			respPanel.add(nameSizeColorLabel);
			respPanel.add(fontNameSizeColorPanel);
			
			addRowSep(respPanel);
			
			respPanel.add(DialogConstants.getDefaultTitleLabel(Messages.getText("Label_position")));

			surfacePosRadioCentered = new LwCheckbox(Messages.getText("Centered"), LwCheckbox.RADIO);
			surfacePosRadioCentered.setSwitchManager(switchMan);
			respPanel.add(leftNoFillConstraints, surfacePosRadioCentered);

			surfacePosRadioAtt = new LwCheckbox(Messages.getText("Rotation_from_att") + " ", LwCheckbox.RADIO);
			surfacePosRadioAtt.setSwitchManager(switchMan);
			
			surfacePosRadioCentered.setState(true);
			
			LwPanel au15 = getFloPa();
			au15.add(leftNoFillConstraints, surfacePosRadioAtt);
			
			au15.add(rotcom);
			respPanel.add(au15);
			resp = respPanel;
			
			if (rotcom.getList().count() == 0) {
				LwAList rotlist = rotcom.getList();
				rotlist.addItem("         ");
				rotcom.setEnabled(false);
				surfacePosRadioAtt.setState(false);
				surfacePosRadioAtt.setEnabled(false);
			}
			break;
			
		default:
			resp = new LwLabel("Unknown shape type: " + shptype);
			break;
		}
		
		LwCombo fldNameCombo = (LwCombo) getLabelAttCombo();
		if (fldNameCombo.getList().count() == 0) {
			fldNameCombo.setEnabled(false);
			activeLabeling.setState(false);
			activeLabeling.setEnabled(false);
		}
		
		
		
		return resp;
	}

	private LwCombo getScalesCombo() {
		
		if (scaleLimitCombo == null) {
			
			scaleLimitCombo = new LwCombo();
			LwList lis = new LwList();
			int len = AVAILABLE_SCALES.length;
			for (int i=0; i<len; i++) {
				lis.addItem(String.valueOf(AVAILABLE_SCALES[i]));
				if (i==0) {
					lis.select(i);
				}
			}
			
			scaleLimitCombo.setList(lis);
		}
		return scaleLimitCombo;
	}

	private LwCombo getOverLineCombo() {
		
		if (overLinePosCombo == null) {
			
			overLinePosCombo = new LwCombo();
			LwCompList listt = new LwCompList();
			listt.addItem(LABEL_ABOVE_LINE);
			listt.addItem(LABEL_COVERS_LINE);
			listt.addItem(LABEL_UNDER_LINE);
			listt.select(0);
			
			overLinePosCombo.setList(listt);
		}
		return overLinePosCombo;
	}

	private LwComponent getRotAttCombo() {
		
		if (rotAttCombo == null) {
			rotAttCombo = new LwCombo();
			
			FeatureAttributeDescriptor[] att_descs = null;
			try {
				att_descs =
					layer.getFeatureStore().getDefaultFeatureType().getAttributeDescriptors();
				LwCompList l = getListWithAtts(att_descs, true);
				rotAttCombo.setList(l);
			} catch (Exception  ex) {
				logger.error("While getting attributes: " + ex.getMessage());
			}
		}
		return rotAttCombo;

	}

	private LwComponent getLabelAttCombo() {
		
		if (labelAttCombo == null) {
			labelAttCombo = new LwCombo();
			
			FeatureAttributeDescriptor[] att_descs = null;
			try {
				att_descs =
					layer.getFeatureStore().getDefaultFeatureType().getAttributeDescriptors();
				LwCompList l = getListWithAtts(att_descs, false);
				labelAttCombo.setList(l);
			} catch (Exception  ex) {
				logger.error("While getting attributes: " + ex.getMessage());
			}
		}
		return labelAttCombo;
	}
	
	private LwComponent getLabelFontCombo() {
		// 
		if (labelFontCombo == null) {
			labelFontCombo = new LwCombo();
			
			try {
				LwCompList l = getListOfFonts();
				labelFontCombo.setList(l);
			} catch (Exception  ex) {
				logger.error("While getting attributes: " + ex.getMessage());
			}
		}
		return labelFontCombo;
		
	}

	private LwCompList getListOfFonts() {
		
		LwCompList listt = new LwCompList(); 
		String[] ffs = FontFamilyManager.getAvailableFontNames();
		for (int i=0; i<ffs.length; i++) {
			listt.addItem(ffs[i]);
		}
		if (listt.count() > 0) {
			listt.select(0);
		}
		return listt;
	}

	private LwCombo getLabelPosCombo() {
		
		if (labelPosCombo == null) {
			labelPosCombo = new LwCombo();
			LwCompList listt = new LwCompList();
			listt.addItem(LABEL_POS_ABOVE);
			listt.addItem(LABEL_POS_UNDER);
			listt.addItem(LABEL_POS_RIGHT);
			listt.addItem(LABEL_POS_LEFT);
			labelPosCombo.setList(listt);
			listt.select(0);
		}
		return labelPosCombo;
	}

	private LwPanel getPanelForType(int t) {
		
		LwPanel resp = new LwPanel();
		
		if (t == TYPES.SURFACE || t == TYPES.MULTISURFACE) {
			resp.setLwLayout(new LwGridLayout(13, 1));
		} else {
			resp.setLwLayout(new LwGridLayout(12, 1));
		}
		
		// common
		// resp.add(DialogConstants.getDefaultLabel(Messages.getText("draw_options")));
		// resp.add(new LwLabel());
		
		addSmallRowSep(resp);

		LwPanel auxp = getBoPa();
		lineWLabel = DialogConstants.getDefaultLabel(Messages.getText("LayerSymbologyDialog.1") + "  ");
		auxp.add(LwBorderLayout.WEST, lineWLabel);
		lineWidthSlider = new LwSlider();
		lineWidthSlider.setValues(1, 9, new int[0], 1, 1);
		lineWidthSlider.setScaleStep(1);
		lineWidthSlider.setPSSize(100, -1);
		lineWidthSlider.addActionListener(this);
		
		lineWidthValueLabel = DialogConstants.getDefaultLabel(" pt ");
		
		auxp.add(LwBorderLayout.CENTER, lineWidthSlider);
		auxp.add(LwBorderLayout.EAST, lineWidthValueLabel);
		resp.add(auxp);

		fillCheck = new LwCheckbox();
		if (t == TYPES.SURFACE || t == TYPES.MULTISURFACE) {
			LwPanel auxp2 = getFloPa();
			auxp2.add(leftNoFillConstraints, fillCheck);
			auxp2.add(leftFillConstraints, DialogConstants.getDefaultLabel(Messages.getText("LayerSymbologyDialog.10") + "  "));
			resp.add(auxp2);
		}

		addSmallRowSep(resp);
		// single
		resp.add(leftNoFillConstraints, exclusiveLegendType[0]);

		fillButton = DialogConstants.getButton("", "", this);
		if (t == TYPES.SURFACE || t == TYPES.MULTISURFACE) {

			LwPanel halfp = new LwPanel();
			halfp.setLwLayout(new LwGridLayout(1, 2, LwToolkit.NONE));

			LwPanel auxp3_1 = getFloPa();
			borderButton = DialogConstants.getButton("", "", this);
			borderButton.getViewMan(true).setBorder("br.plain");
			borderButton.setOpaque(true);
			borderButton.setPSSize(DialogConstants.COLOR_BUTTON_PREF_SIZE,
					DialogConstants.COLOR_BUTTON_PREF_SIZE);
			borderButton.addActionListener(this);
			auxp3_1.add(leftNoFillConstraints, borderButton);
			auxp3_1.add(DialogConstants.getDefaultLabel(" " + Messages.getText("LayerSymbologyDialog.9") + "       "));

			LwPanel auxp3_2 = getFloPa();
			fillButton.getViewMan(true).setBorder("br.plain");
			fillButton.setOpaque(true);		
			fillButton.setPSSize(DialogConstants.COLOR_BUTTON_PREF_SIZE,
					DialogConstants.COLOR_BUTTON_PREF_SIZE);
			fillButton.addActionListener(this);
			auxp3_2.add(leftNoFillConstraints, fillButton);
			auxp3_2.add(DialogConstants.getDefaultLabel(" " + Messages.getText("LayerSymbologyDialog.11")));
			
			halfp.add(leftFillConstraints, auxp3_1);
			halfp.add(leftFillConstraints, auxp3_2);
			resp.add(halfp);
		} else {
			LwPanel auxp3 = getFloPa();
			borderButton = DialogConstants.getButton("", "", this);
			borderButton.getViewMan(true).setBorder("br.plain");
			borderButton.setOpaque(true);
			borderButton.setPSSize(DialogConstants.COLOR_BUTTON_PREF_SIZE,
					DialogConstants.COLOR_BUTTON_PREF_SIZE);
			borderButton.addActionListener(this);
			auxp3.add(leftFillConstraints, borderButton);
			auxp3.add(DialogConstants.getDefaultLabel(" " + Messages.getText("LayerSymbologyDialog.9")));
			resp.add(auxp3);
		}

		addSmallRowSep(resp);

		// range
		resp.add(leftNoFillConstraints, exclusiveLegendType[1]);
		// resp.add(new LwLabel());

		LwPanel auxp4 = getBoPa();
		auxp4.add(LwBorderLayout.WEST, DialogConstants.getDefaultLabel(Messages.getText("LayerSymbologyDialog.10") + "  "));
		// addGap(auxp4, 1, 10);
		LwCombo rangeCombo = null;
		try {
			rangeCombo =
				getRangeSymNumericAttsCombo(layer.getFeatureStore().getDefaultFeatureType().getAttributeDescriptors());
			auxp4.add(LwBorderLayout.CENTER,rangeCombo);
		} catch (Exception ex) {
			logger.error("While getting attributes: " + ex.getMessage());
			// resp.add(new LwLabel());
		}
		resp.add(auxp4);
		
		LwPanel half2 = new LwPanel();
		half2.setLwLayout(new LwGridLayout(1, 2, LwToolkit.NONE));
		
		LwPanel auxp5_1 = getFloPa();
		minButton = DialogConstants.getButton("", "", this);
		minButton.getViewMan(true).setBorder("br.plain");
		minButton.setOpaque(true);
		minButton.setPSSize(DialogConstants.COLOR_BUTTON_PREF_SIZE,
				DialogConstants.COLOR_BUTTON_PREF_SIZE);
		auxp5_1.add(leftNoFillConstraints, minButton);
		LwLabel aux2 = DialogConstants.getDefaultLabel(" " + Messages.getText("min_color") + "       ");
		auxp5_1.add(aux2);


		LwPanel auxp5_2 = getFloPa();
		maxButton = DialogConstants.getButton("", "", this);
		maxButton.getViewMan(true).setBorder("br.plain");
		maxButton.setOpaque(true);
		maxButton.setPSSize(DialogConstants.COLOR_BUTTON_PREF_SIZE,
				DialogConstants.COLOR_BUTTON_PREF_SIZE);
		auxp5_2.add(leftNoFillConstraints, maxButton);
		auxp5_2.add(DialogConstants.getDefaultLabel(" " + Messages.getText("max_color")));
		
		half2.add(auxp5_1);
		half2.add(auxp5_2);
		
		resp.add(half2);
		
		// disable if combo empty
		if (rangeCombo.getList().count() == 0) {
			rangeCombo.setEnabled(false);
			minButton.setEnabled(false);
			minButton.setBackground(Color.gray);
			maxButton.setEnabled(false);
			maxButton.setBackground(Color.gray);
			exclusiveLegendType[1].setEnabled(false);
		}

		addSmallRowSep(resp);

		// unique
		resp.add(leftNoFillConstraints, exclusiveLegendType[2]);
		// resp.add(new LwLabel());

		LwPanel auxp6 = getBoPa();
		auxp6.add(LwBorderLayout.WEST, DialogConstants.getDefaultLabel(Messages.getText("Single value") + "  "));
		// addGap(auxp6, 1, 10);
		LwCombo singleCombo = null;
		try {
			singleCombo = getAllAttsCombo(layer.getFeatureStore().getDefaultFeatureType().getAttributeDescriptors()); 
			auxp6.add(LwBorderLayout.CENTER, singleCombo);
		} catch (Exception ex) {
			logger.error("While getting attributes: " + ex.getMessage());
			// resp.add(new LwLabel());
		}

		resp.add(auxp6);
		
		// disable if combo empty
		if (singleCombo.getList().count() == 0) {
			singleCombo.setEnabled(false);
			exclusiveLegendType[2].setEnabled(false);
		}

		return resp;
	}

	private LwPanel getBoPa() {
		LwPanel resp = new LwPanel();
		LwBorderLayout bo = new LwBorderLayout();
		resp.setLwLayout(bo);
		return resp;
	}

	private LwPanel getFloPa() {
		LwPanel resp = new LwPanel();
		LwFlowLayout bo = new LwFlowLayout();
		resp.setLwLayout(bo);
		return resp;
	}

	private void addGap(LwContainer cont, int wcount, int h) {
		for (int i=0; i<wcount; i++) {
			LwLabel aux = new LwLabel("  ");
			aux.setSize(h, h);
			cont.add(aux);
		}
	}

	private void addRowSep(int s, LwContainer cont) {
		LwPanel aux = new LwPanel();
		aux.setPSSize(s,s);
		aux.setSize(s,s);
		cont.add(aux);
	}

	private void addRowSep(LwContainer cont) {
		addRowSep(10, cont);
	}

	private void addSmallRowSep(LwContainer cont) {
		addRowSep(5, cont);
	}

	private LwCompList getListWithAtts(FeatureAttributeDescriptor[] att_descs, boolean numericonly) {
		
		LwCompList listt = new LwCompList(); 
		for (int i=0; i<att_descs.length; i++) {
			
			if (numericonly) {
				if (DataUtils.isNumericType(att_descs[i].getType())) {
					listt.addItem(att_descs[i].getName());
				}
			} else {
				listt.addItem(att_descs[i].getName());
			}
		}
		if (listt.count() > 0) {
			listt.select(0);
		}
		return listt;
	}
	
	private LwCombo getRangeSymNumericAttsCombo(FeatureAttributeDescriptor[] att_descs) {
		
		if (rangeSymAttCombo == null) {
			rangeSymAttCombo = new LwCombo();
			LwCompList l = getListWithAtts(att_descs, true);
			rangeSymAttCombo.setList(l);
		}
		return rangeSymAttCombo;
	}

	private LwCombo getAllAttsCombo(FeatureAttributeDescriptor[] att_descs) {
		
		if (allAttCombo == null) {
			allAttCombo = new LwCombo();
			LwCompList l = getListWithAtts(att_descs, false);
			allAttCombo.setList(l);
		}
		return allAttCombo;
	}



	public void colorSelected(LwComponent comp, Color c) {
		comp.setBackground(c);
	}

	public String getTitle() {
		return Messages.getText("LayerSymbologyDialog.0");
	}
	
	public static long[] AVAILABLE_SCALES = {
		1000,
		2500,
		5000,
		10000,
		25000,
		50000,
		100000,
		250000,
		500000,
		1000000,
		2500000,
		5000000,
		10000000,
		25000000,
		50000000 };
	
	public int getScaleIndex(String str) {
		
		int len = AVAILABLE_SCALES.length;
		for (int i=0; i<len; i++) {
			if (str.compareToIgnoreCase(String.valueOf(AVAILABLE_SCALES[i])) == 0) {
				return i;
			}
		}
		logger.error("Scale not found: " + str);
		return 0;
	}
	
	public long getScaleForIndex(int i) {
		
		if ((i<0) || (i >= AVAILABLE_SCALES.length)) {
			logger.error("Bad scale index: " + i);
			return AVAILABLE_SCALES[0];
		}
		return AVAILABLE_SCALES[i];
		
	}
	
	public long getSelectedScale(LwAList li) {
		int ind = li.getSelectedIndex();
		return getScaleForIndex(ind);
	}
	
	public void setSelectedScale(LwAList li, long sc) {
		int i = getScaleIndex(String.valueOf(sc));
		li.select(i);
	}

}
