/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.extension.base.layer.raster;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;

import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.fmap.geom.primitive.impl.Envelope2D;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.fmap.mapcontext.layers.FLyrDefault;
import org.gvsig.metadata.exceptions.MetadataException;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class FLayerGenericRaster extends FLyrDefault {
	
	private static final Logger logger = LoggerFactory.getLogger(FLayerGenericRaster.class);
	public static final String DYNCLASS_NAME = "FLayerGenericRaster_Persistent";
	private RasterFileAdapter adapter = null;
	
	public void doDispose() {
		
	}
	

	public FLayerGenericRaster() {
//		id = System.currentTimeMillis();
	}

	public Envelope getFullEnvelope() {
		Rectangle2D fext = adapter.getFullExtent();
		return new Envelope2D(fext.getMinX(), fext.getMinY(), fext
				.getMaxX(), fext.getMaxY());
	}
	
	public void draw(java.awt.image.BufferedImage img,
			java.awt.Graphics2D g,
			org.gvsig.fmap.mapcontext.ViewPort v,
			org.gvsig.tools.task.Cancellable c,
			double d) {
		adapter.draw(g, v);
		
	}

	public void draw(Graphics2D g, ViewPort viewPort) {

//		if (isWithinScale(viewPort.getScale())) {
//			adapter.draw(g, viewPort);
//		}
	}

	public boolean isWritable() {
		// TODO Auto-generated method stub
		return false;
	}

	public RasterAdapter getSource() {
		return adapter;
	}

	public void setSource(RasterFileAdapter ra) {
		adapter = ra;
	}

	public boolean isTiled() {
		// TODO Auto-generated method stub
		return false;
	}

	public int[] getTileSize() {
		// TODO Auto-generated method stub
		return null;
	}
	
	public void wakeUp(boolean driver_is_already_open) {
	}

	public long getId() {
		return 0;
	}	
	
	public String getFilePath() {
		return adapter.getDriver().getFilePath();
	}
	
	public void clean() {
		logger.debug("Cleaning layer: " + getName());
		try {
			adapter.getDriver().close();
			adapter.setDriver(null);
		} catch (IOException e) {
			logger.error("While closing raster driver: " + e.getMessage());
		}
		adapter = null;
	}

	public String getCustomForm() {
		// TODO Auto-generated method stub
		return null;
	}

	public void setCustomFormActive(boolean state) {
		// TODO Auto-generated method stub
		
	}
	
	public String getMetadataName() {
		return null;
	}
	
	public Object getMetadataID() {
		return null;
	}

	public void print(java.awt.Graphics2D g,
			org.gvsig.fmap.mapcontext.ViewPort v,
			org.gvsig.tools.task.Cancellable c,
			double a, org.gvsig.compat.print.PrintAttributes p) {
		
	}
	
	// ================== persistence
	
	// =================================================
	
	public void loadFromState(PersistentState state) throws PersistenceException {
		
		super.loadFromState(state);
		
		try {
			RasterDriver drv = (RasterDriver) state.get("rasterDriver");
			String fpath = state.getString("filePath");
			
			File f = new File(fpath);
			
			RasterFileAdapter adapter = new RasterFileAdapter(f);
			adapter.setDriver(drv);
			setSource(adapter);
			adapter.start();
		} catch (Exception ex) {
			throw new PersistenceException(ex.getMessage());
		}
	}

	public void saveToState(PersistentState state) throws PersistenceException {
		
		super.saveToState(state);
		
		try {
			state.set("rasterDriver", adapter.getDriver());
			state.set("filePath", adapter.getFile().getAbsolutePath());
		} catch (Exception ex) {
			throw new PersistenceException(ex.getMessage());
		}
	}
	
	public DynStruct[] getMetadataChildren() throws MetadataException{
		return null;
	}

	public static void registerPersistent() {
		 DynStruct definition =
			 ToolsLocator.getPersistenceManager().addDefinition(FLayerGenericRaster.class,
					DYNCLASS_NAME,
					 "FLayerGenericRaster Persistent definition",
	                null,
	                null);
	      
		 	//definition.extend(PersistenceManager.PERSISTENCE_NAMESPACE,"FLyrDefault_Persistent");
		 	definition.addDynFieldString("filePath").setMandatory(true);
	        definition.addDynFieldObject("rasterDriver").setMandatory(true);	
	}

}
