/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.base.project.dialog;

import java.awt.Color;
import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Date;

import org.cresques.cts.IProjection;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.extension.base.layer.raster.Utils;
import org.gvsig.mobile.libProjection.impl.CRSResolutor;
import org.gvsig.mobile.libProjection.interfaces.CRSConstants;
import org.gvsig.mobile.libProjection.interfaces.ICRS;
import org.gvsig.mobile.project.Project;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.lw.ColorChooserDialog;
import org.gvsig.mobile.ui.lw.ColorChooserListener;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.FileChooserDialog;
import org.gvsig.mobile.ui.lw.FileChooserListener;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwBorderPan;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwNotebook;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwTextRender;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.event.LwActionListener;

public class ProjectPropertiesDialog extends OkCancelDialog implements LwActionListener,
FileChooserListener, ColorChooserListener {
	
	private LwNotebook mainTabbedPane;
	
//	private static Logger logger = Logger.getLogger(AppSettingsDialog.class);		
	
	// tab view ==================================================
	private LwCombo srsCombo;	
	private LwButton colorChooserButton;	
	private LwTextField minXText, maxXText, minYText, maxYText;	
	
	private LwTextField nameText;
	private LwTextField projectPathText;
	private LwButton projectPathButton;
	private LwTextField descText;
	// private LwTextField dateCreatText;
	private LwTextField dateModifText;

	private ServiceReference gvsigServRef;
	private IGvSigContext gvc;
	private MapControl mc = null;

	
	public ProjectPropertiesDialog(BundleContext bc, ServiceReference servref) {		
		super(false, bc);
		gvsigServRef = servref;
		loadValues(servref);
	}
	
	private void loadValues(ServiceReference sref) {
		
		gvc = (IGvSigContext)bc.getService(sref);
		mc = (MapControl) gvc.getMapControl();
		// ====================================
		File pfile = mc.getProject().getProjectFile();
		// pfile.
		String aux = "";
		if (pfile != null) {
			aux = pfile.getName();
			int lastp = aux.lastIndexOf(".");
			if (lastp != -1) aux = aux.substring(0, lastp);
		}
		getNameText().setText(aux);
		// =====================================
		aux = "";
		if (pfile != null) {
			aux = pfile.getParent();
		}
		getProjectPathtext().setText(aux);
		// =============================================================================
		getDescText().setText(mc.getProject().getDescription());
		// ==================================================
		long modidate = pfile.lastModified();
		getModifDateText().setText(Utils.dateToString(new Date(modidate)));
		// =============
		// ========
		IProjection iproj = mc.getViewPort().getProjection();
		LwAList li = getSrsCombo().getList();
		selectLabel(li, iproj.getAbrev());
		// =======================
		getBgColorChooser().setBackground(mc.getViewPort().getBackColor());
		double d = 0;
		Envelope env = mc.getViewPort().getAdjustedExtent();
		if (env != null) {
			int dec = 2;
			if (iproj.getAbrev().compareToIgnoreCase("epsg:4326") == 0) {
				dec = 6;
			}
			d = env.getMinimum(0);
			getMinXText().setText(Utils.getFormattedDouble(d, dec));
			d = env.getMinimum(1);
			getMinYText().setText(Utils.getFormattedDouble(d, dec));
			d = env.getMaximum(0);
			getMaxXText().setText(Utils.getFormattedDouble(d, dec));
			d = env.getMaximum(1);
			getMaxYText().setText(Utils.getFormattedDouble(d, dec));
		}
	}

	private void selectLabel(LwAList li, String abrev) {
		
		int count = li.count();
		for (int i=0; i<count; i++) {
			Object o = li.getItem(i);
			if (o instanceof LwLabel) {
				LwLabel lbl = (LwLabel) o;
				if (lbl.getText().compareToIgnoreCase(abrev) == 0) {
					li.select(i);
					return;
				}
			}
		}
		
		logger.warn("project srs not found in combo: " + abrev);
		if (abrev.compareToIgnoreCase("epsg:4326") != 0) {
			selectLabel(li, "epsg:4326");
		}
		
	}

	public LwPanel buildMainPanel() {				
//		setSize(240, 260);
		mainTabbedPane = new LwNotebook(LwToolkit.TOP);
//		mainTabbedPane.setSize(240, 260);		
		
		final LwTextRender tab1 = new LwTextRender("");
		tab1.setFont(DialogConstants.getDefaultFont());
		tab1.getTextModel().setText(Messages.getText("ProjectPropertiesDialog.12"));
		
		final LwTextRender tab2 = new LwTextRender("");
		tab2.setFont(DialogConstants.getDefaultFont());
		tab2.getTextModel().setText(Messages.getText("ProjectPropertiesDialog.13"));
		
		mainTabbedPane.addPage(tab1,
				getGeneralSettingsTab());
		mainTabbedPane.addPage(tab2, getViewSettingsTab());
		mainTabbedPane.setBackground(Color.white);
		return mainTabbedPane;
	}
	
	public LwComponent getGeneralSettingsTab() {
		LwConstraints rightC = DialogConstants.getTextFieldConstraints();
		LwConstraints leftC = DialogConstants.getLabelConstraints();	
		
		LwPanel resp = new LwPanel();		
		resp.setLwLayout(new LwBorderLayout());
		
		LwPanel topLeft = new LwPanel();
		topLeft.setLwLayout(new LwGridLayout(2, 1));
		
		LwPanel topRight = new LwPanel();
		topRight.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));		
		
		final LwLabel nameLabel = DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.6"));
		LwLabel pathLabel = DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.7"));
		LwLabel descLabel = DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.9"));
		LwLabel createLabel = DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.10"));
		LwLabel modifLabel = DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.11"));
		
		int width = (int)descLabel.getPreferredSize().width;
		int height = (int)descLabel.getPreferredSize().height;
		
		nameLabel.setPSSize(width, height);
		pathLabel.setPSSize(width, height);
		descLabel.setPSSize(width, height);
		createLabel.setPSSize(width, height);
		modifLabel.setPSSize(width, height);		
		
		topLeft.add(leftC, nameLabel);		
		
		topRight.add(rightC, getNameText());		
		
		topLeft.add(pathLabel);
		LwContainer c = new LwPanel();
		c.setLwLayout(new LwBorderLayout());

		// projectPathText = getProjectPathtext();
		c.add(LwBorderLayout.CENTER, getProjectPathtext());

		projectPathButton = getSearchProjectPathButton();
		c.add(LwBorderLayout.EAST, projectPathButton);
		topRight.add(rightC, c);
		
		LwPanel centerLeft = new LwPanel();
		centerLeft.setLwLayout(new LwGridLayout(1, 1));
		
		LwPanel centerRight = new LwPanel();
		centerRight.setLwLayout(new LwGridLayout(1, 1, LwToolkit.HORIZONTAL | LwToolkit.VERTICAL));
		
		centerLeft.add(leftC, descLabel);
		centerRight.add(getDescText());		
		
		LwPanel bottomLeft = new LwPanel();
		bottomLeft.setLwLayout(new LwGridLayout(2, 1));
		
		LwPanel bottomRight = new LwPanel();
		bottomRight.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));
		
		// bottomLeft.add(leftC, createLabel);
		// this.dateCreatText = DialogConstants.getDefaultTextField();		
		// bottomRight.add(rightC, this.dateCreatText);		
		
		bottomLeft.add(leftC, modifLabel);
		this.dateModifText = getModifDateText(); // 		
		bottomRight.add(rightC, this.dateModifText);
		
		LwPanel top = new LwPanel();
		top.setLwLayout(new LwBorderLayout());
		top.add(LwBorderLayout.WEST, topLeft);
		top.add(LwBorderLayout.CENTER, topRight);
		
		LwPanel center = new LwPanel();
		center.setLwLayout(new LwBorderLayout());
		center.add(LwBorderLayout.WEST, centerLeft);
		center.add(LwBorderLayout.CENTER, centerRight);
		
		LwPanel bot = new LwPanel();
		bot.setLwLayout(new LwBorderLayout());
		bot.add(LwBorderLayout.WEST, bottomLeft);
		bot.add(LwBorderLayout.CENTER, bottomRight);
		
		resp.add(LwBorderLayout.NORTH, top);
		resp.add(LwBorderLayout.CENTER, center);
		resp.add(LwBorderLayout.SOUTH, bot);
				
		return resp;
	}
	
	private LwTextField getModifDateText() {
		
		if (dateModifText == null) {
			dateModifText = DialogConstants.getDefaultTextField();
			dateModifText.setEditable(false);
		}
		return dateModifText;
	}

	private LwTextField getDescText() {
		
		if (descText == null) {
			descText = DialogConstants.getDefaultTextField();
		}
		return descText;
	}

	private LwTextField getNameText() {
		
		if (nameText == null) {
			nameText = DialogConstants.getDefaultTextField();
		}
		return nameText;
	}

	private LwComponent getViewSettingsTab() {
		LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwBorderLayout());
		
		LwConstraints l = DialogConstants.getTextFieldConstraints();		
		
		LwConstraints l1 = new LwConstraints();
		l1.fill = LwToolkit.NONE;
		l1.ax = LwToolkit.LEFT;
		
		LwConstraints l2 = DialogConstants.getLabelConstraints();
		
		LwPanel topLeft = new LwPanel();
		topLeft.setLwLayout(new LwGridLayout(2, 1));
		
		LwPanel topRight = new LwPanel();
		topRight.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));		
		
		topLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.1")));
		topRight.add(l, getSrsCombo());		
		
		topLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.5")));
		topRight.add(l1, getBgColorChooser());		
		
		LwBorderPan center = new LwBorderPan();
		center.add(LwBorderPan.TITLE, DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.2")));

		LwPanel centerLeft = new LwPanel();
		centerLeft.setLwLayout(new LwGridLayout(2, 1));
		
		LwPanel centerRight = new LwPanel();
		centerRight.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));
		
		centerLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.3")));
		LwPanel e1 = new LwPanel();
		e1.setLwLayout(new LwGridLayout(1, 2, LwToolkit.HORIZONTAL));
		
		e1.add(l, getMinXText());
		e1.add(l, getMaxXText());
		
		centerRight.add(e1);
		
		centerLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("ProjectPropertiesDialog.4")));
		LwPanel e2 = new LwPanel();
		e2.setLwLayout(new LwGridLayout(1, 2, LwToolkit.HORIZONTAL));
		
		e2.add(l, getMinYText());
		e2.add(l, getMaxYText());

		centerRight.add(e2);
		
		LwPanel top = new LwPanel();
		top.setLwLayout(new LwBorderLayout());
		top.add(LwBorderLayout.WEST, topLeft);
		top.add(LwBorderLayout.CENTER, topRight);
		
		LwPanel cent = new LwPanel();
		cent.setLwLayout(new LwBorderLayout());
		cent.add(LwBorderLayout.WEST, centerLeft);
		cent.add(LwBorderLayout.CENTER, centerRight);
		
		center.add(LwBorderPan.CENTER, cent);
		
		resp.add(LwBorderLayout.NORTH, top);
		resp.add(LwBorderLayout.CENTER, center);

		return resp;
	}
	
	private LwButton getBgColorChooser() {
		
		if (colorChooserButton == null) {
			colorChooserButton = new LwButton();
			
			colorChooserButton = DialogConstants.getButton("", "", this);
			int size = (int)this.getSrsCombo().getPreferredSize().height;		
			colorChooserButton.setPSSize(size, size);
			colorChooserButton.getViewMan(true).setBorder("br.plain");
			colorChooserButton.setOpaque(true);
			colorChooserButton.addActionListener(this);
		}
		return colorChooserButton;
	}
	
	private LwButton getSearchProjectPathButton() {
		return DialogConstants.getButton("carpeta", "carpeta", this);		
	}

	private LwTextField getProjectPathtext() {
		
		if (projectPathText == null) {
			projectPathText = DialogConstants.getDefaultTextField();
			int h = projectPathText.getHeight();
			int w = projectPathText.getWidth();
			projectPathText.setPSSize(w, 2*h);
			projectPathText.setSize(w, 2*h);
			
			projectPathText.setEditable(false);
		}
		return projectPathText;
	}

	private LwCombo getSrsCombo() {
		
		if (srsCombo == null) {
			srsCombo = new LwCombo();
			LwCompList l = new LwCompList();
			
			ArrayList crsl = CRSResolutor.getCRSs(getBundleContext());
			ICRS icr = null;
			String abrev;
			for (int i=0; i<crsl.size(); i++) {
				icr = (ICRS) crsl.get(i);
				abrev = (String) icr.getProps().get(CRSConstants.CRS_ABREV);
				l.addItem(DialogConstants.getDefaultLabel(abrev));
			}
			srsCombo.setList(l);
		}
		
		return srsCombo;
	}

	private LwTextField getMinXText() {
		if (minXText == null) {
			// minXText, maxXText, minYText, maxYText
			minXText = DialogConstants.getDefaultTextField();
			minXText.setPSSize(20, -1);
			minXText.setEditable(false);
		}
		return minXText;
	}

	private LwTextField getMaxXText() {
		if (maxXText == null) {
			// minXText, maxXText, minYText, maxYText
			maxXText = DialogConstants.getDefaultTextField();
			maxXText.setPSSize(20, -1);
			maxXText.setEditable(false);
		}
		return maxXText;
	}

	private LwTextField getMinYText() {
		if (minYText == null) {
			// minXText, maxXText, minYText, maxYText
			minYText = DialogConstants.getDefaultTextField();
			minYText.setPSSize(20, -1);
			minYText.setEditable(false);
		}
		return minYText;
	}

	private LwTextField getMaxYText() {
		if (maxYText == null) {
			// minXText, maxXText, minYText, maxYText
			maxYText = DialogConstants.getDefaultTextField();
			maxYText.setPSSize(20, -1);
			maxYText.setEditable(false);
		}
		return maxYText;
	}

	public void actionPerformed(Object src, Object data) {
		if (src == this.projectPathButton) {
			final FileChooserDialog f = new FileChooserDialog(this, null, null, FileChooserDialog.DIALOG_TYPE_SELECT);			
			f.setTitle(Messages.getText("ProjectPropertiesDialog.23"));
			f.showDialog();
		} else if (src == this.colorChooserButton) {
			ColorChooserDialog c = new ColorChooserDialog(colorChooserButton, this);			
			c.showDialog();
		} else if (src == this.getOkButton()) {
			
			storeValues();
			WindowManager.getInstance().closeActivePanel();
		} else if (src == this.getCancelButton()) {
			WindowManager.getInstance().closeActivePanel();
		}
	}

	private void storeValues() {
		
		BundleContext the_bc = getBundleContext();
		IGvSigContext gvco = (IGvSigContext) the_bc.getService(gvsigServRef);
		MapControl mcon = (MapControl) gvco.getMapControl();

		LwLabel sellbl = (LwLabel) getSrsCombo().getList().getSelected();
		String selcrsabrev = sellbl.getText();
		IProjection ipro = null;
		try {
			// project projection
			ipro = CRSResolutor.getCRS(selcrsabrev, getBundleContext());
			mcon.getMapContext().setProjection(ipro);
		} catch (Exception ex) {
			logger.error("While getting SRS to store: " + ex.getMessage());
		}
		// description
		mcon.getProject().setDescription(getDescText().getText());
		mcon.setBackColor(getBgColorChooser().getBackground());
		File outf = new File(this.getProjectPathtext().getText() + File.separator + this.getNameText().getText() + ".gvm");
		mcon.getProject().setProjectFile(outf);
		saveProject(mcon.getProject());
	}

	private void saveProject(Project pr) {
		
		PersistenceManager pma = ToolsLocator.getPersistenceManager();
		
		try {
			PersistentState psta = pma.getState(pr); 
			FileOutputStream fos = new FileOutputStream(pr.getProjectFile());
			pma.saveState(psta, fos);
			fos.close();
		} catch (Exception ex) {
			logger.error("While storing project: " + ex.getMessage());
		}
		
	}

	public void fileSelected(String path) {
		getProjectPathtext().setText(path);		
	}
	
	public void colorSelected(LwComponent refcomp, Color c) {
//		colorLabel.setBackground(c);
		if(refcomp.isOpaque()) {
			// System.out.println("isopaque");
		}
		refcomp.setBackground(c);
		// System.out.println(refcomp.getBackground().toString());
	}

	public String getTitle() {
		return Messages.getText("ProjectPropertiesDialog.0");
	}
}


