package org.gvsig.mobile.extension.editing.dialog;

import java.awt.Color;

import org.gvsig.fmap.dal.DataTypes;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.tools.dispose.DisposableIterator;
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureSelection;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.zaval.data.Matrix;
import org.zaval.data.MatrixModel;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCanvas;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.grid.LwGrid;
import org.zaval.lw.grid.LwGridCaption;
import org.zaval.lw.misc.LwGridCompEditor;
import org.zaval.lw.misc.LwGridCompView;

/**
 * @author Vicente Caballero Navarro
 */
public class ModifyEditingTableDialog extends OkCancelDialog {
	private LwGrid grid;
	private FLyrVect layer;
	private LwButton deleteButton;
	private LwButton updateButton;
	private int sel = -1;

	// private LwCombo addType;
	// private ArrayList properties=new ArrayList();

	public ModifyEditingTableDialog(boolean scrollable, BundleContext bc,
			FLyrVect layer) {
		super(scrollable, bc);
		this.getCancelButton().setVisible(false);
		this.layer = layer;
		super.add(LwBorderLayout.CENTER, this.buildMainPanel());
	}

	public void actionPerformed(Object src, Object data) {
		if (src == this.getOkButton()) {
			WindowManager.getInstance().closeActivePanel();
		} else if (src == deleteButton) {
			try {
				FeatureStore featureStore = layer.getFeatureStore();
				FeatureSelection selection = (FeatureSelection) featureStore
						.getFeatureSelection();
				DisposableIterator iterator = selection.iterator();
				while (iterator.hasNext()) {
					Feature feature = (Feature) iterator.next();
					featureStore.delete(feature);
				}
			} catch (DataException e) {
				e.printStackTrace();
			}
			super.add(LwBorderLayout.CENTER, this.buildMainPanel());
		} else if (src == updateButton) {
			if (sel == -1) {
				return;
			}
			try {
				FeatureStore featureStore = layer.getFeatureStore();
				FeatureType featureType = featureStore.getDefaultFeatureType();
				FeatureSelection selection = (FeatureSelection) featureStore
						.getFeatureSelection();
				DisposableIterator iterator = selection.iterator();
				while (iterator.hasNext()) {
					Feature feature = (Feature) iterator.next();
					EditableFeature eFeature = feature.getEditable();
					FeatureAttributeDescriptor[] descriptors = featureType
							.getAttributeDescriptors();
					for (int i = 0; i < descriptors.length; i++) {
						FeatureAttributeDescriptor des = descriptors[i];
						if (des.getDataType().getType() == DataTypes.GEOMETRY) {
							continue;
						}
						MatrixModel model = grid.getModel();
						Object obj = model.get(sel, i);
						Object value = getValueFromLw(des.getDataType().getType(), obj);
						eFeature.set(des.getName(), value);
					}
					featureStore.update(eFeature);
				}
				super.add(LwBorderLayout.CENTER, this.buildMainPanel());
			} catch (DataException e) {
				e.printStackTrace();
			}
		}
	}

	private Object getValueFromLw(int dataType, Object obj) {
		switch (dataType) {
		case DataTypes.BOOLEAN:
			return new Boolean(((LwCheckbox) obj).getState());
		case DataTypes.DOUBLE:
			return new Double(((LwTextField) obj).getText());
		case DataTypes.FLOAT:
			return new Float(((LwTextField) obj).getText());
		case DataTypes.LONG:
			return new Long(((LwTextField) obj).getText());
		case DataTypes.INT:
			return new Integer(((LwTextField) obj).getText());
		}
		return ((LwTextField) obj).getText();
	}

	private LwGrid getGrid() throws DataException {
		FeatureStore featureStore = layer.getFeatureStore();
		FeatureType featureType = featureStore.getDefaultFeatureType();
		Matrix m = new Matrix(2, 2);

		grid = new LwGrid(m) {
			public void mousePressed(LwMouseEvent event) {
				super.mousePressed(event);
				sel = grid.getPosController().getCurrentLine();
				if (sel == -1) {
					return;
				}

				FeatureStore featureStore = layer.getFeatureStore();
				try {
					FeatureSet set = featureStore.getFeatureSet();
					DisposableIterator iterator = set.iterator(sel);
					if (iterator.hasNext()) {
						FeatureSelection selection = (FeatureSelection) featureStore
								.getFeatureSelection();
						selection.deselectAll();
						selection.select((Feature) iterator.next());
					}
				} catch (DataException e) {
					e.printStackTrace();
				}
			}
		};
		LwGridCompEditor editors = new LwGridCompEditor();
		grid.setEditorProvider(editors);
		grid.setViewProvider(new LwGridCompView());
		// grid.setEditorProvider(new LwDefEditors());
		LwGridCaption cap2 = new LwGridCaption(grid);
		FeatureAttributeDescriptor[] descriptors = featureType
				.getAttributeDescriptors();

		for (int i = 0; i < descriptors.length; i++) {
			FeatureAttributeDescriptor descriptor = descriptors[i];
			if (descriptor.getDataType().getType() == DataTypes.GEOMETRY)
				continue;
			cap2.putTitle(i, descriptor.getName());
		}

		grid.setBackground(Color.white);
		grid.add(LwGrid.TOP_CAPTION_EL, cap2);
		grid.usePsMetric(true);
		grid.setColor(LwGrid.SEL2_COLOR, Color.YELLOW);
		grid.setColor(LwGrid.SEL1_COLOR, Color.WHITE);
		grid.setColor(LwGrid.NET_COLOR, Color.WHITE);

		grid.setBackground(Color.white);

		FeatureSet set = featureStore.getFeatureSet();
		DisposableIterator iterator = set.iterator();
		int pos = 0;
		while (iterator.hasNext()) {
			Feature feature = (Feature) iterator.next();
			for (int i = 0; i < descriptors.length; i++) {
				FeatureAttributeDescriptor descriptor = descriptors[i];
				if (descriptor.getDataType().getType() == DataTypes.GEOMETRY)
					continue;
				LwComponent c1 = null;
				if (descriptor.getDataType().getType() == DataTypes.BOOLEAN) {
					c1 = new LwCheckbox();
					((LwCheckbox) c1).setState(((Boolean) feature.get(i))
							.booleanValue());
				} else {
					c1 = new LwTextField();
					((LwTextField) c1).setText(feature.get(i).toString());
					((LwTextField) c1).setPSSize(120, 30);
				}
				c1.setBackground(Color.white);
				m.put(pos, i, c1);
			}
			pos++;
		}
		iterator.dispose();
		return grid;
	}

	public LwPanel buildMainPanel() {
		LwContainer toolbar1 = new LwPanel();
		toolbar1.setBackground(Color.white);
		toolbar1.setLocation(20, 50);
		LwFlowLayout toolbar1Layout = new LwFlowLayout(LwToolkit.LEFT,
				LwToolkit.TOP, LwToolkit.HORIZONTAL, 1);
		toolbar1.setLwLayout(toolbar1Layout);

		LwPanel resp = new LwPanel();
		if (layer == null)
			return resp;
		resp.setLwLayout(new LwBorderLayout());
		try {
			grid = getGrid();
		} catch (DataException e) {
			e.printStackTrace();
		}
		deleteButton = DialogConstants.getButton("eraser", "eraser", this);
		updateButton = DialogConstants.getButton("options", "options", this);
		LwGridCaption verSnapping = new LwGridCaption(grid, LwToolkit.VERTICAL);
		grid.add(LwGrid.LEFT_CAPTION_EL, verSnapping);
		LwCanvas stubSnapping = new LwCanvas();
		stubSnapping.getViewMan(true).setBorder("br.raised");
		grid.add(LwGrid.STUB_EL, stubSnapping);
		int count = grid.getModel().getRows();
		for (int i = 0; i < count; i++)
			verSnapping.putTitle(i, "" + i);

		LwScrollPan sc1 = new LwScrollPan(grid);
		sc1.setIncrements(30, 50, 30, 50);
		resp.add(LwBorderLayout.CENTER, sc1);
		toolbar1.add(this.deleteButton);
		toolbar1.add(this.updateButton);
		LwLabel label=new LwLabel(Messages.getText("rows") + ": " + count);
		label.setBackground(Color.white);
		toolbar1.add(label);
		resp.add(LwBorderLayout.SOUTH, toolbar1);

		return resp;
	}

	public String getTitle() {
		return Messages.getText("ModifyEditingTableDialog.0");
	}
}
