package org.gvsig.mobile.extension.editing.dialog;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Iterator;

import org.gvsig.fmap.dal.DataTypes;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.EditableFeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.EditableFeatureType;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dataTypes.DataType;
import org.osgi.framework.BundleContext;
import org.zaval.data.Matrix;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCanvas;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwList;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.grid.LwGrid;
import org.zaval.lw.grid.LwGridCaption;
import org.zaval.lw.misc.LwGridCompEditor;
import org.zaval.lw.misc.LwGridCompView;

/**
 * @author Vicente Caballero Navarro
 */
public class ModifyFieldTableDialog extends OkCancelDialog {
    private LwGrid      grid;
//    private LwTextField cache;
    private FLyrVect layer;
    private LwButton deleteButton;
    private LwButton addButton;
    private LwCombo addType;
    private ArrayList properties=new ArrayList();
    // private LwGrid gLayers;
    
    public ModifyFieldTableDialog(boolean scrollable, BundleContext bc, FLyrVect layer) {
        super(scrollable, bc);
        this.getCancelButton().setVisible(true);
        this.layer=layer;
        super.add(LwBorderLayout.CENTER, this.buildMainPanel());
    }
    
    public void actionPerformed(Object src, Object data) {
        if (src == this.getOkButton()) {
        	try {
        		FeatureStore featureStore=layer.getFeatureStore();
				FeatureType featureType=featureStore.getDefaultFeatureType();
				EditableFeatureType eFeatureType=featureType.getEditable(); 
				FeatureAttributeDescriptor[] descriptors=featureType.getAttributeDescriptors();
			        
			        for (int i = 0; i < descriptors.length; i++) {
			            FeatureAttributeDescriptor descriptor=descriptors[i];
			            if (descriptor.getDataType().getType()==DataTypes.GEOMETRY)
			            	continue;
			            PropertyTable pt=(PropertyTable)properties.get(i);
			            boolean change=false;
			            if (!descriptor.getName().equals(pt.name.getText())){
			            	change=true;
			            	
			            }
//			            if (!descriptor.getDataTypeName().equals(pt.type.getList().getSelected())){
//			            	change=true;
//			            }
			            if (descriptor.getSize()!=Integer.parseInt(pt.size.getText())){
			            	change=true;
			            } 
			            if (descriptor.getPrecision()!=Integer.parseInt(pt.precision.getText())){
			            	change=true;
			            }
			            if (descriptor.getDefaultValue()!=null && !descriptor.getDefaultValue().toString().equals(pt.valueDef.getText())){
			            	change=true;
			            }
			            if (change){
			            	eFeatureType.remove(descriptor.getName());
			            	EditableFeatureAttributeDescriptor ead = eFeatureType.add(pt.name.getText(), descriptor.getDataType().getType(), Integer.parseInt(pt.size.getText()));
			    			ead.setPrecision(Integer.parseInt(pt.precision.getText()));
			            }
			        }
			        featureStore.update(eFeatureType);
        	} catch (DataException e) {
				e.printStackTrace();
			}
            WindowManager.getInstance().closeActivePanel();
        }else if (src == this.getCancelButton()){
        	WindowManager.getInstance().closeActivePanel();
        } else if (src == deleteButton) {
        	try {
        		int sel=grid.getPosController().getCurrentLine();
        		if (sel==-1){
        			return;
        		}
        		FeatureStore featureStore=layer.getFeatureStore();
        		FeatureType featureType=featureStore.getDefaultFeatureType();
        		EditableFeatureType eFeatureType=featureType.getEditable(); 
        		eFeatureType.remove(grid.getPosController().getCurrentLine());
        		featureStore.update(eFeatureType);
        	} catch (DataException e) {
				e.printStackTrace();
			}
			super.add(LwBorderLayout.CENTER, this.buildMainPanel());
		} else if (src == addButton){
			try {
				FeatureStore featureStore=layer.getFeatureStore();
				FeatureType featureType=featureStore.getDefaultFeatureType();
				EditableFeatureType eFeatureType=featureType.getEditable(); 
				int n=1;
				String newField=Messages.getText("field")+n;
				while (featureType.get(newField)!=null) {
					n++;
					newField=Messages.getText("field")+n;
				}
				int sel=0;
				
				Iterator typesIt = ToolsLocator.getDataTypesManager().iterator();
				while(typesIt.hasNext()){
					DataType type = (DataType) typesIt.next();
					if (addType.getList().getSelected().equals(type.getName())){
						sel=type.getType();
					}
				}
				
//				for (int i = 0; i < DataTypes.TYPE_NAMES.length; i++) {
//					if (addType.getList().getSelected().equals(DataTypes.TYPE_NAMES[i])){
//						sel=i;
//					}
//				}
				eFeatureType.add(newField, sel, 50);
				featureStore.update(eFeatureType);
				super.add(LwBorderLayout.CENTER, this.buildMainPanel());
			} catch (DataException e) {
				e.printStackTrace();
			}
		}
    }
    
    private LwGrid getGrid() throws DataException {
    	FeatureType featureType=layer.getFeatureStore().getDefaultFeatureType();
    	Matrix m = new Matrix(2, 2);
        
        grid = new LwGrid(m);
        // if (layers == null)
        // return g;
        LwGridCompEditor editors = new LwGridCompEditor();
        grid.setEditorProvider(editors);
        grid.setViewProvider(new LwGridCompView());
        
        LwGridCaption cap2 = new LwGridCaption(grid);
        cap2.putTitle(0, Messages.getText("ModifyTableDialog.1"));
        cap2.putTitle(1, Messages.getText("ModifyTableDialog.2"));
        cap2.putTitle(2, Messages.getText("ModifyTableDialog.3"));
        cap2.putTitle(3, Messages.getText("ModifyTableDialog.4"));
        cap2.putTitle(4, Messages.getText("ModifyTableDialog.5"));
        
        grid.setBackground(Color.white);
        grid.add(LwGrid.TOP_CAPTION_EL, cap2);
        grid.usePsMetric(true);
        // g.getPosController().setOffset(10);
        grid.setColor(LwGrid.SEL2_COLOR, Color.YELLOW);
        grid.setColor(LwGrid.SEL1_COLOR, Color.WHITE);
        grid.setColor(LwGrid.NET_COLOR, Color.WHITE);
        
        grid.setBackground(Color.white);
        
        FeatureAttributeDescriptor[] descriptors=featureType.getAttributeDescriptors();
        
        for (int i = 0; i < descriptors.length; i++) {
            FeatureAttributeDescriptor descriptor=descriptors[i];
            if (descriptor.getDataType().getType()==DataTypes.GEOMETRY)
            	continue;
            LwTextField c1 = new LwTextField();
            c1.setText(descriptor.getName());
            c1.setPSSize(160, 30);
            c1.setBackground(Color.white);
            m.put(i, 0, c1);
            
            LwLabel c2 = new LwLabel();
//            LwList list=new LwList();
//            String[] names=DataTypes.TYPE_NAMES;
//            int sel=0;
//            for (int j = 0; j < names.length; j++) {
//            	list.addItem(names[j]);
//            	if (descriptor.getDataTypeName().equals(names[j])){
//            		sel=j;
//            	}
//			}
//            list.select(sel);
//            c2.setList(list);
            c2.setText(descriptor.getDataTypeName());
            c2.setPSSize(120, 30);
            c2.setBackground(Color.white);
            m.put(i, 1, c2);
            
            LwTextField c3 = new LwTextField();
            c3.setText(String.valueOf(descriptor.getSize()));
            c3.setPSSize(100, 30);
            c3.setBackground(Color.white);
            m.put(i, 2, c3);
       
            LwTextField c4 = new LwTextField();
            c4.setText(String.valueOf(descriptor.getPrecision()));
            c4.setPSSize(100, 30);
            c4.setBackground(Color.white);
            m.put(i, 3, c4);
            
            LwTextField c5 = new LwTextField();
            Object defaultValue=descriptor.getDefaultValue();
            if (defaultValue!=null){
            	c5.setText(defaultValue.toString());
            }
            c5.setPSSize(120, 30);
            c5.setBackground(Color.white);
            m.put(i, 4, c5);
            properties.add(new PropertyTable(c1,c3,c4,c5));
        }
        return grid;
    }
    public LwPanel buildMainPanel() {
    	LwContainer toolbar1 = new LwPanel();
		toolbar1.setBackground(Color.white);
		toolbar1.setLocation(20, 50);
		LwFlowLayout toolbar1Layout = new LwFlowLayout(LwToolkit.LEFT,
				LwToolkit.TOP, LwToolkit.HORIZONTAL, 1);
		toolbar1.setLwLayout(toolbar1Layout);
        
        LwPanel resp = new LwPanel();
        if (layer==null)
        	return resp;
        resp.setLwLayout(new LwBorderLayout());
        try {
            grid = getGrid();
        } catch (DataException e) {
            e.printStackTrace();
        }
        deleteButton = DialogConstants.getButton("eraser",
				"eraser", this);
        addButton = DialogConstants.getButton("add", 
        		"add", this);
        addType=new LwCombo();
        LwList list=new LwList();
        
        Iterator typesIt = ToolsLocator.getDataTypesManager().iterator();
       	while(typesIt.hasNext()){
			DataType type = (DataType) typesIt.next();
			list.addItem(type.getName());
		}
//        String[] names=DataTypes.TYPE_NAMES;
//        for (int j = 0; j < names.length; j++) {
//        	list.addItem(names[j]);
//        }
        list.select(DataTypes.STRING);
        addType.setList(list);
        addType.setPSSize(120, 30);
        addType.setBackground(Color.white);
        LwGridCaption verSnapping = new LwGridCaption(grid,
                LwToolkit.VERTICAL);
        grid.add(LwGrid.LEFT_CAPTION_EL, verSnapping);
        LwCanvas stubSnapping = new LwCanvas();
        stubSnapping.getViewMan(true).setBorder("br.raised");
        grid.add(LwGrid.STUB_EL, stubSnapping);
        for (int i = 0; i < grid.getModel().getRows(); i++)
            verSnapping.putTitle(i, "" + i);
        
        LwScrollPan sc1 = new LwScrollPan(grid);
        sc1.setIncrements(30, 50, 30, 50);
        resp.add(LwBorderLayout.CENTER, sc1);
        toolbar1.add(this.deleteButton);
        toolbar1.add(this.addButton);
        toolbar1.add(this.addType);
        resp.add(LwBorderLayout.SOUTH, toolbar1);
        
        return resp;
    }
    
    public String getTitle() {
        return Messages.getText("ModifyTableDialog.0");
    }
    class PropertyTable{
    	private LwTextField name;
//    	private LwCombo type;
    	private LwTextField size;
    	private LwTextField precision;
    	private LwTextField valueDef;
    	
    	public PropertyTable(LwTextField name, LwTextField size, LwTextField precision, LwTextField valueDef) {
			this.name=name;
//			this.type=type;
			this.size=size;
			this.precision=precision;
			this.valueDef=valueDef;
		}
    }
}
