package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.Point2D;
import java.util.ArrayList;

import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.tools.locator.LocatorException;

/**
 * @author Vicente Caballero Navarro
 */
public class AddPolilineFunc extends EditingFunctionality implements InsertablePoint{
    protected ArrayList points = new ArrayList();
    protected Geometry  geom   = null;
    
    public AddPolilineFunc() {
        super();
    }
    
    public void configure() {
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
        int type = event.getID();
        
        if (type == GvSigMouseEventType.MOUSE_PRESSED) {
            Point2D point = getHandlerAdjustedPoint();
            addPoint(point);
        }
    }
    
    public void stop() {
        super.stop();
        if (geom!=null)
        	insertGeometry(geom);
        points.clear();
    }
    
    protected GeneralPathX createGPXLine(ArrayList points) {
        GeneralPathX gpx = new GeneralPathX();
        for (int i = 0; i < points.size(); i++) {
            Point2D p = (Point2D) points.get(i);
            if (i == 0)
                gpx.moveTo(p.getX(), p.getY());
            else
                gpx.lineTo(p.getX(), p.getY());
        }
        return gpx;
    }
    
    public boolean isApplicable() {
        FLyrVect lv = getEditingLayer();
        try {
            int type = lv.getShapeType();
            if (
            		type == Geometry.TYPES.CURVE ||
            		type == Geometry.TYPES.MULTICURVE ||
            		type == Geometry.TYPES.GEOMETRY) {
                return true;
            }
        } catch (ReadException e) {
            e.printStackTrace();
        }
        return false;
    }

	public void addPoint(Point2D point) {
		 points.add(point);
         GeneralPathX gpx = createGPXLine(points);
         try {
             geom = geomManager.createCurve(gpx, Geometry.SUBTYPES.GEOM2D);
             drawGeometry(geom);
         } catch (LocatorException e) {
             e.printStackTrace();
         } catch (CreateGeometryException e) {
             e.printStackTrace();
         }
	}
}
