package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.Point2D;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.tools.dispose.DisposableIterator; 
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureSelection;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.util.UtilFunctions;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;

/**
 * @author Vicente Caballero Navarro
 */
public class ModifyScaleFunc extends SelectionFunc implements InsertablePoint{
    private Point2D firstPoint;
    private Point2D lastPoint;
    private Point2D scalePoint;
    
    public void configure() {
        // TODO Auto-generated method stub
        
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
    	int type = event.getID();
    	if (type == GvSigMouseEventType.MOUSE_PRESSED) {
    		Point2D point = getHandlerAdjustedPoint();
    		addPoint(point);
    	}
    }
    private void scale(double scaleFactor) throws ReadException {
        DisposableIterator iterator = null;
        try {
            FeatureStore featureStore = getEditingLayer().getFeatureStore();

            featureStore.beginEditingGroup(getName());
            iterator = ((FeatureSelection) featureStore.getSelection())
                    .iterator();
            while (iterator.hasNext()) {
                Feature feature = (Feature) iterator.next();
                Geometry geometry = (feature.getDefaultGeometry())
                        .cloneGeometry();
                UtilFunctions.scaleGeom(geometry, scalePoint, scaleFactor,
                        scaleFactor);

                EditableFeature eFeature = feature.getEditable();
                eFeature.setGeometry(featureStore.getDefaultFeatureType()
                        .getDefaultGeometryAttributeName(), geometry);
                featureStore.update(eFeature);
            }
            featureStore.endEditingGroup();
            repaintMap();
        } catch (DataException e) {
            e.printStackTrace();
        } finally {
            if (iterator != null) {
                iterator.dispose();
            }
        }
    }

    public void addPoint(Point2D point) {
    	try{
    		FeatureStore featureStore=getEditingLayer().getFeatureStore();
    		FeatureSelection selection=featureStore.getFeatureSelection();
    		if(selection.isEmpty()){
    			selectWithPoint(point);
    		}else{
    			if (firstPoint==null){
    				firstPoint = point;
    				scalePoint = firstPoint;
    			} else {
    				lastPoint = point;

    				try {
    					double size = getMapControl().getViewPort()
    					.toMapDistance(
    							getMapControl().getWidth());
    					scale(firstPoint.distance(lastPoint) / (size / 40));
    					firstPoint=null;
    				} catch (ReadException e) {
    					e.printStackTrace();
    				}
    			}
    		}
    	} catch (DataException e1) {
    		e1.printStackTrace();
    	}
    }

}
