package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.Point2D;
import java.util.ArrayList;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.tools.dispose.DisposableIterator; 
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.geom.util.Converter;
import org.gvsig.fmap.mapcontext.layers.SpatialCache;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.extension.editing.functionality.split.SplitStrategy;
import org.gvsig.tools.dispose.DisposeUtils;
import org.gvsig.tools.locator.LocatorException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.geom.PrecisionModel;

/**
 * @author Vicente Caballero Navarro
 */
public class ModifySplitFunc extends SelectionVertexFunc implements InsertablePoint{
	protected ArrayList points = new ArrayList();
	protected Geometry  geom   = null;
	    
    public void configure() {
        // TODO Auto-generated method stub
        
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
    	int type = event.getID();
    	if (type == GvSigMouseEventType.MOUSE_PRESSED) {
    		FeatureStore featureStore=getEditingLayer().getFeatureStore();
    		try {
				if (featureStore.getFeatureSelection().isEmpty()){
					Point2D point = getMapPoint(event.getX(), event.getY());
					selectWithPoint(point);
				} else {
					Point2D point = getHandlerAdjustedPoint();
					addPoint(point);
				}
			} catch (DataException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    	}
    }
    
    public boolean isApplicable() {
        FLyrVect lv = getEditingLayer();
        try {
            int type = lv.getShapeType();
            if (type == Geometry.TYPES.CURVE || type == Geometry.TYPES.SURFACE
                    || type == Geometry.TYPES.GEOMETRY) {
                return true;
            }
        } catch (ReadException e) {
            e.printStackTrace();
        }
        return false;
    }

    public void addPoint(Point2D point) {
		 points.add(point);
        GeneralPathX gpx = createGPXLine(points);
        try {
            geom = geomManager.createCurve(gpx, Geometry.SUBTYPES.GEOM2D);
            drawGeometry(geom);
        } catch (LocatorException e) {
            e.printStackTrace();
        } catch (CreateGeometryException e) {
            e.printStackTrace();
        }
	}
    public void stop() {
        super.stop();
        if (geom!=null)
        	splitSelectedGeometryWithDigitizedLine();
        points.clear();
    }
    
    public void splitSelectedGeometryWithDigitizedLine(){
		Coordinate[] digitizedCoords = getPoint2DAsCoordinates((Point2D[])points.toArray(new Point2D[0]));
		LineString splittingLs = new GeometryFactory(
				new PrecisionModel(10000)).createLineString(digitizedCoords);
		DisposableIterator selected = null;
		try {
			FeatureStore store=getEditingLayer().getFeatureStore();
			selected = store.getFeatureSelection().iterator();

//			getCadToolAdapter().getMapControl().getMapContext().beginAtomicEvent();
			store.beginEditingGroup(getName());
			while (selected.hasNext()) {
				Feature feature = (Feature) selected.next();
				org.gvsig.fmap.geom.Geometry ig = feature.getDefaultGeometry();
				com.vividsolutions.jts.geom.Geometry jtsGeo = Converter.geometryToJts(ig);
				if (jtsGeo==null)
					return;
				com.vividsolutions.jts.geom.Geometry splitGeo = SplitStrategy.splitOp(jtsGeo, splittingLs);
				if(splitGeo instanceof GeometryCollection
						&& ((GeometryCollection)splitGeo).getNumGeometries()>1){

					GeometryCollection gc = (GeometryCollection)splitGeo;
					ArrayList geoms0=new ArrayList();
					ArrayList geoms1=new ArrayList();
					if (gc.getNumGeometries()>2){
						com.vividsolutions.jts.geom.Geometry[] splitGroups=createSplitGroups(jtsGeo,splittingLs);
						if (splitGroups.length==0){
							continue;
						}

						for(int j = 0; j < gc.getNumGeometries(); j++){
							com.vividsolutions.jts.geom.Geometry g = gc.getGeometryN(j);
							if (splitGroups[0].contains(g)){
								geoms0.add(g);
							}else{
								geoms1.add(g);
							}
						}
					}else if (gc.getNumGeometries()==2){
						geoms0.add(gc.getGeometryN(0));
						geoms1.add(gc.getGeometryN(1));
					}else{
						continue;
					}
					GeometryCollection gc0=createMulti(geoms0,gc.getFactory());


//					for(int j = 0; j < gc.getNumGeometries(); j++){
//						Geometry g = gc.getGeometryN(j);
						org.gvsig.fmap.geom.Geometry fmapGeo = Converter.jtsToGeometry(gc0);

//						if (j==0){
							EditableFeature eFeature=feature.getEditable();
							eFeature.setGeometry(store.getDefaultFeatureType().getDefaultGeometryAttributeName(), fmapGeo);
							store.update(eFeature);
//						}else{
							GeometryCollection gc1=createMulti(geoms1,gc.getFactory());
							fmapGeo = Converter.jtsToGeometry(gc1);

							EditableFeature newFeature=store.createNewFeature(store.getDefaultFeatureType(), feature);
							newFeature.setGeometry(store.getDefaultFeatureType().getDefaultGeometryAttributeName(), fmapGeo);
							store.insert(newFeature);
							SpatialCache spatialCache=getEditingLayer().getSpatialCache();
							Envelope envelope = fmapGeo.getEnvelope();
							spatialCache.insert(envelope,fmapGeo);
//						}
//					}//for j
				}//if splitGeo

			}

			store.endEditingGroup();

//			getCadToolAdapter().getMapControl().getMapContext().endAtomicEvent();
		} catch (Exception ex) {
			ex.printStackTrace();
		} finally {
			DisposeUtils.dispose(selected);
		}
	}
    private GeometryCollection createMulti(ArrayList geoms,GeometryFactory factory) {
		if (geoms.size()==0)
			return null;
		if (geoms.get(0) instanceof Polygon){
			return new MultiPolygon((Polygon[])geoms.toArray(new Polygon[0]),factory);
		}else if (geoms.get(0) instanceof LineString){
			return new MultiLineString((LineString[])geoms.toArray(new LineString[0]),factory);
		}else if (geoms.get(0) instanceof Point){
			return new MultiPoint((Point[])geoms.toArray(new Point[0]),factory);
		}
		return null;
	}
    
    private com.vividsolutions.jts.geom.Geometry[] createSplitGroups(com.vividsolutions.jts.geom.Geometry splitGeo, LineString splittingLs) {
    	try{
    		com.vividsolutions.jts.geom.Geometry[] geomsJTS=new com.vividsolutions.jts.geom.Geometry[2];
    		com.vividsolutions.jts.geom.Geometry r=splitGeo.getEnvelope();
    		com.vividsolutions.jts.geom.Geometry splitG = SplitStrategy.splitOp(r, splittingLs);
    		if(splitG instanceof GeometryCollection
    				&& ((GeometryCollection)splitG).getNumGeometries()>1){
    			GeometryCollection gc = (GeometryCollection)splitG;
    			for(int j = 0; j < gc.getNumGeometries(); j++){
    				geomsJTS[j]=gc.getGeometryN(j);
    			}
    		}
    		return geomsJTS;
    	}catch (Exception e) {
    		e.printStackTrace();
    	}
    	return null;
    }
    
    public Coordinate[] getPoint2DAsCoordinates(Point2D[] point2d) {
		Coordinate[] solution = new Coordinate[point2d.length];
		for (int i = 0; i < point2d.length; i++) {
			solution[i] = new Coordinate(point2d[i].getX(), point2d[i].getY());
		}
		return solution;
	}
    
    protected GeneralPathX createGPXLine(ArrayList points) {
        GeneralPathX gpx = new GeneralPathX();
        for (int i = 0; i < points.size(); i++) {
            Point2D p = (Point2D) points.get(i);
            if (i == 0)
                gpx.moveTo(p.getX(), p.getY());
            else
                gpx.lineTo(p.getX(), p.getY());
        }
        return gpx;
    }
}
