package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.PathIterator;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.osgi.framework.ServiceReference;

/**
 * @author Vicente Caballero Navarro
 */
public class RemoveVertexFunc extends SelectionVertexFunc {
    
    public boolean execution(ServiceReference gvSIGMobileRef) {
        boolean execute = super.execution(gvSIGMobileRef);
        try {
            FeatureStore fs = getEditingLayer().getFeatureStore();
            Geometry geom = removeSelectedVertex();
            EditableFeature eFeature = selectedFeatureHandler.getEditable();
            eFeature.setGeometry(fs.getDefaultFeatureType()
                    .getDefaultGeometryAttributeName(), geom);
            fs.update(eFeature);
            selectedFeatureHandler = null;
            selectedHandler = null;
            repaintMap();
        } catch (DataException e) {
            e.printStackTrace();
        }
        return execute;
    }
    
    public RemoveVertexFunc() {
        super();
    }
    
    public void configure() {
        // TODO Auto-generated method stub
        
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
        
    }
    
    private Geometry removeSelectedVertex() {
        GeneralPathX newGp = new GeneralPathX();
        double[] theData = new double[6];
        
        PathIterator theIterator;
        int theType;
        int numParts = 0;
        theIterator = selectedFeatureHandler.getDefaultGeometry()
                .getPathIterator(null, getFlatNess()); // , flatness);
        int numSegmentsAdded = 0;
        while (!theIterator.isDone()) {
            theType = theIterator.currentSegment(theData);
            switch (theType) {
                case PathIterator.SEG_MOVETO:
                    if (theData[0] == selectedHandler.getX()
                            && theData[1] == selectedHandler.getY()) {
                        theIterator.next();
                        theType = theIterator.currentSegment(theData);
                        newGp.moveTo(theData[0], theData[1]);
                    } else {
                        newGp.moveTo(theData[0], theData[1]);
                    }
                    numParts++;
                    numSegmentsAdded++;
                    break;
                
                case PathIterator.SEG_LINETO:
                    if (theData[0] == selectedHandler.getX()
                            && theData[1] == selectedHandler.getY()) {
                        break;
                    } else {
                        newGp.lineTo(theData[0], theData[1]);
                    }
                    numSegmentsAdded++;
                    break;
                
                case PathIterator.SEG_QUADTO:
                    newGp
                            .quadTo(theData[0], theData[1], theData[2],
                                    theData[3]);
                    numSegmentsAdded++;
                    break;
                
                case PathIterator.SEG_CUBICTO:
                    newGp.curveTo(theData[0], theData[1], theData[2],
                            theData[3], theData[4], theData[5]);
                    numSegmentsAdded++;
                    break;
                
                case PathIterator.SEG_CLOSE:
                    newGp.closePath();
                    break;
            } // end switch
            
            theIterator.next();
        } // end while loop
        
        Geometry shp = null;
        switch (selectedFeatureHandler.getDefaultGeometry().getType()) {
            // case Geometry.TYPES.POINT: // Tipo punto
            // shp = createPoint(firstPoint.getX(), firstPoint.getY());
            // break;
            
            case Geometry.TYPES.CURVE:
                try {
                    shp = geomManager.createCurve(newGp,
                            Geometry.SUBTYPES.GEOM2D);
                } catch (CreateGeometryException e) {
                    e.printStackTrace();
                }
                break;
            case Geometry.TYPES.SURFACE:
            case Geometry.TYPES.CIRCLE:
            case Geometry.TYPES.ELLIPSE:
                try {
                    shp = geomManager.createSurface(newGp,
                            Geometry.SUBTYPES.GEOM2D);
                } catch (CreateGeometryException e) {
                    e.printStackTrace();
                }
                break;
        }
        return shp;
    }
    
    public boolean isApplicable() {
        FLyrVect lv = getEditingLayer();
        try {
            int type = lv.getShapeType();
            if (type == Geometry.TYPES.CURVE || type == Geometry.TYPES.SURFACE
                    || type == Geometry.TYPES.GEOMETRY) {
                if (selectedHandler!=null)
                    return true;
            }
        } catch (ReadException e) {
            e.printStackTrace();
        }
        return false;
    }
}
