/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.gps.dialog;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;

/**
 * Visualizes one point and the direction have been taken.
 * This dialog is used to add a new point landmark and if the GPS is active, says the direction line to reach new point.
 * The new point is able to being edited by user with the same attributes as the rest of the layer features.
 * 
 * When the user clicks on "OK", The landmark will be created. 
 *  
 * @author Carlos Snchez Perin
 *
 */
public class FinishPointDialog extends OkCancelDialog {	
	
	private static final long serialVersionUID = 1L;	

	
    private LwTextField longText;    
    private LwTextField latText;
	
//    private LayersTable toc = null;
//	private MapControl map;
//	private MapSymbolIcon flagIcon;
//	private MapFinishTrackSupport flagTrackSupport;
//	private MapSymbolIcon actual_flag = null;
//	private MapFinishTrack actual_track = null;
	
	/**
	 * Constructor. Needed to get symbols painted.
	 * @param canvas 		where the icon will be painted
	 * @param p 			parent frame, needed for graphical purposes
	 */
	public FinishPointDialog(BundleContext bc) {	
		super(false, bc);
	}

	public void actionPerformed(Object arg0, Object arg1) {
		// TODO Auto-generated method stub
		WindowManager.getInstance().closeActivePanel();
		
	}

	public LwPanel buildMainPanel() {
		
		final LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwBorderLayout());
		
		final LwConstraints l = DialogConstants.getTextFieldConstraints();
		final LwConstraints l1 = DialogConstants.getLabelConstraints();

		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(2, 1));
		
		LwPanel center = new LwPanel();
		center.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));
		
		LwLabel t = DialogConstants.getDefaultLabel(Messages.getText("GPSFinishPointDialog.1"));
		

		left.add(l1, t);
		longText = getTextField();
		center.add(l,longText);
		
		left.add(l1, DialogConstants.getDefaultLabel(Messages.getText("GPSFinishPointDialog.2")));
		latText = getTextField();
		center.add(l,latText);
		
		resp.add(LwBorderLayout.WEST, left);
		resp.add(LwBorderLayout.CENTER, center);
		
//		t.getViewMan(true).setBg(new LwImgRender(ImageUtils
//				.getResourceImage(), LwView.ORIGINAL));

		
		
		return resp;
	}
	
	private LwTextField getTextField() {		
		final LwTextField resp = DialogConstants.getDefaultTextField();
		resp.setPSSize(80, -1);
		return resp;
	}

	public String getTitle() {
		return Messages.getText("GPSFinishPointDialog.0");
	}
}