/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package org.gvsig.mobile.extension.gps.dialog;

import java.util.Properties;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.extension.gps.PrefsListener;
import org.gvsig.mobile.gps.GPSConstants;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.FileChooserDialog;
import org.gvsig.mobile.ui.lw.FileChooserListener;
import org.gvsig.mobile.ui.lw.UIConstants;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwSpin;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.mask.BasicMaskValidator;
import org.zaval.lw.mask.LwMaskTextField;


/**
 * This dialog enables the selection of the file connection
 * @author manu
 *
 */
public class GPSFilePrefsDialog extends OkCancelPrefsDialog implements FileChooserListener {

	// row 1
	private LwTextField fileNameText;
	private LwButton fileNameButton;	
	// row 2
	private LwSpin intervalSpin;	

	/**
	 * Constructor
	 */
	public GPSFilePrefsDialog(BundleContext bc, Properties prefs, PrefsListener listener) {
			super(false, bc, prefs, listener);
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.ui.lw.OkCancelDialog#getTitle()
	 */
	public String getTitle() {
		return Messages.getText("GPSFilePrefsDialog.title");
	}

	/**
	 * Fills the dialog with its controls
	 *
	 */
	public LwPanel buildMainPanel() {

		// --------- create controls: ----------------

		// labels
		final LwLabel fileNameLabel = DialogConstants.getDefaultLabel(Messages.getText("GPSFilePrefsDialog.file_name"));
		final LwLabel intervalLabel = DialogConstants.getDefaultLabel(Messages.getText("GPSFilePrefsDialog.interval"));
		// resize labels to homogenize
		int width = Math.max((int)fileNameLabel.getPreferredSize().width,
				(int)intervalLabel.getPreferredSize().width);
		int height = Math.max((int)fileNameLabel.getPreferredSize().height,
				(int)intervalLabel.getPreferredSize().height);
		fileNameLabel.setPSSize(width, height);
		intervalLabel.setPSSize(width, height);

		// text fields
		fileNameText = getFileNameText();
		
		// spins
		intervalSpin = getIntervalSpin();		
		
		// buttons
		fileNameButton = getFileNameButton();

		// --------- create layouts and fill the panel: ----------------
		LwConstraints rightC = DialogConstants.getTextFieldConstraints();
		LwConstraints leftC = DialogConstants.getLabelConstraints();

		// the panel
		LwPanel panel = new LwPanel();
		panel.setLwLayout(new LwBorderLayout());

		// a left column
		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(2, 1));
		
		// a right column, horizontal fill
		LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));		

		// fill left column
		left.add(leftC, fileNameLabel);		
		left.add(leftC, intervalLabel);				

		// fill right column
		LwContainer c = new LwPanel(); // a container for the combo + button
		c.setLwLayout(new LwBorderLayout());
		c.add(LwBorderLayout.CENTER, fileNameText); // the combo takes more space
		c.add(LwBorderLayout.EAST, fileNameButton);
		right.add(rightC, c);
		right.add(rightC, intervalSpin);

		// add columns to panel
		panel.add(LwBorderLayout.WEST,left);
		panel.add(LwBorderLayout.CENTER,right);

		return panel;		

	}	

	private LwTextField getFileNameText() {
		return new LwTextField ("", 255);
	}

	private LwSpin getIntervalSpin() {
		LwSpin ret = new LwSpin();
		LwMaskTextField mask1 = (LwMaskTextField)ret.getElement(LwSpin.EDITOR_EL);
		mask1.setMask("nnnn");
		mask1.setValidator(new BasicMaskValidator(0, 9999));
		ret.setStep(50);
		ret.setValue(200);
		return ret;
	}

	private LwButton getFileNameButton() {
		return DialogConstants.getButton("folder_transp", "folder_transp", this);		
	}
	
	public void actionPerformed(Object src, Object data) {
		// Call parent method (OkCancelPrefsDialog)
		super.actionPerformed(src, data);
		if (src == fileNameButton) {
			final FileChooserDialog f =
				new FileChooserDialog(this, null, new String[][]{UIConstants.FILTER_ALL}, FileChooserDialog.DIALOG_TYPE_SELECT);
			f.setTitle(Messages.getText("GPSFilePrefsDialog.file_chooser_title"));
			f.showDialog();
		} 
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.ui.lw.FileChooserListener#fileSelected(java.lang.String)
	 */
	public void fileSelected(String path) {
		this.fileNameText.setText(path);
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.dialog.OkCancelPrefsDialog#collectPrefs()
	 */
	public Properties collectPrefs(Properties initPrefs) {
		initPrefs.setProperty(GPSConstants.K_FILE_NAME, fileNameText.getText());
		initPrefs.setProperty(GPSConstants.K_INTERVAL, Integer.toString(intervalSpin.getValue()));
		return initPrefs;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.dialog.OkCancelPrefsDialog#selectDefaults(java.util.Properties)
	 */
	public void selectDefaults(Properties initPrefs) {
		if (initPrefs.containsKey(GPSConstants.K_FILE_NAME)) {
			fileNameText.setText(initPrefs.getProperty(GPSConstants.K_FILE_NAME));			
		}
		if (initPrefs.containsKey(GPSConstants.K_INTERVAL)) {
			int interval = 0;
			try {
				interval = Integer.parseInt((initPrefs.getProperty(GPSConstants.K_INTERVAL)));
			} catch (NumberFormatException e) {
			}
			intervalSpin.setValue(interval);
		}
	}

}
