/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package org.gvsig.mobile.extension.gps.dialog;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;


/**
 * This dialog enables the selection of the NMEA messages to be processed
 * @author juangui
 *
 */
public class GPSNmeaPrefsDialog extends OkCancelDialog {

	// fixed width for labels that are on the left side of the dialog
	private static final int LABELS_WIDTH = 65;
	private static final int BUTTONS_WIDTH = 22;

	// row 1
	private LwCheckbox dbtChkbox;
	private LwCheckbox ggaChkbox;

	// row 2
	private LwCheckbox gllChkbox;
	private LwCheckbox gsvChkbox;

	// row 3
	private LwCheckbox hdgChkbox;
	private LwCheckbox hdtChkbox;

	// row 4
	private LwCheckbox rmcChkbox;
	private LwCheckbox vtgChkbox;


	/**
	 * Constructor
	 */
	public GPSNmeaPrefsDialog(BundleContext bc) {
		super(false, bc);
		
	}

	/**
	 * Fills the dialog with its controls
	 *
	 */
	public LwPanel buildMainPanel() {

		// --------- create controls: ----------------
		
		LwConstraints l = DialogConstants.getLabelConstraints();

		LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwGridLayout(8, 1));
		// title (row 0)		
//		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
//		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
//		titleLabel.setAlignment(1);
//		titleLabel.setSize(LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);

		// row 1
		dbtChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSNmeaPrefsDialog.1")), LwCheckbox.CHECK); //$NON-NLS-1$
		dbtChkbox.setState(true);
		
//		dbtChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
//		dbtChkbox.addMouseListener(this);
//		dbtChkbox.setEnabled(false);

		ggaChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSNmeaPrefsDialog.2")), LwCheckbox.CHECK); //$NON-NLS-1$
//		ggaChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
//		ggaChkbox.addMouseListener(this);
		ggaChkbox.setState(true);
//		ggaChkbox.setEnabled(false);
		
		resp.add(l, dbtChkbox);
		resp.add(l, ggaChkbox);

		// row 2
		gllChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSNmeaPrefsDialog.3")), LwCheckbox.CHECK); //$NON-NLS-1$
//		gllChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
//		gllChkbox.addMouseListener(this);
		gllChkbox.setState(true);
//		gllChkbox.setEnabled(false);
		
		gsvChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSNmeaPrefsDialog.4")), LwCheckbox.CHECK); //$NON-NLS-1$
//		gsvChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
//		gsvChkbox.addMouseListener(this);
		gsvChkbox.setState(true);
//		gsvChkbox.setEnabled(false);
		
		resp.add(l, gllChkbox);
		resp.add(l, gsvChkbox);

		// row 3
		hdtChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSNmeaPrefsDialog.5")), LwCheckbox.CHECK); //$NON-NLS-1$
//		hdtChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
//		hdtChkbox.addMouseListener(this);
		hdtChkbox.setState(true);
//		hdtChkbox.setEnabled(false);
		
		hdgChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSNmeaPrefsDialog.6")), LwCheckbox.CHECK); //$NON-NLS-1$
//		hdgChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
//		hdgChkbox.addMouseListener(this);
		hdgChkbox.setState(true);
//		hdgChkbox.setEnabled(false);
		
		resp.add(l, hdtChkbox);
		resp.add(l, hdgChkbox);

		// row 4
		rmcChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSNmeaPrefsDialog.7")), LwCheckbox.CHECK); //$NON-NLS-1$
//		rmcChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
//		rmcChkbox.addMouseListener(this);
		rmcChkbox.setState(true);
//		rmcChkbox.setEnabled(false);
		
		vtgChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSNmeaPrefsDialog.8")), LwCheckbox.CHECK); //$NON-NLS-1$
//		vtgChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
//		vtgChkbox.addMouseListener(this);
		vtgChkbox.setState(true);
//		vtgChkbox.setEnabled(false);		
		
		resp.add(l, rmcChkbox);
		resp.add(l, vtgChkbox);
		
		LwScrollPan s = new LwScrollPan(resp);
		
		return s;
	}	

	public void actionPerformed(Object arg0, Object arg1) {
		// TODO Auto-generated method stub
		WindowManager.getInstance().closeActivePanel();
	}

	public String getTitle() {
		return Messages.getText("GPSNmeaPrefsDialog.0");
	}	
}
