package org.gvsig.mobile.extension.gps.dialog;

import java.util.Properties;

import org.gvsig.mobile.extension.gps.PrefsListener;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwLabel;

public abstract class OkCancelPrefsDialog extends OkCancelDialog {
	protected final PrefsListener listener;
	private final Properties prefs;

	/**
	 * Constructor
	 * @param scrollable
	 * @param bc
	 * @param prefs
	 * @param listener
	 */
	public OkCancelPrefsDialog(boolean scrollable, BundleContext bc, Properties prefs, PrefsListener listener) 
	throws IllegalArgumentException {
		super(scrollable, bc);
		if (listener != null)
			this.listener = listener;
		else
			throw new IllegalArgumentException("Listener can't be null.");
		if (prefs != null)
			this.prefs = prefs;
		else
			this.prefs = new Properties();
		selectDefaults(this.prefs);
	}

	/**
	 * Collect all preferences from the dialog in a single properties object.
	 * This method must be implemented to return the new preferences of the dialog.
	 * The initial properties are given as parameter so that hidden values can also
	 * be put in the new properties.
	 * 
	 * @param initPrefs
	 *   A copy of the initial properties.
	 * @return
	 *   This method shall return a Properties instance with all preferences of the dialog.
	 */
	public abstract Properties collectPrefs(Properties initPrefs);

	/**
	 * Use at this point initPrefs to get the initial values and select them in the dialog.
	 * Since the buildMainPanel() is called in the constructor before knowing the iniPrefs, the selection of
	 * initial values has to be done separately.
	 * @param initPrefs
	 *   A Properties instance with the initial preferences.
	 */
	public abstract void selectDefaults(Properties initPrefs);

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.ui.lw.OkCancelDialog#actionPerformed(java.lang.Object, java.lang.Object)
	 */
	public void actionPerformed(Object src, Object data) {
		if (src == this.getOkButton()) {
			Properties endPrefs = collectPrefs((Properties)prefs.clone());
			if (!endPrefs.equals(prefs)) {
				listener.prefsChanged(this, endPrefs);
			}
			WindowManager.getInstance().closeActivePanel();
		} else if (src == this.getCancelButton()) {
			WindowManager.getInstance().closeActivePanel();
		}
	}

	/**
	 * Utility method to get the index of a combo list
	 * @param combo
	 * @param text
	 * @return
	 */
	protected int comboIndexOf(LwCombo combo, String text) {
		LwAList list = combo.getList();
		for (int i=0;i<list.count();i++) {
			if (((LwLabel)list.getItem(i)).getText().equals(text))
				return i;
		}
		return -1;
	}


}
