package org.gvsig.mobile.extension.gps.functionality;

import java.util.Properties;

import org.gvsig.location.LocationException;
import org.gvsig.location.LocationProvider;
import org.gvsig.location.nmea.FileNmeaLocationProvider;
import org.gvsig.location.nmea.RxtxNmeaLocationProvider;
import org.gvsig.location.nmea.BtNmeaLocationProvider;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.gps.GPSConstants;
import org.gvsig.mobile.gps.GPSResolutor;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class GPSConnectFunc extends Functionality {
	
	private boolean connected = false;
	
	private static final Logger logger = LoggerFactory.getLogger(GPSConnectFunc.class);

	public GPSConnectFunc() {
		super();
	}

	public void configure() {
		// TODO Auto-generated method stub		
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {
		try {
			if (connect()) {
				logger.debug("GPS connected.");
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		logger.debug("Could not connect to GPS.");
		return false;
	}

	public void stop() {
		super.stop();
		// Disconnect GPS
		disconnect();
		logger.debug("GPS disconnected.");
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * Connect location provider
	 * @return
	 *   true if the location provider was successfully instantiated
	 */
	public boolean connect() {
		Properties gpsPrefs = GPSResolutor.getGPSPrefs();
		String protocol, target;
		LocationProvider provider = null;
		if (gpsPrefs != null) {
			if (gpsPrefs.containsKey(GPSConstants.K_PROTOCOL)) {
				protocol = gpsPrefs.getProperty(GPSConstants.K_PROTOCOL);
				if (gpsPrefs.containsKey(GPSConstants.K_TARGET)) {
					target = gpsPrefs.getProperty(GPSConstants.K_TARGET);
					try {
						if (protocol.equals(GPSConstants.V_PROTOCOL_NMEA)) {
							if (target.equals(GPSConstants.V_TARGET_COMM)) {
								provider = new RxtxNmeaLocationProvider(gpsPrefs.getProperty(GPSConstants.K_PORT_NAME),
										Integer.parseInt(gpsPrefs.getProperty(GPSConstants.K_BAUD_RATE)), null);
							} else if (target.equals(GPSConstants.V_TARGET_BTSPP)) {
								provider = new BtNmeaLocationProvider(gpsPrefs.getProperty(GPSConstants.K_DEVICE_ADDR), null);
							} else if (target.equals(GPSConstants.V_TARGET_FILE)) {
								provider = new FileNmeaLocationProvider(gpsPrefs.getProperty(GPSConstants.K_FILE_NAME),
										Integer.parseInt(gpsPrefs.getProperty(GPSConstants.K_INTERVAL)), null);
								//GPSResolutor.setLocationProvider(bc, provider);
								//connected = true;
							} else if (target.equals(GPSConstants.V_TARGET_HTTP)) {
							}
						} else if (protocol.equals(GPSConstants.V_PROTOCOL_SIRF)) {
							if (target.equals(GPSConstants.V_TARGET_COMM)) {						
							} else if (target.equals(GPSConstants.V_TARGET_BTSPP)) {
							} else if (target.equals(GPSConstants.V_TARGET_FILE)) {
							} else if (target.equals(GPSConstants.V_TARGET_HTTP)) {
							}
						} else if (protocol.equals(GPSConstants.V_PROTOCOL_TSIP)) {
							if (target.equals(GPSConstants.V_TARGET_COMM)) {						
							} else if (target.equals(GPSConstants.V_TARGET_BTSPP)) {
							} else if (target.equals(GPSConstants.V_TARGET_FILE)) {
							} else if (target.equals(GPSConstants.V_TARGET_HTTP)) {
							}
						} else if (protocol.equals(GPSConstants.V_PROTOCOL_GPSD)) {
							if (target.equals(GPSConstants.V_TARGET_COMM)) {						
							} else if (target.equals(GPSConstants.V_TARGET_BTSPP)) {
							} else if (target.equals(GPSConstants.V_TARGET_FILE)) {
							} else if (target.equals(GPSConstants.V_TARGET_HTTP)) {
							}
						}
						if (provider != null) {
							GPSResolutor.setLocationProvider(provider);
							connected = true;
						}
					} catch (NumberFormatException e) {
						// A string to number conversion failed
						logger.error("Number format exception whem converting location provider parameters. " +
								e.getMessage());
					} catch (LocationException e) {
						// The location provider instantiation failed
						logger.error("Could not instantiate the location provider. " + e.getMessage());
					}
				}
			}
			
		}
		return connected;
	}

	/**
	 * Disconnect location provider
	 */
	public void disconnect() {
		LocationProvider provider = GPSResolutor.getLocationProvider();
		if (provider != null)
			provider.reset();
		connected = false;
		provider = null;
//		GPSResolutor.setLocationProvider(bc, provider);
	}

}
