/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.wfs.dialog;

import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.exception.InitializeException;
import org.gvsig.fmap.dal.exception.ProviderNotRegisteredException;
import org.gvsig.fmap.dal.exception.ValidateDataParametersException;
import org.gvsig.fmap.geom.Geometry.TYPES;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.fmap.mapcontext.rendering.legend.IVectorLegend;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.symbology.basic.DefaultSymbol;
import org.gvsig.mobile.ui.lw.AppSettings;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.gvsig.mobile.wfs.dal.serverexplorer.WFSServerExplorer;
import org.gvsig.mobile.wfs.dal.serverexplorer.WFSServerExplorerParameters;
import org.gvsig.mobile.wfs.dal.store.WFSStoreParameters;
import org.gvsig.mobile.wfs.dal.store.WFSStoreProvider;
import org.gvsig.mobile.wfs.utils.WFSTreeNode;
import org.gvsig.remoteclient.wfs.WFSClient;
import org.gvsig.remoteclient.wfs.WFSFeature;
import org.gvsig.tools.locator.LocatorException;
import org.osgi.framework.BundleContext;
import org.zaval.data.Item;
import org.zaval.data.Tree;
import org.zaval.data.TreeModel;
import org.zaval.data.event.TreeListener;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.misc.CItem;
import org.zaval.lw.misc.LwCustomTreeEditor;
import org.zaval.lw.misc.LwCustomTreeView;
import org.zaval.lw.tree.LwTree;

/**
 * @author <a href="mailto:csanchez@prodevelop.es">Carlos Snchez</a>
 */
public class AddWFSDialog extends OkCancelDialog implements TaskStatusListener,
		TreeListener {

	private LwCombo serversCombo;
	private LwCombo srsCombo;
	private LwCombo formatsCombo;
	private LwTextField serverText;
	private LwButton serverButton;
	private LwTree wfsLayersTree;
	private WFSClient wfsClient;
	private HashMap srsMap = new HashMap();	
	private String selectedLayerName = "NONE";
	private LayerManagerNotifier tocNotifier;
	private int selectedSRS = 0;
	private WFSServerExplorer serverExplorer;
	private TreeMap layers = new TreeMap();
	private HashMap titles = new HashMap();

	
	public AddWFSDialog(LayerManagerNotifier tocNotifier, BundleContext bc) {
		super(false, bc);
		this.tocNotifier = tocNotifier;
	}

	public LwPanel buildMainPanel() {
		final LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwBorderLayout());

		final LwConstraints l = DialogConstants.getTextFieldConstraints();
		final LwConstraints l1 = DialogConstants.getLabelConstraints();

		/**
		 * The top panel contains on WEST the labels and on CENTER the
		 * textfields It's on the NORTH of the back panel
		 */
		final LwPanel top = new LwPanel();
		top.setLwLayout(new LwBorderLayout());

		/**
		 * The labels
		 */
		final LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(4, 1));

		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("WFSDialog.2")));
		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("WFSDialog.3")));


		/**
		 * The textfields, filling the panel horizontally
		 */
		final LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(4, 1, LwToolkit.HORIZONTAL));

		serversCombo = getServersCombo();
		right.add(l, serversCombo);

		/**
		 * Textfield + Button
		 */
		LwPanel composite = new LwPanel();
		composite.setLwLayout(new LwBorderLayout());

		serverText = getServerText();
		serverText.setText("");
		composite.add(LwBorderLayout.CENTER, serverText);

		serverButton = getServerButton();
		composite.add(LwBorderLayout.EAST, serverButton);

		right.add(l, composite);

//		srsCombo = getSRSCombo();
//		right.add(l, srsCombo);

//		formatsCombo = getFormatsCombo();
//		right.add(l, formatsCombo);

		top.add(LwBorderLayout.WEST, left);
		top.add(LwBorderLayout.CENTER, right);

		/**
		 * An empty tree
		 */
		this.wfsLayersTree = new LwTree(new Tree(new Item("..."))) {
			public void mousePressed(LwMouseEvent e) {
				try {
					super.mousePressed(e);
					CItem c = (CItem) wfsLayersTree.getItemAt(e.getX(), e
							.getY());					
					CItem selected = (CItem) wfsLayersTree.getSelectedItem();					
					
					if (c == null || selected == null || c != selected)
						return;
					
					onItemSelected(c, true);

					final TreeModel data = wfsLayersTree.getModel();					

					// si tiene hijos marcarlos todos
					// updateChildren(data, c, checked);
					uncheckSiblings(data, c);
					// si es el nico hijo desmarcado aplicar tambin al
					// padre
					// y a los padres de los padres
					// updateParents(data, c, checked);

					// String name = getLayerName(c);

//					final WFSLayerNode layer;
//					if (name == null)
//						layer = wfsDriver.getLayersTree();
//					else
//						layer = wfsDriver.getLayer(name);
//
//					fillSRS(layer.getAllSrs(), checked, layer);
				} catch (Exception e1) {
					e1.printStackTrace();
				} finally {
					
				}
			}
		};
		LwCustomTreeView c = new LwCustomTreeView();
		c.setDefFont(DialogConstants.getDefaultFont());
		LwCustomTreeEditor ct = new LwCustomTreeEditor();
		this.wfsLayersTree.setViewProvider(c);
		this.wfsLayersTree.setEditorProvider(ct);
		this.wfsLayersTree.getViewMan(true).setBorder("br.etched");
		this.wfsLayersTree.setColor(LwTree.SEL2_COLOR, Color.blue);

		/**
		 * The tree must be scrollable. It's located on the CENTER of the back
		 * panel
		 */
		LwScrollPan t = new LwScrollPan(this.wfsLayersTree);

		resp.add(LwBorderLayout.NORTH, top);
		resp.add(LwBorderLayout.CENTER, t);
		return resp;
	}
	
	private String getLayerName(CItem item) {
		String layerTitle = item.getValue().toString();
		layerTitle = layerTitle.substring(layerTitle
				.indexOf("@ch.on:") + 7, layerTitle.length());
		//String name = wfsClient.getLayerName(layerTitle);
		//return name;
		return layerTitle;
	}




	
	private void clearSRS() {
		LwCompList list;
		if (srsCombo == null) {
			this.srsCombo = new LwCombo();
			list = new LwCompList();
		} else {
			list = (LwCompList) this.srsCombo.getList();
		}
		list.removeAll();
		srsMap.clear();
	}



	private void uncheckSiblings(TreeModel data, CItem c) {
		
		CItem parent = (CItem) data.getParent(c);

		if (parent != null) {
			// boolean lastChildren = true;
			final int children_ = data.getChildrenCount(parent);
			CItem child_;
			if (children_ > 0) {
				for (int i = 0; i < children_; i++) {
					child_ = (CItem) data.getChildAt(parent, i);
					if (child_ != null && child_ != c) {
						onItemSelected(child_, false);		
					}
				}
			}
		}
	}


	private LwButton getServerButton() {
		return DialogConstants.getButton("add", "add", this);
	}

	private LwTextField getServerText() {
		return DialogConstants.getDefaultTextField();
	}

	private LwCombo getServersCombo() {
		final LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
//		l.addItem(DialogConstants.getDefaultLabel(""));
		l.addSelectionListener(this);
		this.loadFromPersistence(l);
		resp.setList(l);
		return resp;
	}

	private void loadFromPersistence(LwAList l) {

		String[] wfs = AppSettings.wfsServers;

		l.removeAll();
//		l.addItem(DialogConstants.getDefaultLabel(""));

		if (wfs != null) {
			for (int i = 0; i < wfs.length; i++) {
				if (wfs[i].length() > 0)
					l.addItem(DialogConstants.getDefaultLabel(wfs[i]));
			}
		}
	}

	public void actionPerformed(Object arg0, Object arg1) {

		if (arg0 == this.getOkButton()) {
			WFSTreeNode layer;
			//WFSStoreFactory.setUrl(wfsClient.getUrl().toString());
			StringBuffer s = new StringBuffer();

				int init=selectedLayerName.lastIndexOf(":");
				String name  = selectedLayerName.substring(init+1, selectedLayerName.length());
				if (name == null)
					layer = getLayersTree();
				else
					layer=(WFSTreeNode) layers.get(titles.get(name));
					
				
				//s.append(layer.getName());

			//WFSStoreFactory.setVersion(wfsClient.getVersion());
			//WFSStoreFactory.setFeatureType(s.toString());
			// ArrayList onelist = new ArrayList();
			// onelist.add(selectedLayerName);
			loadLayer(selectedLayerName, name);
			tocNotifier.updateTOC();
			close();
		} else if (arg0 == this.getCancelButton()) {
			close();
		} else if (arg0 == this.serverButton) {
			callGetCapabilities(serverText.getText());
		} else if (arg0 instanceof LwAList) {
			try {				
				String url = this.getServerUrl();
				if (url != null && url.trim().compareTo("") != 0) {
					callGetCapabilities(url);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}

			// System.out.println("Alist");
		} else if (arg1 instanceof CItem) {
			
		}
	}

	private void callGetCapabilities(String server) {
		try {
			DataManager dataManager = DALLocator.getDataManager();
			//Create the server explorer parameters
			WFSServerExplorerParameters parameters = 
				(WFSServerExplorerParameters) dataManager
					.createServerExplorerParameters(WFSServerExplorer.NAME);			
			parameters.setUrl(server);
						
			//Create the server explorer
			serverExplorer = (WFSServerExplorer)dataManager.createServerExplorer(parameters);
				
				
//				wfsConnector = WFSConnectorFactory.getFMapDriverForURL(new URL(server));			
//				
//				if (wfsConnector.connect(new WFSCancellable())) {
//					WFSTreeNode w = wfsConnector.getLayersTree();
//					this.loadWFSResponse(w);
//					this.repaint();
//				}
			if (serverExplorer!=null){
				WFSTreeNode tree = this.getLayersTree();
				this.loadWFSResponse(tree);
				this.repaint();
			}
				
		} catch (Exception e) {
			clearSRS();
			TreeModel data = new Tree();
			if (this.wfsLayersTree == null) {
				wfsLayersTree = new LwTree(data);
			} else {
				wfsLayersTree.setModel(data);			
			}
		}
			
			
			//Update the forms
//			getLblTitle().setText(serverExplorer.getTitle());
//			getTxtAbstract().setText(serverExplorer.getAbstract());
//			lblServerTypeValue.setText(serverExplorer.getServerType());

//			Hashtable features = serverExplorer.getFeatures();
//			Iterator iter=features.keySet().iterator();

			//List cacheProviders=dataManager.getFeatureCacheProviders();
			//Iterator iter=cacheProviders.iterator();

//			CItem citem;
//			StringBuffer sb = new StringBuffer();
//			String layerName = "";
//			LwCheckbox c;
//			boolean first = true;
//			while (iter.hasNext()) {
//				layerName = (String) iter.next();
//				sb.append("0;@ch.off:").append(layerName).append(";@ch.on:").append(layerName);
//				citem = new CItem(sb.toString());
//				c = new LwCheckbox(DialogConstants.getDefaultLabel(layerName),LwCheckbox.CHECK);
//				citem.setEditor(c);
//				citem.setFont(DialogConstants.getDefaultFont());
//				wfsLayersTree.getModel().add((LwPanel)parent, citem);
//				sb.delete(0, sb.length());
//	
//				// first selected
//				if (first) {
//					onItemSelected(citem, true);
//					first = false;
//				}
//			}
				
//				cmbCache.addItem(featureCacheProvider);

					
//			if(!serverExplorer.getServerType().toLowerCase().startsWith("wfs 1.0")){
//				cmbCache.setEnabled(true);
//			} else{
//				cmbCache.setEnabled(false);
//			}
//			wfsParamsPanel.setServerExplorer(serverExplorer);
//			connected = true;
//			wfsClient = wfsClientFactory.getFMapDriverForURL(new URL(server));			
			
//				if (wfsClient.connect(new WFSCancellable())) {
//				WFSTreeNode w = wfsClient.getLayersTree();
//				this.loadWFSResponse(w);
//				this.repaint();
//			}
			
			
//		} catch (Exception e) {
//			
//			clearSRS();
//			TreeModel data = new Tree();
//			if (this.wfsLayersTree == null) {
//				wfsLayersTree = new LwTree(data);
//			} else {
//				wfsLayersTree.setModel(data);			
//			}
//		}
	}
	




	/**
	 * Changes the status of an CItem containing a LwCheckBox
	 * 
	 * @param item
	 * @returns true if the item has been checked
	 */

	private void onItemSelected(CItem item, boolean check) {
		String value = ((CItem) item).getValue().toString();
		char ch = value.charAt(0);
		boolean isCheckable = true;

		if (ch == '0' || ch == '1') {
		} else {
			isCheckable = false;
		}

		if (isCheckable) {
			ch = check ? '1' : '0';
			value = String.valueOf(ch) + value.substring(1, value.length());
			((CItem) item).resetValue(value);
			
			this.repaint();
			
			if (check) {
				selectedLayerName = (String) item.getValue(); // (String) wfsLayersTree.getSelectedItem().getValue();
			}
		}
	}
	
//	private void addLayerName(String name) {
//		if (!selectedLayersName.contains(name)) {
//			selectedLayersName.add(name);
//		}
//	}
//	
//	private void removeLayerName(String name) {
//		if (selectedLayersName.contains(name)) {
//			selectedLayersName.remove(name);			
//		}		
//	}
	

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the url of the server selected by the user.
	 */
	public String getServerUrl() {
		String url = null;
		try {
			Object selected = serversCombo.getList().getSelected();			
			
			if (selected != null) {
				url = ((LwLabel) selected).getText(); 
			}			
		} catch (Exception e) {
			//Ignore
		}	
		return url;
	}

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the SRS selected by the user.
	 */
	public String getSrs() {
		return ((LwLabel) srsCombo.getList().getSelected()).getText();
	}
	

	/**
	 * This method is called after the dialog is closed. Gets the full extent of
	 * the layer to be added. This not fully functional. The dialog simply keeps
	 * a unique extent for each SRS, regardless of the layers.
	 * 
	 * TODO Keep the extent of each layer for each SRS.
	 * 
	 * @param name
	 *            layer name
	 * @param srs
	 *            SRS that will be used in the requests
	 * @param def
	 *            default extent to be returned if no data is available
	 * @return the full extent of the layer
	 */
	public Rectangle2D getFullExtent(String name, String srs, Rectangle2D def) {
		return null;
	}


	public void taskFinished(ITask task) {

	}

	public void taskInited(ITask task) {

	}

	public void taskProgress(ITask task, int progress) {

	}

	public String getTitle() {
		// TODO: Localize this
		 return Messages.getText("WFSDialog.1");
	}

	public void itemInserted(Object arg0, Item arg1) {
		// TODO Auto-generated method stub
	}

	public void itemModified(Object arg0, Item arg1) {
		// TODO Auto-generated method stub
	}

	public void itemRemoved(Object arg0, Item arg1) {
		// TODO Auto-generated method stub
	}

	private void fillLayers(WFSTreeNode rootLayer, TreeModel data, Item parent) {
		ArrayList children = rootLayer.getChildren();
		final int length = children.size();

		CItem citem;
		StringBuffer sb = new StringBuffer();
		WFSTreeNode layer = null;
		String layerName = "";
		LwCheckbox c;

		for (int i = 0; i < length; i++) {
			layer = (WFSTreeNode) children.get(i);
			layerName = layer.getTitle();
			sb.append("0;@ch.off:").append(layerName).append(";@ch.on:")
					.append(layerName);
			citem = new CItem(sb.toString());
			c = new LwCheckbox(DialogConstants.getDefaultLabel(layerName),
					LwCheckbox.CHECK);
			
			citem.setEditor(c);
			citem.setFont(DialogConstants.getDefaultFont());
			data.add(parent, citem);
			sb.delete(0, sb.length());

			// first selected
			if (i == 0) {
				onItemSelected(citem, true);
			}
		}
	}

	private void loadWFSResponse(WFSTreeNode rootLayer) {
		
		if (rootLayer == null) {
			// TODO: Log or throw exception
			return;
		}

		String rootName = rootLayer.getTitle();
		if (rootName == null) rootName = rootLayer.getName();
		StringBuffer sb = new StringBuffer();
		sb.append("0;@ch.off:").append(rootName).append(";@ch.on:").append(rootName);
		CItem root = new CItem(sb.toString());
		LwCheckbox rootlbl = new LwCheckbox(DialogConstants.getDefaultLabel(rootName),
						LwCheckbox.CHECK);
		root.setEditor(rootlbl);
		root.setFont(DialogConstants.getDefaultFont());
		//Item root = new Item(rootLayer.getTitle());
		TreeModel data = new Tree(root);

		clearSRS();
		onItemSelected(root, true);
		fillLayers(rootLayer,data, root);
		if (this.wfsLayersTree == null) {
			wfsLayersTree = new LwTree((org.zaval.data.TreeModel) data);
		} else {
			wfsLayersTree.setModel((org.zaval.data.TreeModel) data);			
		}
	}
	
	/**
	 * @return A tree containing the info of all layers available on this
	 *         server.
	 */
	public WFSTreeNode getLayersTree() {
		WFSTreeNode fmapRootLayer = null;
		if (serverExplorer.getFeatures() != null) {
			fmapRootLayer = new WFSTreeNode();
			if(serverExplorer.getServerType()!=null)
				fmapRootLayer.setName(serverExplorer.getServerType());
			else {
				fmapRootLayer.setName("WFS "+serverExplorer.getVersion());
			}
			fmapRootLayer.setParent(null);
			ArrayList children = new ArrayList();
			Hashtable features = serverExplorer.getFeatures();
			Enumeration enumeration = features.elements();
			while (enumeration.hasMoreElements()) {
				WFSFeature feature = (WFSFeature) enumeration.nextElement(); 
				// obtiene una feature y avanza
				children.add(parseTree(feature, fmapRootLayer));
				fmapRootLayer.setChildren(children);
			}	
		}
		return fmapRootLayer;
	}
	
	/**
	 * Parses the client's layer node and translates it to a fmap's layer node
	 * 
	 * @param WMSLayer
	 * @return WMSLayerNode
	 */
	private WFSTreeNode parseTree(WFSFeature feature, WFSTreeNode parentNode) {

		WFSTreeNode myNode = new WFSTreeNode();

		// Name
		myNode.setName(feature.getName());

		// Title
		myNode.setTitle(feature.getTitle());

		// SRS
		myNode.setSrs(feature.getSrs());

		// Parent layer
		myNode.setParent(parentNode);

		// Abstract
		myNode.setAbstract(feature.getAbstract());

		// LatLonBox
		//if (feature.getLatLonBbox() != null)
			//myNode.setLatLonBox(feature.getLatLonBbox().toString());

		// Keywords
		Vector keywords = feature.getKeywords();
		for (int i = 0; i < keywords.size(); i++) {
			myNode.addKeyword((String) keywords.get(i));
		}

		if (myNode.getName() != null){
			layers.put(myNode.getName(), myNode);
			titles.put(myNode.getTitle(),myNode.getName());
		}
		return myNode;
	}

	public void loadLayer(String layername, String toc_name) {
		
		String name = null;
		FLyrVect layer = null;
		WFSTreeNode layerNode = null;
		Map attributeMap = new HashMap();

		StringBuffer serverRequest = new StringBuffer();

//		attributeMap.put("version", serverExplorer.getVersion());
//		attributeMap.put("layer", serverRequest.toString());

		WFSStoreProvider store=null;
		
		try {
			WFSStoreParameters param = (WFSStoreParameters) DALLocator.getDataManager().createStoreParameters(WFSStoreProvider.NAME);
			param.setVersion(serverExplorer.getVersion());
			param.setUrl(serverExplorer.getUrl());
			//param.setFeatureType((WFSFeature)(serverExplorer.getFeatureInfo(null, layername)));
			store = (WFSStoreProvider) DALLocator.getDataManager().createStore(param);
		} catch (InitializeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ProviderNotRegisteredException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LocatorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ValidateDataParametersException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		int index = layerNode.getName().indexOf(":");
		
//		if (index>0){
//			String namespaceprefix = layerNode.getName().substring(0,index);
//			client.getServiceInformation().getNamespace(namespaceprefix);
//			String namespace = wfsConnector.getNamespace(namespaceprefix);
//			WFSStoreFactory.setNamespace(namespace);
//			WFSStoreFactory.setNamespacePrefix(namespaceprefix);
//		}
		

		//		store = serverExplorer.getFeatureInfo(nameSpace, layerName).WFSStoreFactory.createWFSStore();
		if ((store != null)) {

			try {

				layer = new FLyrVect();
				layer.setDataStore(store.getFeatureStore());
				IVectorLegend lgnd = MapContextLocator.getMapContextManager()
						.createDefaultVectorLegend(TYPES.SURFACE);

//				int r = 50 + rnd.nextInt(150);
//				int g = 50 + rnd.nextInt(150);
//				int b = 50 + rnd.nextInt(150);
//				Color c = new Color(r, g, b);

				DefaultSymbol s = new DefaultSymbol();
//				s.setBorderColor(c.darker());
//				s.setColor(c);
				s.setWidth(1);
				s.setFilled(true);

//				int gtype = store.getDefaultFeatureType().getDefaultGeometryAttribute().getGeometryType();
				

				lgnd.setDefaultSymbol(s);
				layer.setLegend(lgnd);

				// if (this.getMapControl() == null) {
				// IGvSigContext gv = (IGvSigContext)bc.getService(servRef);
				// this.getMapControl() = (MapControl)gv.getMapControl();
				// }

//				layer.setParentLayer(this.getMapControl().getMapContext()
//						.getLayers());
//
//				IProjection ipro = this.getMapControl().getMapContext()
//						.getProjection();

				// IProjection ipro = CRSResolutor.getCRS("EPSG:23030", bc);
				// canvas.getViewPort().setProjection(ipro);
//				layer.setProjection(ipro);
//				layer.setName(toc_name);

//				this.getMapControl().addLayer(layer);
//				this.getMapControl().repaint();
				// gvSIGMobileRef.;
			} catch (Exception ex) {
				System.err.println("Error: " + ex.getMessage());
			}
		}
	}
}