package org.gvsig.mobile.extension.wkprojection.jhlabs.adapter;

import java.util.Hashtable;

import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.Ellipsoid;
import org.gvsig.mobile.libProjection.interfaces.CRSConstants;
import org.gvsig.mobile.libProjection.interfaces.IEllipsoid;

public class EllipsoidAdapter implements IEllipsoid {

	private Ellipsoid elli;
	private Hashtable props = new Hashtable();
	
	/**	 * Flattening	 */
	private double f;
	/**	 * Second Excentricity	 */
	private double se;
	/**	 * Second Excentricity ^2	 */
	private double se2;
	/**	 * Radio de curvatura polar	 */
	private double c;

	public EllipsoidAdapter(Ellipsoid e) {
		elli = e;
		f = 1 - Math.sqrt(1-e.eccentricity2);
		double a = e.equatorRadius;
		double b = e.poleRadius;
		se = Math.sqrt(((Math.pow(a,2))-(Math.pow(b,2)))/((Math.pow(b,2))));		
		se2 = Math.pow(se,2);
		c = (Math.pow(a,2))/b;
		
		props.put(CRSConstants.ELLIPSOID_ABREV, elli.getShortName());
	}
	
	public String getEllipsoidAbrev() {
		return elli.getShortName();
	}

	public double getFirstExcentricity() {
		return elli.eccentricity;
	}

	public double getFirstExcentricitySquare() {
		return elli.eccentricity2;
	}

	public double getFlattening() {
		// TODO Auto-generated method stub
		return f;
	}

	public double getPolarCurvature() {
		return c;
	}

	public Hashtable getProps() {
		return props;
	}

	public double[] getRadios(double lat) {
		double rlat = lat*Math.PI/180;
		double denomin = (1-elli.eccentricity2*(Math.pow(Math.sin(rlat),2)));
		/* radio meridiano*/
		double rm = (elli.equatorRadius*(1-elli.eccentricity2))/(Math.pow(denomin, 1.5));
		/* radio primer vertical*/
		double rn = elli.equatorRadius/(Math.sqrt(denomin));
		/* radio gaussiano */
		double rg = Math.sqrt(rm*rn);
		
		double[] lRadios = {rm, rn, rg}; 
		return lRadios;
	}

	public double getSecondExcentricity() {
		return se;
	}

	public double getSecondExcentricitySquare() {
		return se2;
	}

	public double getSemiMajorAxis() {
		return elli.equatorRadius;
	}

	public double getSemiMinorAxis() {
		return elli.poleRadius;
	}

	public void setEllipsoidAbrev(String abrev) {
		elli.setShortName(abrev);
	}

	public void setFirstExcentricity(double data) {
		elli.eccentricity = data;
	}

	public void setFirstExcentricitySquare(double data) {
		elli.eccentricity2 = data;
	}

	public void setFlattening(double data) {
		f = data;
	}

	public void setPolarCurvature(double data) {
		c = data;
	}

	public void setSecondExcentricity(double data) {
		se = data;
	}

	public void setSecondExcentricitySquare(double data) {
		se2 = data;
	}

	public void setSemiMajorAxis(double data) {
		elli.equatorRadius = data;
	}

	public void setSemiminorAxis(double data) {
		elli.poleRadius = data;
	}
	
	public Object getImplementation() {
		return elli;
	}

}
