package org.gvsig.mobile.extension.wkprojection.jhlabs.adapter;

import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.Hashtable;

import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.Ellipsoid;
import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.Projection;
import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.TransverseMercatorProjection;
import org.gvsig.mobile.libProjection.impl.CRSResolutor;
import org.gvsig.mobile.libProjection.interfaces.CRSConstants;
import org.gvsig.mobile.libProjection.interfaces.IEllipsoid;
import org.gvsig.mobile.libProjection.interfaces.IMapProjection;
import org.osgi.framework.BundleContext;

public class MapProjectionAdapter implements IMapProjection {

	private BundleContext bc;
	private Projection labsProjection;
	private HashMap params = new HashMap();
	private Hashtable props = new Hashtable();
	
	public MapProjectionAdapter(Projection p, BundleContext _bc) {
		bc = _bc;
		labsProjection = p;
		setAbrev(p.toString());
	}

	public void initialize() {
		
		String strv = null;
		Integer intv = null;
		Double douv = null;
		Object v = null;
		IEllipsoid elli = null;
		
		if ((bc != null) && (v = params.get(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV)) != null) {
			strv = (String) v;
			elli = CRSResolutor.getEllipsoid(strv, bc);
			labsProjection.setEllipsoid((Ellipsoid) elli.getImplementation());
		}
		
		if (((v = params.get(IMapProjection.PARAM_PROJECTION_UTM_ZONE)) != null) &&
		(labsProjection instanceof TransverseMercatorProjection)){
			intv = (Integer) v;
			((TransverseMercatorProjection) labsProjection).setUTMZone(intv.intValue());
		}
		
		if ((v = params.get(IMapProjection.PARAM_PROJECTION_FALSE_E)) != null) {
			douv = (Double) v;
			labsProjection.setFalseEasting(douv.doubleValue());
		}
		
		if ((v = params.get(IMapProjection.PARAM_PROJECTION_FALSE_N)) != null) {
			douv = (Double) v;
			labsProjection.setFalseNorthing(douv.doubleValue());
		}
		
		if ((v = params.get(IMapProjection.PARAM_PROJECTION_KFACTOR)) != null) {
			douv = (Double) v;
			labsProjection.setScaleFactor(douv.doubleValue());
		}
		
		if ((v = params.get(IMapProjection.PARAM_PROJECTION_LAT_1)) != null) {
			douv = (Double) v;
			labsProjection.setProjectionLatitude1Degrees(douv.doubleValue());
		}
		
		if ((v = params.get(IMapProjection.PARAM_PROJECTION_LAT_2)) != null) {
			douv = (Double) v;
			labsProjection.setProjectionLatitude2Degrees(douv.doubleValue());
		}
		
		if ((v = params.get(IMapProjection.PARAM_PROJECTION_LATITUDE)) != null) {
			douv = (Double) v;
			labsProjection.setProjectionLatitudeDegrees(douv.doubleValue());
		}
		
		if ((v = params.get(IMapProjection.PARAM_PROJECTION_LONGITUDE)) != null) {
			douv = (Double) v;
			labsProjection.setProjectionLongitudeDegrees(douv.doubleValue());
		}
		labsProjection.initialize();
	}
	
	public String getAbrev() {
		return (String) props.get(CRSConstants.PROJECTION_ABREV);
	}

	public HashMap getParams() {
		return params;
	}

	public Hashtable getProps() {
		return props;
	}

	public Point2D project(Point2D p) {
		Point2D.Double resp = new Point2D.Double(0, 0);
		labsProjection.transform(new Point2D.Double(p.getX(), p.getY()), resp);
		return resp;
	}

	public Point2D projectInverse(Point2D p) {
		Point2D.Double resp = new Point2D.Double(0, 0);
		labsProjection.inverseTransform(new Point2D.Double(p.getX(), p.getY()), resp);
		return resp;
	}

	public void setAbrev(String abrev) {
		props.put(CRSConstants.PROJECTION_ABREV, abrev);
	}

	public void setParams(HashMap p) {
		params.putAll(p);
	}
	
	public IMapProjection cloneWithoutParams() {
		return new MapProjectionAdapter((Projection) labsProjection.clone(), bc);
	}
	
	// **************************************************************
	// ** MANUAL ****************************************************
	// **************************************************************
	
	public void manualSetEllipsoid(Ellipsoid e) {
		if (e != null) {
			labsProjection.setEllipsoid(e);
		}
	}
	
//	public void manualSetUtmZona(int z) {
//		if (labsProjection instanceof TransverseMercatorProjection) {
//			((TransverseMercatorProjection) labsProjection).setUTMZone(z);
//		}
//	}
//	
//	public void manualSetFalseEasting(double e) {
//		labsProjection.setFalseEasting(e);
//	}
//	
//	public void manualSetFalseNorthing(double n) {
//		labsProjection.setFalseNorthing(n);
//	}
//	
//	public void manualSetScaleFactor(double f) {
//		labsProjection.setScaleFactor(f);
//	}
//
//	public void manualSetProjLatitude(double v) {
//		labsProjection.setProjectionLatitudeDegrees(v);
//	}
//	
//	public void manualSetProjLatitude1(double v) {
//		labsProjection.setProjectionLatitude1Degrees(v);
//	}
//	
//	public void manualSetProjLatitude2(double v) {
//		labsProjection.setProjectionLatitude2Degrees(v);
//	}
//	
//	public void manualSetProjLongitude(double v) {
//		labsProjection.setProjectionLongitudeDegrees(v);
//	}
//	
//	public void manualInitialize() {
//		labsProjection.initialize();
//	}


}
