/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.wms.dialog;

import java.awt.Color;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.cresques.cts.IProjection;
import org.gvsig.fmap.mapcontext.exceptions.LoadLayerException;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.extension.wms.fmap.drivers.wms.FMapWMSDriver;
import org.gvsig.mobile.extension.wms.fmap.drivers.wms.FMapWMSDriverFactory;
import org.gvsig.mobile.extension.wms.fmap.drivers.wms.WMSCancellable;
import org.gvsig.mobile.extension.wms.fmap.layers.FLyrWMS;
import org.gvsig.mobile.extension.wms.fmap.layers.WMSLayerNode;
import org.gvsig.mobile.libProjection.impl.CRSResolutor;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.lw.AppSettings;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.MessageBox;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.data.Item;
import org.zaval.data.Tree;
import org.zaval.data.TreeModel;
import org.zaval.data.event.TreeListener;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.misc.CItem;
import org.zaval.lw.misc.LwCustomTreeEditor;
import org.zaval.lw.misc.LwCustomTreeView;
import org.zaval.lw.tree.LwTree;

public class AddWMSDialog extends OkCancelDialog implements TaskStatusListener,
		TreeListener {

	private LwCombo serversCombo;
	private LwCombo srsCombo;
	private LwCombo formatsCombo;
	private LwTextField serverText;
	private LwButton serverButton;
	private LwTree wmsLayersTree;
	private FMapWMSDriver wmsDriver;
	private HashMap srsMap = new HashMap();
	private ArrayList selectedLayersName = new ArrayList();
	private LayerManagerNotifier tocNotifier;
	private int selectedSRS = 0;
	private ServiceReference reference;
	
	// private Envelope currentExt = null;
	
	private static final Logger logger = LoggerFactory.getLogger(AddWMSDialog.class);

	public AddWMSDialog(LayerManagerNotifier tocNotifier, BundleContext bc, ServiceReference gvSIGMobileRef) {
		super(false, bc);
		this.tocNotifier = tocNotifier;
		this.reference=gvSIGMobileRef;
	}

	public LwPanel buildMainPanel() {
		final LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwBorderLayout());

		final LwConstraints l = DialogConstants.getTextFieldConstraints();
		final LwConstraints l1 = DialogConstants.getLabelConstraints();

		/**
		 * The top panel contains on WEST the labels and on CENTER the
		 * textfields It's on the NORTH of the back panel
		 */
		final LwPanel top = new LwPanel();
		top.setLwLayout(new LwBorderLayout());

		/**
		 * The labels
		 */
		final LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(4, 1));

		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("WMSDialog.2")));
		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("WMSDialog.3")));
		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("WMSDialog.4")));
		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("WMSDialog.5")));

		/**
		 * The textfields, filling the panel horizontally
		 */
		final LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(4, 1, LwToolkit.HORIZONTAL));

		serversCombo = getServersCombo();
		right.add(l, serversCombo);

		/**
		 * Textfield + Button
		 */
		LwPanel composite = new LwPanel();
		composite.setLwLayout(new LwBorderLayout());

		serverText = getServerText();
		serverText.setText("");
		composite.add(LwBorderLayout.CENTER, serverText);

		serverButton = getServerButton();
		composite.add(LwBorderLayout.EAST, serverButton);

		right.add(l, composite);

		srsCombo = getSRSCombo();
		right.add(l, srsCombo);

		formatsCombo = getFormatsCombo();
		right.add(l, formatsCombo);

		top.add(LwBorderLayout.WEST, left);
		top.add(LwBorderLayout.CENTER, right);

		/**
		 * An empty tree
		 */
		this.wmsLayersTree = new LwTree(new Tree(new Item("..."))) {
			public void mousePressed(LwMouseEvent e) {
				try {
					super.mousePressed(e);
					CItem c = (CItem) wmsLayersTree.getItemAt(e.getX(), e
							.getY());
					CItem selected = (CItem) wmsLayersTree.getSelectedItem();

					if (c == null || selected == null || c != selected)
						return;
					
					boolean checked = AddWMSDialog.this.onItemSelected(c);

					final TreeModel data = wmsLayersTree.getModel();

					// si tiene hijos marcarlos todos
					updateChildren(data, c, checked);
					// si es el nico hijo desmarcado aplicar tambin al
					// padre
					// y a los padres de los padres
					updateParents(data, c, checked);

					String name = getLayerName(c);

					final WMSLayerNode layer;
					if (name == null)
						layer = wmsDriver.getLayersTree();
					else
						layer = wmsDriver.getLayer(name);

					fillSRS(layer.getAllSrs(), checked, layer);
				} catch (Exception e1) {
					logger.error(e1.getMessage());
					e1.printStackTrace();
				} finally {

				}
			}
		};
		LwCustomTreeView c = new LwCustomTreeView();
		c.setDefFont(DialogConstants.getDefaultFont());
		LwCustomTreeEditor ct = new LwCustomTreeEditor();
		this.wmsLayersTree.setViewProvider(c);
		this.wmsLayersTree.setEditorProvider(ct);
		this.wmsLayersTree.getViewMan(true).setBorder("br.etched");
		this.wmsLayersTree.setColor(LwTree.SEL2_COLOR, Color.blue);

		/**
		 * The tree must be scrollable. It's located on the CENTER of the back
		 * panel
		 */
		LwScrollPan t = new LwScrollPan(this.wmsLayersTree);

		resp.add(LwBorderLayout.NORTH, top);
		resp.add(LwBorderLayout.CENTER, t);
		return resp;
	}

	private String getLayerName(CItem item) {
		String layerTitle = item.getValue().toString();
		layerTitle = layerTitle.substring(layerTitle.indexOf("@ch.on:") + 7,
				layerTitle.length());
		return layerTitle;
	}

	private void updateMap(HashMap map, String key, final boolean add) {
		if (map == null)
			map = new HashMap();

		// Number of layers checked with the same key
		int value = 0;
		// We check if the key must be deleted
		boolean delete = false;

		if (map.containsKey(key)) {
			value = Integer.valueOf(map.get(key).toString()).intValue();
			if (add) {
				value++;
			} else {
				value--;
			}

			if (value <= 0) {
				delete = true;
			}

			if (delete) {
				map.remove(key);
			} else {
				map.remove(key);
				map.put(key, String.valueOf(value));
			}
		} else {
			if (add)
				map.put(key, String.valueOf(value));
		}
	}

	/**
	 * fills the SRS combo
	 * 
	 * @param SRS
	 * @param add
	 */
	private void fillSRS(final Vector SRS, final boolean add, WMSLayerNode layer) {
		int length = SRS.size();

		// inicializamos el combo y recuperamos la lista
		LwCompList list;
		if (srsCombo == null) {
			this.srsCombo = new LwCombo();
			list = new LwCompList();
		} else {
			list = (LwCompList) this.srsCombo.getList();
		}

		selectedSRS = list.getSelectedIndex();

		// eliminamos SRS repetidos
		Vector cSRS = new Vector();
		String s;
		for (int i = 0; i < length; i++) {
			s = SRS.elementAt(i).toString();
			if (s != null && !cSRS.contains(s)) {
				cSRS.add(s);
			}
		}

		// En un Hashmap nos guardamos los SRS y las repeticiones
		String srs;
		length = cSRS.size();
		for (int i = 0; i < length; i++) {
			try {
				srs = cSRS.elementAt(i).toString();
				if (srs != null)
					updateMap(srsMap, srs, add);
			} catch (Exception ignore) {
				ignore.printStackTrace();
			}
		}

		// recorremos el hashmap y llenamos el combo
		Iterator it = srsMap.keySet().iterator();

		list.removeAll();
		while (it.hasNext()) {
			srs = it.next().toString();
			if (srs != null) {
				list.addItem(DialogConstants.getDefaultLabel(srs));
			}
		}

		// recursivamente hacemos lo mismo con los srs de los hijos
		ArrayList children = layer.getChildren();
		WMSLayerNode l;
		if (children != null) {
			final int size = children.size();

			for (int i = 0; i < size; i++) {
				l = (WMSLayerNode) children.get(i);
				if (l != null) {
					this.fillSRS(l.getAllSrs(), add, l);
				}
			}
		}

		if (list.count() == 0)
			list.addItem(DialogConstants.getDefaultLabel(""));
		srsCombo.setList(list);

		if (selectedSRS < 0)
			selectedSRS = 0;

		srsCombo.getList().select(selectedSRS);
	}

	private void clearSRS() {
		LwCompList list;
		if (srsCombo == null) {
			this.srsCombo = new LwCombo();
			list = new LwCompList();
		} else {
			list = (LwCompList) this.srsCombo.getList();
		}
		list.removeAll();
		srsMap.clear();
	}
	
	private void clearFormats() {
		LwCompList list;
		if (formatsCombo == null) {
			this.formatsCombo = new LwCombo();
			list = new LwCompList();
		} else {
			list = (LwCompList) this.formatsCombo.getList();
		}
		list.removeAll();
	}

	private void fillFormats(Vector formats) {
		final int length = formats.size();

		// inicializamos el combo y recuperamos la lista
		LwCompList list;
		if (formatsCombo == null) {
			this.formatsCombo = new LwCombo();
			list = new LwCompList();
		} else {
			list = (LwCompList) this.formatsCombo.getList();
		}
		list.removeAll();

		String format;
		for (int i = 0; i < length; i++) {
			try {
				format = formats.get(i).toString();
				if (format != null) {
					list.addItem(DialogConstants.getDefaultLabel(format));
				}
			} catch (Exception ignore) {
			}
		}

		if (list.count() == 0)
			list.addItem(DialogConstants.getDefaultLabel(""));
		formatsCombo.setList(list);
		formatsCombo.getList().select(0);
	}

	private void updateChildren(TreeModel data, CItem c, boolean checked) {
		final int children = data.getChildrenCount(c);
		// si tiene hijos marcarlos todos
		// TODO y los hijos de los hijos
		CItem child;
		if (children > 0) {
			for (int i = 0; i < children; i++) {
				child = (CItem) data.getChildAt(c, i);
				if (child != null) {
					onItemSelected(child, checked);
					updateChildren(data, child, checked);
				}
			}
		}
	}

	private void updateParents(TreeModel data, CItem c, boolean checked) {
		CItem parent = (CItem) data.getParent(c);

		if (parent != null) {
			boolean lastChildren = true;
			final int children_ = data.getChildrenCount(parent);
			CItem child_;
			if (children_ > 0) {
				for (int i = 0; i < children_; i++) {
					child_ = (CItem) data.getChildAt(parent, i);
					if (child_ != null && child_ != c) {
						char o = child_.getValue().toString().charAt(0);
						if (o == '1') {
							lastChildren = false;
							break;
						}
					}
				}
			}

			if (checked)
				onItemSelected(parent, checked);

			if (lastChildren)
				onItemSelected(parent, checked);

			updateParents(data, parent, !lastChildren || checked);
		}

	}

	private LwCombo getFormatsCombo() {
		final LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
		l.addItem(DialogConstants.getDefaultLabel(""));
		resp.setList(l);
		return resp;
	}

	private LwCombo getSRSCombo() {
		final LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
		l.addItem(DialogConstants.getDefaultLabel(""));
		resp.setList(l);
		return resp;
	}

	private LwButton getServerButton() {
		return DialogConstants.getButton("add", "add", this);
	}

	private LwTextField getServerText() {
		return DialogConstants.getDefaultTextField();
	}

	private LwCombo getServersCombo() {
		final LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
		// l.addItem(DialogConstants.getDefaultLabel(""));
		l.addSelectionListener(this);
		this.loadFromPersistence(l);
		resp.setList(l);
		return resp;
	}

	private void loadFromPersistence(LwAList l) {

		String[] wms = AppSettings.wmsServers;

		l.removeAll();
		// l.addItem(DialogConstants.getDefaultLabel(""));

		if (wms != null) {
			for (int i = 0; i < wms.length; i++) {
				if (wms[i].length() > 0)
					l.addItem(DialogConstants.getDefaultLabel(wms[i]));
			}
		}
	}

	public boolean isValidSRS(String SRS) {
		IProjection p = CRSResolutor.getCRS(SRS, bc);
		return p != null;
	}

	public void actionPerformed(Object arg0, Object arg1) {
		
		
		if (arg0 == this.getOkButton()) {
			try {
				final int length = selectedLayersName.size();

				final String SRS = this.getSrs();


				final String format = this.getImageFormat();

				String name = null;
				WMSLayerNode layer;
				Map m;
				FLyrWMS fl = null;

				m = new HashMap();

				m.put("host", wmsDriver.getUrl().toString());
				m.put("srs", SRS);
				m.put("height", String.valueOf(mapControl.getHeight()));
				m.put("width", String.valueOf(mapControl.getWidth()));
				m.put("format", format);
				
				StringBuffer s = new StringBuffer();

				for (int i = 0; i < length; i++) {
					name = this.selectedLayersName.get(i).toString();
					if (name == null)
						layer = wmsDriver.getLayersTree();
					else
						layer = wmsDriver.getLayer(name);
					
					if (layer!=null)
						s.append(name);

					if ((i < length - 1)&&(wmsDriver.getLayer(this.selectedLayersName.get(i+1).toString())!=null)) {
						s.append(",");
					}
				}

				m.put("layer", s.toString());
				
				//Setting the first fullenvelope
				IGvSigContext gvc = (IGvSigContext)bc.getService(reference);
				MapControl mc = (MapControl) gvc.getMapControl();

				fl = new FLyrWMS(m);
				fl.addLayerListener(mapControl);
				fl.setFullEnvelope(MapControl.getExtentFromFullExtentAndImageSize(fl.getFullEnvelope(), mc.getViewPort().getImageSize(), false));
				mapControl.addLayer(fl);
				if(fl.getProjection()!=null)
					mapControl.getViewPort().setProjection(fl.getProjection());
				else{					
					mapControl.getViewPort().setProjection(CRSResolutor.getCRS(((LwLabel)srsCombo.getList().getSelected()).getText(), bc));
				}
			} catch (LoadLayerException e) {
				logger.error(e.getMessage());
			} catch (Exception e) {
				logger.error(e.getMessage());
			} finally {
				tocNotifier.updateTOC();
				close();
			}
		} else if (arg0 == this.getCancelButton()) {
			tocNotifier.updateTOC();
			close();
		} else if (arg0 == this.serverButton) {
			// wmsc = new WMSClient(
			// "http://www.idee.es/wms/IDEE-Base/IDEE-Base");
			callGetCapabilities(serverText.getText());

		} else if (arg0 instanceof LwAList) {
			try {
				String url = this.getServerUrl();
				if (url != null && url.trim().compareTo("") != 0) {
					callGetCapabilities(url);
				}
			} catch (Exception e) {
				logger.error(e.getMessage());
			}			
		} else if (arg1 instanceof CItem) {

		}
	}

	private void callGetCapabilities(String server) {
		try {
			wmsDriver = FMapWMSDriverFactory
					.getFMapDriverForURL(new URL(server));

			if (wmsDriver.connect(new WMSCancellable())) {
				WMSLayerNode w = wmsDriver.getLayersTree();
				loadWMSResponse(w);
			}

		} catch (Exception e) {
			MessageBox.showDialog(false, bc, Messages.getText("Unable_to_connect_to_server") + ". " + e.getMessage(), null,
					MessageBox.MESSAGE_TYPE_ERROR, MessageBox.MESSAGE_ONLY_OK );
			logger.error(e.getMessage());
			
			this.clearSRS();
			this.clearFormats();
			if (wmsLayersTree == null) {
				wmsLayersTree = new LwTree(new Tree());
			} else {
				wmsLayersTree.setModel(new Tree());
			}
			
		}
		this.repaint();
	}

	/**
	 * Changes the status of an CItem containing a LwCheckBox
	 * 
	 * @param item
	 * @returns true if the item has been checked
	 */

	private boolean onItemSelected(CItem item) {
		
		if (item == null) {
			return false;
		}
		
		String value = "";
		value = ((CItem) item).getValue().toString();
		char ch = value.charAt(0);
		boolean isCheckable = true;
		boolean isChecked = false;
		String name = null;
		if (ch == '0') {
			ch = '1';
			isChecked = true;
			name = this.getLayerName(item);
			if (name != null)
				this.addLayerName(name);
		} else if (ch == '1') {
			ch = '0';
			isChecked = false;
			name = this.getLayerName(item);
			if (name != null)
				this.removeLayerName(name);
		} else {
			isCheckable = false;
			isChecked = false;
		}

		if (isCheckable) {
			value = String.valueOf(ch) + value.substring(1, value.length());
			((CItem) item).resetValue(value);
			this.repaint();
		}
		return isChecked;
	}

	/**
	 * Changes the status of an CItem containing a LwCheckBox
	 * 
	 * @param item
	 * @returns true if the item has been checked
	 */

	private boolean onItemSelected(CItem item, boolean check) {
		String value = ((CItem) item).getValue().toString();
		char ch = value.charAt(0);
		boolean isCheckable = true;
		boolean isChecked = false;

		if (ch == '0') {
			ch = '1';
			isChecked = true;
		} else if (ch == '1') {
			ch = '0';
			isChecked = false;
		} else {
			isCheckable = false;
			isChecked = false;
		}

		String name = null;
		if (isCheckable) {
			if (check) {
				ch = '1';
				isChecked = true;
				name = this.getLayerName(item);
				if (name != null)
					this.addLayerName(name);
			} else {
				ch = '0';
				isChecked = false;
				name = this.getLayerName(item);
				if (name != null)
					this.removeLayerName(name);
			}
			value = String.valueOf(ch) + value.substring(1, value.length());
			((CItem) item).resetValue(value);

			this.repaint();
		}
		return isChecked;
	}

	private void addLayerName(String name) {
		if (!selectedLayersName.contains(name)) {
			selectedLayersName.add(name);
		}
	}

	private void removeLayerName(String name) {
		if (selectedLayersName.contains(name)) {
			selectedLayersName.remove(name);
		}
	}

	private int insertIfNotPresent(LwAList ch, String str, int j) {

		int cnt = ch.count();
		ArrayList list = new ArrayList();

		String text = "";
		for (int i = 0; i < cnt; i++) {
			text = ((LwLabel) ch.getItem(i)).getText();
			if (text.length() > 0) {
				list.add(text);
			}
		}

		String[] current_list = (String[]) list.toArray(new String[0]);
		int index = isContained(str, current_list);
		if (index == -1) {
			ch.insertItem(j, DialogConstants.getDefaultLabel(str));
			index = j;
		}
		return index;
	}

	private int isContained(String str, String[] arr) {

		for (int i = 0; i < arr.length; i++) {
			if (arr[i].compareToIgnoreCase(str) == 0) {
				return i;
			}
		}
		return -1;
	}

	private String[] getServerChoiceItems() {

		final LwAList l = serversCombo.getList();
		int cnt = l.count();

		String[] resp = new String[cnt];
		for (int i = 0; i < cnt; i++)
			resp[i] = ((LwLabel) l.getItem(i)).getText();
		return resp;
	}

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the url of the server selected by the user.
	 */
	public String getServerUrl() {
		String url = null;
		try {
			Object selected = serversCombo.getList().getSelected();

			if (selected != null) {
				url = ((LwLabel) selected).getText();
			}
		} catch (Exception e) {
			// Ignore
		} finally {
			return url;
		}
	}

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the SRS selected by the user.
	 */
	public String getSrs() {
		return ((LwLabel) srsCombo.getList().getSelected()).getText();
	}

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the format of the images to be requested.
	 */
	private String getImageFormat() {
		return (((LwLabel) formatsCombo.getList().getSelected()).getText());
	}


	public void taskFinished(ITask task) {

	}

	public void taskInited(ITask task) {

	}

	public void taskProgress(ITask task, int progress) {
		// TODO Auto-generated method stub

	}

	public String getTitle() {
		// TODO: Localize this
		return Messages.getText("WMSDialog.1");
		// return "WMS Dialog";
	}

	public void itemInserted(Object arg0, Item arg1) {

	}

	public void itemModified(Object arg0, Item arg1) {
		// TODO Auto-generated method stub
	}

	public void itemRemoved(Object arg0, Item arg1) {
		// TODO Auto-generated method stub
	}

	private void fillLayers(WMSLayerNode rootLayer, TreeModel data, Item parent) {
		ArrayList children = rootLayer.getChildren();
		final int length = children.size();

		CItem citem;
		StringBuffer sb = new StringBuffer();
		WMSLayerNode layer = null;
		String layerName = "";
		LwCheckbox c;

		for (int i = 0; i < length; i++) {
			layer = (WMSLayerNode) children.get(i);
			layerName = layer.getName();
			//TODO make the layer titles shown instead the layers names
			sb.append("0;@ch.off:").append(layerName).append(";@ch.on:")
					.append(layerName);
			citem = new CItem(sb.toString());
			c = new LwCheckbox(DialogConstants.getDefaultLabel(layerName),
					LwCheckbox.CHECK);
			citem.setEditor(c);
			citem.setFont(DialogConstants.getDefaultFont());
			data.add(parent, citem);
			sb.delete(0, sb.length());
			fillLayers(layer, data, citem);
		}
	}

	private void loadWMSResponse(WMSLayerNode rootLayer) {

		if (rootLayer == null) {
			logger.error("error while loading WMS response: rootLayer is null");
			return;
		}

		String rootName = rootLayer.getTitle();
		if (rootName == null)
			rootName = rootLayer.getName();
		StringBuffer sb = new StringBuffer();
		sb.append("0;@ch.off:").append(rootName).append(";@ch.on:").append(
				rootName);
		CItem root = new CItem(sb.toString());
		LwCheckbox c = new LwCheckbox(
				DialogConstants.getDefaultLabel(rootName), LwCheckbox.CHECK);
		root.setEditor(c);
		root.setFont(DialogConstants.getDefaultFont());
		// Item root = new Item(rootLayer.getTitle());
		TreeModel data = new Tree(root);

		clearSRS();
		fillSRS(rootLayer.getAllSrs(), true, rootLayer);

		fillLayers(rootLayer, data, root);

		fillFormats(wmsDriver.getFormats());


		if (this.wmsLayersTree == null) {
			wmsLayersTree = new LwTree((org.zaval.data.TreeModel) data);
		} else {
			wmsLayersTree.setModel((org.zaval.data.TreeModel) data);
		}

	}
}