package org.gvsig.mobile.gpe;

import java.awt.Color;
import java.io.File;
import java.util.Random;

import org.cresques.cts.IProjection;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureStore;
import org.gvsig.fmap.dal.store.gpe.GPEStoreParameters;
import org.gvsig.fmap.dal.store.gpe.GPEStoreProvider;
import org.gvsig.fmap.geom.Geometry.TYPES;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.fmap.mapcontext.rendering.legend.IVectorLegend;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.extensions.interfaces.IFileProviderFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.IFileType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.symbology.basic.DefaultSymbol;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.MessageBox;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class GPEFuncRead extends Functionality implements
IFileProviderFunctionality {

	protected Logger logger = LoggerFactory.getLogger(GPEFuncRead.class);
	protected String path = null;
	protected Random rnd = new Random(System.currentTimeMillis());
	protected static DataManager dm = DALLocator.getDataManager();

	private boolean canceled = false;
	
	public static final String ID = "GPEFuncID";
	
	public GPEFuncRead(BundleContext context) {
		super(ID, context);
	}

	public String getFilePath() {
		return path;
	}

	public void setFilePath(String filePath) {
		path = filePath;
	}

	public void configure() {
		// TODO Auto-generated method stub

	}

	public boolean execution(ServiceReference gvSIGMobileRef) {
		
		File gpefile = new File(path);

		DefaultFeatureStore store = null;
		GPEStoreParameters gpeParameters = null;
		FLyrVect lyr = null;
		DefaultSymbol s = null;

		try {
			
			if (dm == null)
				dm = DALLocator.getDataManager();

			IProjection ipro = getMapControl().getMapContext().getProjection();

			gpeParameters = (GPEStoreParameters) dm.createStoreParameters(GPEStoreProvider.NAME);
			
			gpeParameters.setFile(gpefile);
			gpeParameters.setCRS(ipro);
			
			store = (DefaultFeatureStore) dm.createStore(gpeParameters);

			lyr = new FLyrVect();
			lyr.setDataStore(store);

			IVectorLegend lgnd = MapContextLocator.getMapContextManager()
					.createDefaultVectorLegend(TYPES.SURFACE);

			int r = 50 + rnd.nextInt(150);
			int g = 50 + rnd.nextInt(150);
			int b = 50 + rnd.nextInt(150);
			Color c = new Color(r, g, b);

			s = new DefaultSymbol();
			s.setBorderColor(c.darker());
			s.setColor(c);
			s.setWidth(1);
			s.setFilled(true);

			lgnd.setDefaultSymbol(s);
			lyr.setLegend(lgnd);

			lyr.setParentLayer(getMapControl().getMapContext().getLayers());

			// IProjection ipro = CRSResolutor.getCRS("EPSG:23030", bc);
			// canvas.getViewPort().setProjection(ipro);
			lyr.setProjection(ipro);

			String name = gpefile.getName().toLowerCase();
			int lasp = name.lastIndexOf(".");
			if (lasp == -1) lasp = name.length();
			name = name.substring(0, lasp);
			
			if (name.length() > 0) {
				name = name.substring(0,1).toUpperCase() + name.substring(1);
			}

			lyr.setName(name);
			
			if (canceled) {
				store.close();
				store.dispose();
				return false;
			}

			getMapControl().addLayer(lyr);
			
			
			
		} catch (Throwable ex) {
			System.err.println("While creating GPE layer: " + ex.getMessage());
			logger.error("While creating GPE layer: " + ex.getMessage());
			
			WindowManager.getInstance().closeActiveUnlessMain();

			if (store != null) {
				try {
					store.close();
					store.dispose();
					stop();
				} catch (Throwable th) {
					logger.error("While closing/disp bad store: " + th.getMessage());
				}
			}

			MessageBox.showDialog(true, bc,
					Messages.getText("Error_while_opening_file") + ": " + ex.getMessage(),
					null, MessageBox.MESSAGE_TYPE_ERROR, MessageBox.MESSAGE_ONLY_OK);

			return false;
		}
		
		getMapControl().repaint();
		stop();
		return true;
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		// TODO Auto-generated method stub

	}
	
	
	public void cancel() {
		canceled = true;
	}

	private IFileType fileType;

	public IFileType getFileType() {
		return fileType;
	}

	public void setFileType(IFileType ft) {
		fileType = ft;
	}


}
