package org.gvsig.mobile.project;

import java.io.File;

import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dispose.Disposable;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.persistence.Persistent;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;

public class Project implements Persistent, Disposable {

	private File projectFile = null;
	private MapContext mapContext = null;
	private String desc = "Description.";
	
	
	public Project() { }
	
	public Project(MapContext mc) {
		mapContext = mc;
	}

	
	public File getProjectFile() {
		return projectFile;
	}
	
	public String getDescription() {
		return desc;
	}

	public void setDescription(String d) {
		desc = d;
	}

	public void setProjectFile(File f) {
		this.projectFile = f;
	}

	public MapContext getMapContext() {
		return mapContext;
	}

	public void setMapContext(MapContext c) {
		this.mapContext = c;
	}

	public void loadFromState(PersistentState state)
			throws PersistenceException {
		
		desc = state.getString("description");
		if (desc == null) {
			desc = "Description.";
		}
		String filestr = state.getString("file");
		projectFile = new File(filestr);
		mapContext = (MapContext) state.get("mapContext");
	}

	public void saveToState(PersistentState state) throws PersistenceException {
		state.set("file", projectFile.getAbsolutePath());
		state.set("mapContext", mapContext);
		state.set("description", desc);
	}
	
	public static void registerPersistence() {
		DynStruct definition =
			 ToolsLocator.getPersistenceManager().addDefinition(Project.class,
					"Project_Persistent",
					"Project Persistent definition",
	                null,
	                null);
	      
	    definition.addDynFieldString("file").setMandatory(true);
	    definition.addDynFieldString("description").setMandatory(true);
	    definition.addDynFieldObject("mapContext").setMandatory(true);
	    		
	}

	public void dispose() {
		projectFile = null;
		mapContext.dispose();
		mapContext = null;
	}

}
