package org.gvsig.mobile.tasks;

import java.util.LinkedList;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
 * It's a pool of threads used to run tasks
 * Currently it's a singleton
 * 
 * @author jcarras
 *
 */
public class WorkQueue
{
	
	private static final Logger logger = LoggerFactory.getLogger(WorkQueue.class);

    //private final int nThreads;
    private final PoolWorker[] threads;
    private final LinkedList queue;
    private static final WorkQueue instance = new WorkQueue(5);
    
    public static WorkQueue getInstance(){
    	return instance;
    }
    
    
    private WorkQueue(int nThreads)
    {
        //this.nThreads = nThreads;
        queue = new LinkedList();
        threads = new PoolWorker[nThreads];

        for (int i=0; i<nThreads; i++) {
            threads[i] = new PoolWorker("WorkQueue Worker - " + i);
            threads[i].start();
        }
    }

    public void execute(Runnable r) {
        synchronized(queue) {
            queue.addLast(r);
            queue.notify();
        }
        
    }

    private class PoolWorker extends Thread {
    	public PoolWorker(String name){
    		setName(name);
    	}
        public void run() {
            Runnable r;

            while (true) {
                synchronized(queue) {
                    while (queue.isEmpty()) {
                        try
                        {
                            queue.wait();
                        }
                        catch (InterruptedException ignored)
                        {
                        	// Syztem.out.println("One of the worker threads was interrupted while it was waiting.");
                        }
                    }

                    r = (Runnable) queue.removeFirst();
                }

                // If we don't catch RuntimeException, 
                // the pool could leak threads
                try {
                    r.run();
                }
                catch (RuntimeException e) {
                	logger.error("Runtime exception (" + e.getClass().getName() + "): " + e.getMessage());
                	e.printStackTrace();
                }
            }
        }
    }
    
    public void finalize()  
    {
    	for (int i=0; i<threads.length; i++) {
    		try {
    			threads[i].interrupt();
    		} catch (Exception ex) {
    			System.err.println("While interrupting thread: " + ex.getMessage());
    		}
        }
   }

}