package org.gvsig.mobile.ui.awt.mapcontrol;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

import org.gvsig.fmap.crs.CRSFactory;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.Geometry.SUBTYPES;
import org.gvsig.fmap.geom.Geometry.TYPES;
import org.gvsig.fmap.geom.exception.CreateEnvelopeException;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.fmap.geom.util.Converter;
import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.fmap.mapcontext.events.ColorEvent;
import org.gvsig.fmap.mapcontext.events.ExtentEvent;
import org.gvsig.fmap.mapcontext.events.ProjectionEvent;
import org.gvsig.fmap.mapcontext.events.listeners.ViewPortListener;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontext.layers.LayerEvent;
import org.gvsig.fmap.mapcontext.layers.LayerListener;
import org.gvsig.fmap.mapcontext.layers.SpatialCache;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.fmap.mapcontext.rendering.legend.events.LegendChangedEvent;
import org.gvsig.fmap.mapcontext.rendering.legend.events.listeners.LegendListener;
import org.gvsig.mobile.app.Application;
import org.gvsig.mobile.graphicLayer.SimpleGraphicLayer;
import org.gvsig.mobile.project.Project;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.gvsig.mobile.ui.awt.mapcontrol.snapping.ISnapper;
import org.gvsig.mobile.ui.awt.mapcontrol.snapping.ISnapperVectorial;
import org.gvsig.mobile.ui.lw.AppSettings;
import org.gvsig.tools.task.Cancellable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwCanvas;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.event.LwMouseListener;
import org.zaval.lw.event.LwMouseMotionListener;

public class MapControl extends LwCanvas implements
ViewPortListener, LwMouseListener, LwMouseMotionListener, LayerListener, LegendListener {	

	private static Logger logger = LoggerFactory.getLogger(MapControl.class);
	public static final String INFO_ACTIVATED_DYN_PROPERTY_NAME = "INFO_ACTIVATED_DYN_PROPERTY_NAME";


	private int status = DIRTY;
	private final int DRAW_INTERVAL = 100;
	private final int ACETATE_DRAW_INTERVAL = 100;
	private Timer timer;
	private Timer acetatetimer;
	
	Cancellable cancel = new Cancellable() {
		public boolean isCanceled() {
			return false;
		}

		public void setCanceled(boolean canceled) {
			// No queremos que se pueda cancelar la impresi�n.
		}
	};
	
	// private MapContext mapContext;
	private Project project;
	
	private static final GeometryManager geomManager = GeometryLocator
	    .getGeometryManager();	
	protected static MapControlManager mapControlManager = MapControlLocator.getMapControlManager();

	private Graphics2D mapGraphics;
	private BufferedImage mapImage;
	private Graphics2D bufferGraphics;
	private Image buffer;	
	
	// private Color back_Color = AppSettings.defaultBgColor;
	
	private int imageOffsetX = 0;
	private int imageOffsetY = 0;
	
	public boolean panMode = false;
	
	
	/**
     * Kind of geometry drawn to identify the kind of control point selected by the cursor's mouse.
     */
    private ISnapper usedSnap = null;
	
	/**
	 * <p>One of the possible status of <code>MapControl</code>. Determines that all visible information has been
	 * drawn and its updated.</p>
	 */
	public static final int UPDATED = 0;

	/**
	 * <p>One of the possible status of <code>MapControl</code>. Determines that not all visible information has been
	 * drawn or isn't updated.</p>
	 */
	public static final int DIRTY = 1;

	/**
	 * <p>One of the possible status of <code>MapControl</code>. Determines that only the graphical layer must
	 * be drawn / updated.</p>
	 */
	public static final int ONLY_GRAPHICS = 2;

	
	/**
	 * Sets a new offset for the image (used with pan tool)
	 */
	public void setOffsetAndRefresh(int offsetx, int offsety, boolean refresh) {
		imageOffsetX = offsetx;
		imageOffsetY = offsety;
//		logger.debug("MapControl - setOffsetAndRefresh, imageOffsetX = " + imageOffsetX);
//		logger.debug("MapControl - setOffsetAndRefresh, imageOffsetY = " + imageOffsetY);
		if (refresh) {
			repaint();
		}
	}
	
	public MapControl() {
		
		ViewPort viewPort = new ViewPort(CRSFactory.getCRS("EPSG:4326"));
		viewPort.addViewPortListener(this);

		FLayers lyrs = new FLayers();
		lyrs.setProjection(viewPort.getProjection());
		
		Project p = new Project(new MapContext(lyrs, viewPort));
		// viewPort.refreshExtent();
		
		File pfile = new File(
				AppSettings.defaultSavePath.getAbsolutePath()
				+ File.separator
				+ AppSettings.defaultProjectFileName);
		p.setProjectFile(pfile);
		
		initWith(p);
		setBackColor(AppSettings.defaultBgColor);
		initAcetateLayer(); 
	}
	
	public void initAcetateLayer() {
		SimpleGraphicLayer layer = new SimpleGraphicLayer();
//		DefaultGraphicLayer layer = new DefaultGraphicLayer();
		try {
			//layer.initialize(getViewPort().getProjection());	
			layer.setLegend(MapContextLocator.getMapContextManager().createDefaultVectorLegend(TYPES.GEOMETRY));
		} catch (Exception e) {
			e.printStackTrace();			
		}
		
		getMapContext().setGraphicsLayer(layer);
		this.setDirty();
	}
	
	TimerTask timerTask = new TimerTask() {
		public void run() {
			paintLayers();
		}
	};
	
	TimerTask acetateTask = new TimerTask() {
		public void run() {			
			
			if (status != ONLY_GRAPHICS) {
				return;
			}
			paintAcetate();
			status = UPDATED;
		}
	};
	private Point2D mapHandlerAdjustedPoint;
	
//	
//	public Image getMapImage() {
//		return mapImage;
//	}
	
//	public Graphics2D getMapGraphics() {
//		return mapGraphics;
//	}
	
	public void addLayer(FLayer lyr) {	
		
		if (lyr instanceof FLyrVect) {
			if (lyr.hasDynValue(INFO_ACTIVATED_DYN_PROPERTY_NAME)) {
				lyr.setDynValue(INFO_ACTIVATED_DYN_PROPERTY_NAME, new Boolean(true));
			}
		}
		
		getMapContext().getLayers().addLayer(lyr);
		lyr.addLayerListener(this);
		if (lyr instanceof FLyrVect) {
			((FLyrVect) lyr).addLegendListener(this);
		}
		setDirty();
	}
	
	public BufferedImage getMapImage() {
		return mapImage;
	}
	
	public void addLayer(int pos, FLayer lyr) {		
		getMapContext().getLayers().addLayer(pos, lyr);
		lyr.addLayerListener(this);
		if (lyr instanceof FLyrVect) {
			((FLyrVect) lyr).addLegendListener(this);
		}
		setDirty();
	}
	
//	/**
//     * Changes the order of the layers in those indices.
//     *
//     * @param _a
//     *            one of the indices
//     * @param _b
//     *            the other index
//     */
//    public void swapFromBottom(int _a, int _b) {
//    	FLayer[] lyrs = this.getArrayOfLayers();
//        
//        int last_index = lyrs.length - 1;
//
//        int a = last_index - _a;
//        int b = last_index - _b;
//
//        FLayer aux = lyrs[a];
//        lyrs[a] = lyrs[b];
//        lyrs[b] = aux;
//        
//        ArrayList layers = new ArrayList();
//
//        for (int i = 0; i < lyrs.length; i++) {
//        	getMapContext().getLayers().removeLayer(lyrs[i]);
//            layers.add(lyrs[i]);
//        }
//        
//        for (int i = 0; i < lyrs.length; i++) {
//        	getMapContext().getLayers().addLayer((FLayer)layers.get(i));            
//        }
//    }
	
	public void updateLayerPosition(int pos, FLayer lyr) {
		FLayer clonedLayer = null;
		try {
			clonedLayer = lyr.cloneLayer();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		
		FLayers lyrs = getMapContext().getLayers();
		lyrs.removeLayer(lyr);
		if (clonedLayer != null)			
			getMapContext().getLayers().addLayer(pos, clonedLayer);
		else
			getMapContext().getLayers().addLayer(pos, lyr);
		
		// mapcontrol is set to dirty when TOC is closed
		// to avoid dialog freezing causwd by redraw
		// setDirty();
	}
	
	public void removeLayer(FLayer lyr) {
		getMapContext().getLayers().removeLayer(lyr);
		// mapcontrol is set to dirty when TOC is closed
		// to avoid dialog freezing causwd by redraw
		// setDirty();
	}

	public void update(Graphics g){
		
		if (g != null) {
			paint(g);
		} else {
			logger.warn("Did not update graphics (g is NULL)");
		}
		
	}

	/**
	 * Mapcontrol is dirty if: Mapcontrol is resized
	 * or there is a dirty layer or
	 * viewport is dirty
	 * @return
	 */
	public boolean isDirty(){
		return (status == DIRTY);
	}
	
	public void paintAcetate(){
		
		if (mapGraphics == null || buffer ==null || bufferGraphics == null) {
			return;
		}
		
		bufferGraphics.drawImage(mapImage,0,0,null);
		
		try {
			MapContext mc = project.getMapContext();
			if (mc.getGraphicsLayer().isVisible()) {
				mc.getGraphicsLayer().draw(null, bufferGraphics, mc.getViewPort(), cancel, 0);
			}
		} catch (Exception e) {
			logger.error("While drawing acetate: " + e.getMessage());
		}
		status = UPDATED;
		repaint();
	}

	public boolean paintLayers() {
		//It refreshes the information stored in the buffer image
		if (!isDirty() || (mapGraphics == null))
			return false;
		
		// Graphics gaux = img.getGraphics();
		mapGraphics.setColor(getViewPort().getBackColor());
		mapGraphics.fillRect(0, 0, this.width, this.height);
		
		
		try {
			// GraphicLayer gl = this.getMapContext().getGraphicsLayer();
			boolean gl_visible = getMapContext().getGraphicsLayer().isVisible();

			long scale = getMapContext().getScaleView();
			
			// this.getMapContext().setGraphicsLayer(null);
			getMapContext().getGraphicsLayer().setVisible(false);
			this.getMapContext().draw(null, mapGraphics, cancel, scale);
			getMapContext().getGraphicsLayer().setVisible(gl_visible);
			// this.getMapContext().setGraphicsLayer(gl);
			// this.getMapContext().setGraphicsLayer(gl);
		} catch (Exception e1) {
			logger.error("While drawiung layers: " + e1.getMessage());
			e1.printStackTrace();
		}
		
		
		//repaint();
		status = UPDATED;
		panMode = false;
		setOffsetAndRefresh(0, 0, false);
		
		paintAcetate();
		
		return true;
		
		
	}
	
	public void paint(Graphics g) {
		
		if (buffer == null) {
			return;
		}
		g.clearRect(0, 0, getWidth(), getHeight());
		if(panMode) {
			g.drawImage(buffer, imageOffsetX, imageOffsetY, null);
		} else {
			g.drawImage(buffer, 0,0, null);
		}
	}
	

	
	public void setSize(int w, int h) {
		if (this.width==w && this.height==h)
			return;
		super.setSize(w, h);
		try {
			updateMapSize(w, h);
		} catch (CreateEnvelopeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
//	public void setSize(Dimension d){
//		super.setSize(d);
//		updateMapSize(d.width, d.height);
//	}
	
	private void updateMapSize(int w, int h) throws CreateEnvelopeException {
		if (w>0&&h>0){
			mapImage = ImageUtils.createImage(w, h);
			mapGraphics = (Graphics2D) mapImage.getGraphics();
			buffer = ImageUtils.createImage(w, h);
			bufferGraphics = (Graphics2D) buffer.getGraphics();
			
			ViewPort vp = project.getMapContext().getViewPort();
			Envelope currext = vp.getAdjustedExtent();
			Dimension dim = new Dimension(w, h);
	
			vp.setImageSize(dim);
			//mapContext.getViewPort().setClipRect(new Rectangle2D.Double(0,0,getWidth(),getHeight()));
			//mapContext.getViewPort().refreshExtent();
			
			if ((currext == null)) {
//				Envelope e = GeometryLocator.getGeometryManager().createEnvelope(0,0,w,h,SUBTYPES.GEOM2D);
//				vp.setEnvelope(e);
//				vp.refreshExtent();
			} else {
				Envelope newrect = getExtentFromFullExtentAndImageSize(currext, dim, true);
				vp.setEnvelope(newrect);
			}
			setDirty();
		}
	}


	public void setBackColor(Color c) {
		getViewPort().setBackColor(c);
		setDirty();
	}
	
    /**
    *
    * Computes the minimum rectangle with the width/height ratio that contains
    * the provided extent.
    *
    * @param fext
    *            the extent to be covered
    * @param imgdim
    *            the size of the screen
    * @return the minimum rectangle with the width/height ratio that contains
    *         the provided extent.
     * @throws CreateEnvelopeException 
    */
	// TODO: MOVE THIS METHOD TO MAPCONTEXT
   public static Envelope getExtentFromFullExtentAndImageSize(
		   Envelope fext,
		   Dimension imgdim,
		   boolean visible_remains) throws CreateEnvelopeException {
	   
	   
	   
       double ext_ratio = fext.getGeometry().getBounds2D().getWidth() / fext.getGeometry().getBounds2D().getHeight();
       double img_ratio = (1.0 * imgdim.width) / (1.0 * imgdim.height);

       if (ext_ratio == img_ratio) {
           // same ratio
           return fext;
       }
       else {
           if (ext_ratio > img_ratio) {
               // cinemascope
               double newexth = fext.getGeometry().getBounds2D().getWidth() / img_ratio;
               double yoff = 0.5 * (newexth - fext.getGeometry().getBounds2D().getHeight());

               return geomManager.createEnvelope(
            		   fext.getGeometry().getBounds2D().getMinX(),
            		   fext.getGeometry().getBounds2D().getMinY() - yoff,
            		   fext.getGeometry().getBounds2D().getMaxX(),
            		   fext.getGeometry().getBounds2D().getMaxY() + yoff,
            		   SUBTYPES.GEOM2D);
           }
           else {
               // canadian flag
               double newextw = fext.getGeometry().getBounds2D().getHeight() * img_ratio;
               double xoff = 0.5 * (newextw - fext.getGeometry().getBounds2D().getWidth());

               return geomManager.createEnvelope(
            		   fext.getGeometry().getBounds2D().getMinX() - xoff,
            		   fext.getGeometry().getBounds2D().getMinY(),
            		   fext.getGeometry().getBounds2D().getMaxX() + xoff,
            		   fext.getGeometry().getBounds2D().getMaxY(),
            		   SUBTYPES.GEOM2D);
           }
       }
   }

	public ViewPort getViewPort() {
		return project.getMapContext().getViewPort();
	}

	public void setDirty() {
		status = DIRTY;
		repaint();
	}

	public void extentChanged(ExtentEvent e, boolean repaint) {
		setDirty();
	}

	private void adjustEventToHandler(LwMouseEvent event) {
		Point2D point = new Point2D.Double(event.getX(),event.getY());
		Point2D mapPoint = getViewPort().toMapPoint(point);
		try {
			mapHandlerAdjustedPoint = adjustPointToHandler(mapPoint);
		} catch (Exception ex) {
			logger.warn("Strange exception caught: " + ex.getMessage());
			logger.warn("If this happened during editing then it should be investigated: " + ex.getMessage());
		}
	}
	public Point2D adjustPointToHandler(Point2D mapPoint) {
		if (mapControlManager == null) {
			mapControlManager = MapControlLocator.getMapControlManager();
		}
	    Point2D mapHandlerAdjustedPoint = null;
	    ArrayList layersToSnap = getMapContext().getLayersToSnap();
	    
	    double mapTolerance = getViewPort().toMapDistance(mapControlManager.getTolerance());
	    double minDist = mapTolerance;
	    double middleTol=mapTolerance * 0.5;
	    org.gvsig.fmap.geom.primitive.Envelope r;
	    com.vividsolutions.jts.geom.Envelope e = null;
	    try {
	        r = geomManager.createEnvelope(mapPoint.getX() - middleTol,
	                mapPoint.getY() - middleTol,
	                mapPoint.getX() + middleTol,
	                mapPoint.getY() + middleTol,
	                SUBTYPES.GEOM2D);
	        
	        e = Converter.convertEnvelopeToJTS(r);
	    } catch (CreateEnvelopeException e1) {
	        e1.printStackTrace();
	    }
	    
	    usedSnap = null;
	    for (int j = 0; j < layersToSnap.size(); j++)
	    {
	        FLyrVect lyrVect = (FLyrVect) layersToSnap.get(j);
	        SpatialCache cache = lyrVect.getSpatialCache();
	        if (lyrVect.isVisible())
	        {
	            // La lista de snappers est� siempre ordenada por prioridad. Los de mayor
	            // prioridad est�n primero.
	            java.util.List geoms = cache.query(e);
	            for (int n=0; n < geoms.size(); n++) {
	                Geometry geom = (Geometry) geoms.get(n);
	                for (int i = 0; i < mapControlManager.getSnapperCount(); i++){
	                    ISnapper theSnapper = mapControlManager.getSnapperAt(i);
	                    if (!theSnapper.isEnabled())
	                    	continue;
	                    if (usedSnap != null){
	                        // Si ya tenemos un snap y es de alta prioridad, cogemos ese. (A no ser que en otra capa encontremos un snapper mejor)
	                        if (theSnapper.getPriority() > usedSnap.getPriority())
	                            break;
	                    }
	                    Point2D theSnappedPoint = null;
	                    if (theSnapper instanceof ISnapperVectorial){
	                        theSnappedPoint = ((ISnapperVectorial) theSnapper).getSnapPoint(mapPoint, geom, mapTolerance);
	                    }
	                    if (theSnappedPoint != null) {
	                        double distAux = theSnappedPoint.distance(mapPoint);
	                        if (minDist > distAux)
	                        {
	                            minDist = distAux;
	                            usedSnap = theSnapper;
	                            mapHandlerAdjustedPoint=theSnappedPoint;
	                        }
	                    }
	                }
	            } // for n
	        } // visible
	    }
	    if (usedSnap != null){
	    	return mapHandlerAdjustedPoint;
		} else{
	    	return mapPoint;
		}
	}

	public Point2D getHandlerAdjustedPoint(){
		return mapHandlerAdjustedPoint;
	}
	
	public void mouseClicked(LwMouseEvent arg0) {
	    adjustEventToHandler(arg0);
	    Application.getInstance().getEvents().newEvent(arg0);
		
	}

	public void mouseEntered(LwMouseEvent arg0) {
		Application.getInstance().getEvents().newEvent(arg0);
	}

	public void mouseExited(LwMouseEvent arg0) {
		Application.getInstance().getEvents().newEvent(arg0);
		
	}

	public void mousePressed(LwMouseEvent arg0) {
		//for some reason we are loosing the ID of the event
		LwMouseEvent ev = new LwMouseEvent(
				this,LwMouseEvent.MOUSE_PRESSED,arg0.getX(),arg0.getY(),
				arg0.getMask(),arg0.getClickCount());
		adjustEventToHandler(ev);
		Application.getInstance().getEvents().newEvent(ev);
		
	}

	public void mouseReleased(LwMouseEvent arg0) {
		//for some reason we are loosing the ID of the event
		LwMouseEvent ev = new LwMouseEvent(
				this,LwMouseEvent.MOUSE_RELEASED,arg0.getX(),arg0.getY(),
				arg0.getMask(),arg0.getClickCount());
//		adjustEventToHandler(ev);
		Application.getInstance().getEvents().newEvent(ev);
		
	}

	public void endDragged(LwMouseEvent arg0) {
		Application.getInstance().getEvents().newEvent(arg0);
		
	}

	public void mouseDragged(LwMouseEvent arg0) {
		LwMouseEvent ev = new LwMouseEvent(
				this,LwMouseEvent.MOUSE_DRAGGED,arg0.getX(),arg0.getY(),
				arg0.getMask(),arg0.getClickCount());
		Application.getInstance().getEvents().newEvent(ev);
		
	}

	public void mouseMoved(LwMouseEvent arg0) {
		Application.getInstance().getEvents().newEvent(arg0);
	}

	public void startDragged(LwMouseEvent arg0) {
		Application.getInstance().getEvents().newEvent(arg0);
	}
	
	public ArrayList getLayersForReading() {
		ArrayList resp = new ArrayList();
		FLayers lyrs = getMapContext().getLayers();
		int cnt = lyrs.getLayersCount();
		for (int i=0; i<cnt; i++) {
			resp.add(lyrs.getLayer(i));
		}
		return resp;
	}

	
	public FLayer[] getArrayOfLayers() {
		
		FLayers lyrs = getMapContext().getLayers();
		int cnt = lyrs.getLayersCount();
		
		FLayer[] resp = new FLayer[cnt];
		for (int i=0; i<cnt; i++) {
			resp[i] = lyrs.getLayer(i);
		}
		return resp;
	}
	
	/**
	 * Tells the map control to recenter the map at the given pixel. It's the screen pixel
	 * where the user has clicked while using the recenter tool
	 *
	 * @param x_ coordinates to recenter to (screen pixel reference system)
	 * @param y_ coordinates to recenter to (screen pixel reference system)
	 */
	public void recenter(int x_, int y_){
		Envelope oldenv = getViewPort().getAdjustedEnvelope();
		
//		Rectangle2D oldext = getViewPort().getAdjustedExtent();
		if (oldenv == null) {
			return;
		}

		// ------------------------- starts to work
//		if (isProcessing()) {
//			return;
//		} else {
//			setProcessing(true);
//		}


		int src_x = x_;
		int src_y = y_;

//		logger.debug(Utils.time() + "CenterOnPointListener, point" + " =" + src_x + ", " + src_y); //$NON-NLS-1$ //$NON-NLS-2$

		Point2D geop = getViewPort().toMapPoint(src_x, src_y);

		double half_w = 0.5 * oldenv.getGeometry().getBounds2D().getWidth();
		double half_h = 0.5 * oldenv.getGeometry().getBounds2D().getHeight();
		Envelope env;
		try {
			env = geomManager.createEnvelope(geop.getX() - half_w, geop.getY() - half_h
					, geop.getX() + half_w, geop.getY() + half_h,
					SUBTYPES.GEOM2D);
			getViewPort().setEnvelope(env);
		} catch (CreateEnvelopeException e) {
			// TODO LOG THIS
			e.printStackTrace();
		}
//		Rectangle2D.Double newext = new Rectangle2D.Double(
//				geop.getX() - half_w, geop.getY() - half_h, 2 * half_w, 2 * half_h);
		
//		getViewPort().setExtent(newext, true);		

	}

	public void backColorChanged(ColorEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void extentChanged(ExtentEvent arg0) {
		setDirty();		
	}

	public void projectionChanged(ProjectionEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
	public MapContext getMapContext() {
		return project.getMapContext();
	}

	public void setMapContext(MapContext mapContext) {
		project.setMapContext(mapContext);
	}
	
	public void refreshAcetate() {
		status = ONLY_GRAPHICS;
//		paintAcetate();
//		repaint();
	}
	
//	public void draw(Point2D p) {
//		this.bufferGraphics.drawRect((int)p.getX(), (int)p.getY(), 5, 5);
//		refreshAcetate();
////		repaint();
//	}
	
	public FLayer getLayer(String layerName) {
		FLayers layers = project.getMapContext().getLayers();
		final int size = layers.getLayersCount();
		
		FLayer temp = null;
		for (int i = 0; i < size; i++) {
			temp = layers.getLayer(i);
			try {
				if (temp.getName().compareToIgnoreCase(layerName) == 0) {
					break;
				}
			} catch (Exception ignore) {
				
			}
		}
		
		return temp;
	}
	
	/**
	 * Deletes a layer with the given layerName
	 * @param layerName the name of the layer
	 */
	public void removeLayer(String layerName) {
		
		FLayer temp = this.getLayer(layerName);
		
		if (temp != null)
			project.getMapContext().getLayers().removeLayer(temp);
		
		// mapcontrol is set to dirty when TOC is closed
		// to avoid dialog freezing causwd by redraw
		// setDirty();
	}

	public void activationChanged(LayerEvent e) {
		// mapcontrol is set to dirty when TOC is closed
		// to avoid dialog freezing causwd by redraw
		// setDirty();
	}

	public void drawValueChanged(LayerEvent e) {
		// mapcontrol is set to dirty when TOC is closed
		// to avoid dialog freezing causwd by redraw
		// setDirty();
	}

	public void nameChanged(LayerEvent e) {
	}

	public void visibilityChanged(LayerEvent e) {
		// mapcontrol is set to dirty when TOC is closed
		// to avoid dialog freezing causwd by redraw
		// setDirty();
	}

	public void editionChanged(LayerEvent e) {
		setDirty();
	}

	public void initWith(Project pr) { // MapContext contxt, File projFile) {
		
		if (timer == null) {
			timer = new Timer();
			timer.scheduleAtFixedRate(timerTask, 0, DRAW_INTERVAL);
		}
		if (acetatetimer == null) {
			acetatetimer = new Timer();
			acetatetimer.scheduleAtFixedRate(acetateTask, ACETATE_DRAW_INTERVAL, ACETATE_DRAW_INTERVAL);
		}
		
		if (project != null) {
			project.dispose();
		}
		project = pr;
	}

	public Project getProject() {
		return project;
	}

	public void legendChanged(LegendChangedEvent e) {
		// mapcontrol is set to dirty when TOC is closed
		// to avoid dialog freezing causwd by redraw
		// setDirty();
	}
	
	public ArrayList getLayersForInfo() {
		
		ArrayList resp = new ArrayList();
		FLayers layers = project.getMapContext().getLayers();
		final int size = layers.getLayersCount();
		
		FLayer temp = null;
		for (int i = 0; i < size; i++) {
			temp = layers.getLayer(i);
			if (temp.hasDynValue(INFO_ACTIVATED_DYN_PROPERTY_NAME)) {
				if (((Boolean) temp.getDynValue(INFO_ACTIVATED_DYN_PROPERTY_NAME)).booleanValue()) {
					resp.add(temp);
				}
			}
		}
		return resp;
	}
	

	
}
