package org.gvsig.mobile.ui.awt.mapcontrol.impl;

import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;

import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.mobile.ui.awt.mapcontrol.snapping.AbstractSnapper;
import org.gvsig.mobile.ui.awt.mapcontrol.snapping.ISnapperVectorial;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.LineSegment;

/**
 * @author Vicente Caballero Navarro
 */
public class NearestPointSnapper extends AbstractSnapper implements ISnapperVectorial {
    public static GeometryManager geomManager = GeometryLocator.getGeometryManager();
    
    public NearestPointSnapper() {
		setPriority(2);
	}
    
    public Point2D getSnapPoint(Point2D point, Geometry geom, double tolerance) {
		Point2D resul = null;
		Coordinate c = new Coordinate(point.getX(), point.getY());

		PathIterator theIterator = geom.getPathIterator(null, geomManager.getFlatness()); //polyLine.getPathIterator(null, flatness);
		double[] theData = new double[6];
		double minDist = tolerance;
		Coordinate from = null, first = null;
		while (!theIterator.isDone()) {
			//while not done
			int theType = theIterator.currentSegment(theData);

			switch (theType) {
				case PathIterator.SEG_MOVETO:
					from = new Coordinate(theData[0], theData[1]);
					first = from;
					break;

				case PathIterator.SEG_LINETO:

					Coordinate to = new Coordinate(theData[0], theData[1]);
					LineSegment line = new LineSegment(from, to);
					Coordinate closestPoint = line.closestPoint(c);
					double dist = c.distance(closestPoint);
					if ((dist < minDist)) {
						resul = new Point2D.Double(closestPoint.x, closestPoint.y);
						minDist = dist;
					}

					from = to;
					break;
				case PathIterator.SEG_CLOSE:
					line = new LineSegment(from, first);
					closestPoint = line.closestPoint(c);
					dist = c.distance(closestPoint);
					if ((dist < minDist)) {
						resul = new Point2D.Double(closestPoint.x, closestPoint.y);
						minDist = dist;
					}

					from = first;
					break;


			} //end switch

			theIterator.next();
		}

		return resul;
	}

	public String getToolTipText() {
		return "nearest_point";
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.fmap.mapcontrol.tools.snapping.snappers.ISnapper#draw(org.gvsig.fmap.mapcontrol.PrimitivesDrawer, java.awt.geom.Point2D)
	 */
//	public void draw(PrimitivesDrawer primitivesDrawer, Point2D pPixels) {
//		primitivesDrawer.setColor(getColor());
//		int half = getSizePixels() / 2;
//		int x1 = (int) (pPixels.getX() - half);
//		int x2 = (int) (pPixels.getX() + half);
//		int y1 = (int) (pPixels.getY() - half);
//		int y2 = (int) (pPixels.getY() + half);
//
//		primitivesDrawer.drawLine(x1, y1, x2, y1); // abajo
//		primitivesDrawer.drawLine(x1, y2, x2, y2); // arriba
//		primitivesDrawer.drawLine(x1, y1, x2, y2); // abajo - arriba
//		primitivesDrawer.drawLine(x1, y2, x2, y1); // arriba - abajo
//	}
}
