package org.gvsig.mobile.ui.awt.tools;

import java.awt.Dimension;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.gvsig.mobile.core.auxfunctions.ShowMessage;
import org.gvsig.mobile.core.auxfunctions.StringConstructor;
import org.gvsig.mobile.core.concepts.ComboTool;
import org.gvsig.mobile.core.concepts.DefaultToolBarFunctionality;
import org.gvsig.mobile.core.concepts.HelpTool;
import org.gvsig.mobile.core.exceptions.GvSigPropertyNotAttachedException;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.tool.ToolPool;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LayoutContainer;
import org.zaval.lw.Layoutable;
import org.zaval.lw.LwLayout;
import org.zaval.lw.LwPanel;

public class ToolBarGroup extends LwPanel implements TaskStatusListener {

	private Hashtable toolbars = new Hashtable();
	ToolPool toolPool = null; // holds every tool
	public ToolPool getToolPool() {
		return toolPool;
	}

	Hashtable helpTools = new Hashtable();
	ToolPool leftToolPool;
	ToolBar leftToolbar;
	ToolBar activeToolbar;
	BundleContext bc;

	public ToolBarGroup(BundleContext context) {
		this.bc = context;
		ShowMessage.Show("----------------", true);
		setLwLayout(new TBGLayout());
		// setBounds(10, 10, 500, 300);
		toolPool = new ToolPool();

		leftToolPool = new ToolPool();
		leftToolbar = new RollingToolBar("TOOLBARGROUP",
				ToolBar.ALIGNEMENT_VERTICAL);
		leftToolbar.setAlignment(ToolBar.ALIGNEMENT_VERTICAL);
		add(leftToolbar);
		setOpaque(false);
	}

	//	
	// public void paint(Graphics g){
	// for (int i=0; i<count(); i++){
	// LwComponent comp = (LwComponent)get(i);
	// g.translate(comp.getX(), comp.getY());
	// comp.paint(g);
	// }
	// }

	public HelpTool addHelpTool(ITool tool, ToolBar toolbar) {

		Vector tools = new Vector();
		tools.add(tool);

		BundleContext bc = tool.getBundleContext();

		HelpTool helpTool = new HelpTool(tools, toolbar.getKey(), toolbar
				.getKey()
				+ "_HELP", bc);

		// the functionality of the helptool does nothing but broadcasting
		// its state
		DefaultToolBarFunctionality f = new DefaultToolBarFunctionality(toolbar
				.getKey(), bc);

		helpTool.setFunctionalityID(f.getID());

//		bc.registerService(IFunctionality.class.getName(), f, f.getProps());

		ToolCombo helpComponent = new ToolCombo(helpTool, (LwPanel) this
				.getLwParent(), true);
		toolbar.helpComponent = helpComponent;
		// toolbar.addTool(helpComponent);
		toolbar.addTool(helpComponent.getDropDown());

		toolPool.addTool(toolbar.helpComponent.getDropDown().getTool());

		try {
			helpTools.put(helpTool.getKey(), helpTool);
		} catch (GvSigPropertyNotAttachedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return helpTool;
	}

	public synchronized void addTool(ITool tool) {
		try {
			ShowMessage.Show("ADDING TOOL: "+tool.getKey(), true);
		} catch (GvSigPropertyNotAttachedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		// we allow introducing tools before its toolbar has been
		// added to reduce the problems with the extensions loading order

		//we use this flag to check whether the toolbar has been created for
		//the first time or not
		boolean createdToolbar = false;
		try {
			ToolBar toolbar = null;
			if (!toolbars.containsKey(tool.getToolbarKey())) {
				toolbar = new ToolBar(tool.getToolbarKey(),
						ToolBar.ALIGNEMENT_HORIZONTAL);
				toolbars.put(tool.getToolbarKey(), toolbar);
				ShowMessage.Show(StringConstructor.Construct(new String[] {
						"from addTool ", tool.getKey() }), true);

				add((ToolBar) toolbars.get(tool.getToolbarKey()));
				createdToolbar = true;
			}
			
			if (toolbar == null)
				toolbar = (ToolBar) toolbars.get(tool.getToolbarKey());
			
			
			toolPool.addTool(tool);

			//we check if there is a component at the position of the tool
			//we are going to insert
			ToolComponent comp = toolbar.getToolComponentAt(tool.getPosition());

			//if the position is free we simply add the toolcomponent to the toolbar
			if (comp == null) {
				ToolComponent tw2 = new ToolComponent(tool);
				toolbar.addTool(tw2);
			//if the position is occupied by another component
			} else {
				//The component can be a Toolcombo so we add the tool to the Toolcombo
				if (comp instanceof ToolCombo) {
					//TODO: ALBERTO -> use constants for that kind of tool keys
					ITool t = toolPool.getTool(toolbar.getKey()+"_COMBO_"+tool.getPosition());
					if (!(t instanceof ComboTool)) {
						//TODO: ALBERTO -> This should not have sense
					} else {
						((ComboTool)t).addTool(tool);		
						if (tool.getFuncSolved() != null)
							tool.getFuncSolved().addObserver(comp);
					}
				//if the position is occupied by another ToolComponent we have to 
				//create a ToolCombo with the previous ToolComponent and the one
				//we are inserting
				} else if (comp instanceof ToolComponent){
					// Remove the previous toolcomponent
					toolbar.removeTool(comp);
					toolPool.removeTool(comp.getTool().getFunctionalityID());					
					
					Vector tools = new Vector();
					tools.add(comp.getTool());
					tools.add(tool);
						
					ComboTool newCombo = new ComboTool(tools, toolbar.getKey(), toolbar.getKey()+"_COMBO_"+tool.getPosition(), bc);
					
					// the functionality of the helptool does nothing but broadcasting
					// its state
					DefaultToolBarFunctionality f = new DefaultToolBarFunctionality(toolbar.getKey()+"_COMBO_"+tool.getPosition(), bc);

					newCombo.setFunctionalityID(f.getID());

					bc.registerService(IFunctionality.class.getName(), f, f.getProps());					
					
					ToolCombo tw = new ToolCombo(newCombo, (LwPanel)this.getLwParent(), false);
					
					// We add the ToolCombo and the DropDownComponent in order to show the
					// tools in the combo
					this.addComboTools(newCombo);
					toolbar.addTool(tw);
					toolbar.addTool(tw.getDropDown());
					toolPool.addTool(newCombo);		
					toolPool.addTool(tw.getDropDown().getTool());								
				}
			}

			// if a new toolbar has been created we add the helptool to it
			if (createdToolbar) {
				addHelpTool(tool, toolbar);				
			} else {
				// TODO: ALBERTO
				// se aade el tool al helptool (OJO a la posicin)
				HelpTool h = (HelpTool) helpTools.get(toolbar.getKey()
						+ "_HELP");
				h.addTool(tool);
				if (tool.getFuncSolved() != null)
					tool.getFuncSolved().addObserver(toolbar.helpComponent);
			}
		} catch (GvSigPropertyNotAttachedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		repaint();
	}
	
	private void addComboTools(ComboTool c) {		
		Vector tools = c.getTools();
		final int size = tools.size();
		
		for(int i = 0; i<size; i++) {			
			toolPool.addTool((ITool)tools.elementAt(i));
		}
	}

	public void removeTool(ITool tool) {
		/*
		 * TODO: ALBERTO -> ADEMS HABR QUE QUITAR LOS TASKSTATUSLISTENER
		 * 
		 */
		ShowMessage.Show("REMOVING TOOL", true);
		ToolBar toolbar;
		try {
			//the toolbar where the tool is placed
			toolbar = (ToolBar) toolbars.get(tool.getToolbarKey());
			
			//the component we want to remove
			ToolComponent tc = (ToolComponent)toolbar.getToolComponentAt(tool.getPosition());
			
			//first we check if the component retrieved is a ToolCombo
			if (tc instanceof ToolCombo) {
				ToolCombo tcomb = (ToolCombo)tc;
				
				// this don't have much sense
				if (tcomb.numOfTools() == -1) return;

				//si tiene ms de dos elementos se quita del combotool
				else if (tcomb.numOfTools() > 2) {
					ITool t = toolPool.getTool(toolbar.getKey()+"_COMBO_"+tool.getPosition());
					if (!(t instanceof ComboTool)) {
						//TODO: do something
					} else {
						toolPool.removeTool(tool.getFunctionalityID());
						
						((ComboTool)t).removeTool(tool);
						HelpTool h = (HelpTool) helpTools.get(tool.getToolbarKey()
								+ "_HELP");
						h.removeTool(tool);
					}
				//si tiene exactamente dos elementos hay que quitar el toolcombo y crear un tool
				//component con el tool que queda
				} else if (tcomb.numOfTools() == 2) {
					toolbar.removeTool(tcomb);
					toolbar.removeTool(tcomb.getDropDown());
					
					ComboTool t = (ComboTool)toolPool.getTool(toolbar.getKey()+"_COMBO_"+tool.getPosition());
					t.removeTool(tool);
					
					toolPool.removeTool(t.getKey());
					toolPool.removeTool(tcomb.getDropDown().getTool().getKey());
					
					// We add the ToolCombo and the DropDownComponent in order to show the
					// tools in the combo
//					this.addComboTools(newCombo);
//					
//					toolPool.addTool(tw.getDropDown().getTool());
//					toolPool.addTool(newCombo);
					
					ToolComponent tw2 = new ToolComponent(t.getToolAt(0));
					toolbar.addTool(tw2);
					
					HelpTool h = (HelpTool) helpTools.get(tool.getToolbarKey()
							+ "_HELP");
					h.removeTool(tool);
				}
			//if the component is not a toolcombo we simply remove it from the toolbar and
			//the help tool component
			} else if (tc instanceof ToolComponent) {
			
				toolPool.removeTool(tool.getFunctionalityID());
				//TODO: ALBERTO -> Usar constantes
				HelpTool h = (HelpTool) helpTools.get(tool.getToolbarKey()
						+ "_HELP");
				h.removeTool(tool);
				toolbar.removeTool(tool.getKey());
			}
		} catch (GvSigPropertyNotAttachedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// ToolComponent tw = new ToolComponent(tool);
		// try {
		//			
		// } catch (GvSigPropertyNotAttachedException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }

		repaint();

	}

	public void addToolBarButton(ToolBarButton tbb) {

		leftToolPool.addTool(tbb.getTool());
		leftToolbar.addTool(tbb);
		
//		this.height = this.leftToolbar.count() * tbb.getHeight();
//		this.setSize(this.width, this.height);

		/**
		 * each toolbar must perform a IFunctionality
		 * 
		 * @see <code>DefaultToolBarFunctionality</code>
		 */
		IFunctionality f = tbb.getTool().getFuncSolved();
		if (f != null)
			f.addObserver(this);
	}

	public void removeToolBarButton(String key) {

		leftToolPool.removeTool(key);

		if (this.activeToolbar == this.toolbars.get(key)) {
			activeToolbar = null;
		}

		leftToolbar.removeTool(key);
		this.getLwLayout().layout(this);
	}

	private void activateToolbar(String key) {
		Object tbo = toolbars.get(key);
		if (tbo != null) {
			ToolBar tb = (ToolBar) tbo;
			activeToolbar = tb;
			this.getLwLayout().layout(this);
		}
	}

	public void taskFinished(ITask task) {
	}

	public void taskInited(ITask task) {

		// a toolbar button has been released and we activate its
		// toolbar
		activateToolbar(((IFunctionality) task).getID());
	}

	public void taskProgress(ITask task, int progress) {
	}

	class TBGLayout implements LwLayout {
		public TBGLayout() {

		}

		public void componentAdded(Object id, Layoutable lw, int index) {

		}

		public void componentRemoved(Layoutable lw, int index) {

		}

		public Dimension calcPreferredSize(LayoutContainer target) {
			return new Dimension(target.getWidth(), target.getHeight());
		}

		public synchronized void layout(LayoutContainer target) {
			// locate all components

			if (leftToolbar == null || toolbars.size() == 0)
				return;

			if (activeToolbar == null) {
				try {
//					activeToolbar = (ToolBar)((ToolComponent) leftToolbar.getTools()
//							.next()).getTool();
					
					boolean found = false;
					Iterator it = leftToolbar.getTools();
					
					ToolBarButton tbb = null;
					while (!found && it.hasNext()) {
						tbb = ((ToolBarButton)it.next());
						if(toolPool.hasTools(tbb.getTool().getKey())) {
							found = true;
						}
					}
					
					// activeToolbar = (ToolBar) toolbars.get("SAMPLE");
					// ((ToolComponent)leftToolbar.getTools().next()).getTool().getFuncSolved().run();
//					IFunctionality f = ((ToolComponent) leftToolbar.getTools()
//							.next()).getTool().getFuncSolved();
					if(found) {
						IFunctionality f = tbb.getTool().getFuncSolved();  
							if (f != null)
								f.run();
					}
					
				} catch (Exception ignored) {
					
				}
			}

			if (activeToolbar == null) {
				return;
			}

			Iterator it = toolbars.keySet().iterator();
			while (it.hasNext()) {
				ToolBar tb = (ToolBar) toolbars.get(it.next());
				tb.setVisible(false);
			}
			
			activeToolbar.setVisible(true);

			leftToolbar.setLocation(getInsets().left, getInsets().top);

			leftToolbar.setSize(leftToolbar.getPreferredSize().width,
					leftToolbar.getPreferredSize().height);
			leftToolbar.validate();
			activeToolbar.setLocation(leftToolbar.getX()
					+ leftToolbar.getWidth(), getInsets().top);
			activeToolbar.setSize(width - getInsets().left - getInsets().right
					- activeToolbar.getX(),
					activeToolbar.getPreferredSize().height);
		}
	}
}
