package org.gvsig.mobile.ui.awt.tools;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;

import org.gvsig.mobile.core.exceptions.GvSigPropertyNotAttachedException;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.tasks.WorkQueue;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwCanvas;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.event.LwMouseListener;

public class ToolComponent extends LwCanvas implements LwMouseListener,
		TaskStatusListener {

	static private Logger logger = LoggerFactory.getLogger(ToolComponent.class);

	protected ITool tool;
	boolean active = false;
	protected int width = 0;
	protected int height = 0;
	Color activeColor = new Color(255, 174, 0, 160);
	Color inActiveColor = new Color(0, 0, 0, 255);
	
	protected Image img_black;
	protected Image img_orange;

	public void paint(Graphics g) {
		


		// setOpaque(false);
		if (active) {
			g.drawImage(img_orange, 0, 0, null);
			// g.setColor(activeColor);
		} else {
			g.drawImage(img_black, 0, 0, null);
			// g.setColor(inActiveColor);
		}
		// g.fillRect(0, 0, width, height);
				
	}

	public ToolComponent(ITool tool) {

		this.tool = tool;
		setOpaque(false);

		// we want to be notified when the functionality starts and ends
		IFunctionality f = tool.getFuncSolved();
		if (f != null)
			f.addObserver(this);

		String strkey = "";
		try {
			strkey = tool.getKey(); 
//			this.img = ImageUtils.getResourceImage(tool.getBundleContext().getBundle().getLocation().substring(23), tool.getIcon());
			if (strkey.indexOf("DROP") != -1 ||
					strkey.indexOf("HELP") != -1) {
				this.img_orange = ImageUtils.getResourceImage(tool.getIcon()); //  + ".png.orange");
				this.img_black = ImageUtils.getResourceImage(tool.getIcon()); //  + ".png.black");
			} else {
				this.img_orange = ImageUtils.getResourceImage(tool.getBundleContext().getBundle(), tool.getIcon() + ".png.orange");
				this.img_black = ImageUtils.getResourceImage(tool.getBundleContext().getBundle(), tool.getIcon() + ".png.black");
			}
			// we allow having different sized ToolComponents (for example
			// DropDownComponent)
			width = this.img_black.getWidth(null);
			height = this.img_black.getHeight(null);

		} catch (GvSigPropertyNotAttachedException e) {
			logger.error("While getting icons: " + e.getMessage());
		}

		if (width < 0) {
			logger.error("PROBLEM WITH ICON: " + strkey);
			System.err.println("PROBLEM WITH ICON: " + strkey);
		}
		
		setPSSize(width, height);
		setSize(width, height);
	}
	
	protected ToolComponent() {
		// used by subclasses who do not need other constructor
	}

	public void start() {	
		if (tool.getFuncSolved() != null)
			WorkQueue.getInstance().execute(tool.getFuncSolved());
		repaint();
	}

	/*
	 * Stop all tasks initiated by this tool
	 */
	public void stop() {
		if (tool.getFuncSolved() != null)
			tool.getFuncSolved().stop();
		repaint();
	}

	public void changeState() {
		// System.out.println("State changed");
		if (active) {
			// TODO: if it's a RUN_INMEDIATELY_TOOL ask the user if he wants
			// to interrupt the task
			stop();
		} else {
			// System.out.println("start");
			start();
		}
	}

	public void mouseClicked(LwMouseEvent e) {
	}

	public void mouseEntered(LwMouseEvent e) {
	}

	public void mouseExited(LwMouseEvent e) {
	}

	public void mousePressed(LwMouseEvent e) {
	}

	public void mouseReleased(LwMouseEvent e) {
		// when the mouse is released the functionality starts
		// and every observer (TaskStatusListener) is notified
		changeState();
	}

	public void taskFinished(ITask task) {		
		active = false;
		repaint();
	}

	public void taskInited(ITask task) {		
		active = true;		
		repaint();
	}

	public void taskProgress(ITask task, int progress) {
	}

	public ITool getTool() {
		return tool;
	}

	public boolean isActive() {
		return active;
	}
}
