package org.gvsig.mobile.ui.lw;

import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Hashtable;

import org.gvsig.compat.CompatLocator;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.auxfunctions.ResourceReader;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.Persistent;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;
import org.slf4j.LoggerFactory;

public class AppSettings implements Persistent {

	final static private org.slf4j.Logger logger = LoggerFactory.getLogger(FLyrVect.class);
	
	public static Hashtable editingPrefs  = new Hashtable();
	
	/**
	 * The default SRS used by the application
	 */
	public static String defaultSRS = "EPSG:4326";
	
	/**
	 * The default initial bounding box used by the application
	 */
	public static Rectangle2D defaultBBox = new Rectangle2D.Double(-180, -90, 360, 180);
	
	/**
	 * The default project path used by the application
	 */
	public static File defaultSavePath =
		new File (System.getProperty("user.home") + File.separator + "gvm_projects");
	public static String defaultProjectFileName = "defaultProject.gvm";
	
	/**
	 * The default User Forms path used by the application
	 */
	public static File defaultFormsSavePath = new File (System.getProperty("user.home"));
	
	/**
	 * Default path to open any FileChooserDialog
	 */
	public static String defaultPath = System.getProperty("user.dir");
	public static String lastPath = defaultPath;

	/**
	 * The default waypoint path used by the application
	 */
	public static String defaultWaypointPath = System.getProperty("user.home") + File.separator + "gvm_projects";
	/**
	 * The default tracklog path used by the application
	 */
	public static String defaultTracklogPath = System.getProperty("user.home") + File.separator + "gvm_projects";
	/**
	 * The default raw data path used by the application
	 */
	public static String defaultRawDataPath = System.getProperty("user.home") + File.separator + "gvm_projects";
	
	/**
	 * The default background color used by the application
	 */
	public static Color defaultBgColor = Color.WHITE;
	
	/**
	 * The default icon prefix used by the application
	 */
	public static String size_Factor = "";
	
	public static String defSizeFactor = "24x24";
	
	/**
	 * The default suffix used by the application for tracklog files (GPS functionality)
	 */
	public static String tracklogSuffix = "_tracklog";
	
	/**
	 * The current suffix used by the application for tracklog files (GPS functionality)
	 */
	public static String waypointsSuffix = "_waypoints";
	
	/**
	 * The current locale (language) used by the application
	 */
	public static String localeLanguage = "en";
	
	/**
	 * The default locale (language) used by the application
	 */
	public static String defaultLocaleLanguage = "en";
	
	static {
		
		AppSettings.registerPersistent();
		load(new AppSettings());
		
		if (!isAvailableLocale(localeLanguage)) {
			localeLanguage = defaultLocaleLanguage;
		}
	}
		
	/**
	 * The default value for whether a default startup project is declarted
	 */
	public static boolean useDefaultProject = false;
	
	/**
	 * The default project opened by application at startup
	 */
	public static File defaultProject = null;	

	/**
	 * Utility String used to store without problems WMS URLs that have ampersands
	 */
	public static final String AMPERSAND_REPLACED = "_AMPERSAND_";
	
	/**
	 * Max. number of WMS servers to be stored in the persistence file. 
	 */
	public static final int WMS_SERVERS_PERSISTED_MAX_COUNT = 20;
	
	/**
	 * The name of the persistence file. Will be stored in user home folder.
	 */
	public static final String APP_SETTINGS_FILE_NAME = "gvsig-mobile-settings.txt";
	
	/**
	 * The name of the persistence file. Will be stored in user home folder.
	 */
	public static final long[] MAX_MEM_SHP_SIZE_VAL = {
		100000,500000,2000000,5000000,15000000,Long.MAX_VALUE
	};
	public static final String[] MAX_MEM_SHP_SIZE_STR = {
		"100 KB","500 KB","2 MB","5 MB","15 MB","NO LIMIT"
	};
	public static long maxMemShpSize = 5000000;

	
//	/**
//	 * The folder where the persistence file is stored 
//	 */
//	public static final File APP_SETTINGS_PERSISTENCE_FOLDER_ =
//		IconResourceReader.getWorkingDirFile("persistence", "appsettings.xml");
	

	private static String checkSizeFactor(String txt) {
		int len = txt.length();
		if ((len < 4) || (txt.toLowerCase().indexOf("x") != 0)) {
			return defSizeFactor; 
		} else {
			return txt;
		}
	}

	private static String[] checkWmsServers(String _txt) {
		String txt = restoreAmpersand(_txt);
		return CompatLocator.getStringUtils().split(txt, ";"); // Utilities.d();
	}

	private static File checkFile(String txt) {
		File f = null;
		try {
			f = new File(txt);
		} catch (Exception bad_file_ex) { }
		return f;
	}

	private static boolean checkBoolean(String txt) {
		return (txt.compareTo("1") == 0); 
	}

//	private static File checkPath(String txt) {
	private static void checkPath(File file, String txt) {
		
		try {
//			defaultSavePath = new File(txt);
			file = new File(txt);
		} catch (Exception ex) {
//			//logger.error("While reading persisted path: " + ex.getMessage());
		}
//		return defaultSavePath;
	}

	private static String checkEPSG(String txt) {
		if (txt.length() < 6) return defaultSRS;
		if (txt.substring(0, 5).compareToIgnoreCase("epsg:") != 0) return defaultSRS;
		// ok:
		return txt;
	}

	private static Color splitToColor(String txt) {
		
		String[] rgb = CompatLocator.getStringUtils().split(txt, ",");
		if (rgb.length != 3) return defaultBgColor;
		
		int r = 0, g = 0, b = 0;
		
		try {
			r = Integer.parseInt(rgb[0]);
			if (r < 0 || r > 255) throw new Exception("Bad R value: " + r);
			g = Integer.parseInt(rgb[1]);
			if (g < 0 || g > 255) throw new Exception("Bad G value: " + g);
			b = Integer.parseInt(rgb[2]);
			if (b < 0 || b > 255) throw new Exception("Bad B value: " + b);
		} catch (Exception ex) {
//			//logger.debug("Invalid band values: " + ex.getMessage());
			return defaultBgColor;
		}

		return new Color(r, g, b);
	}

	private static Rectangle2D splitToRectangle2D(String txt) {
		String[] xywh = CompatLocator.getStringUtils().split(txt, ",");
		if (xywh.length != 4) return defaultBBox;
		
		double x = 0;
		double y = 0;
		double w = 0;
		double h = 0;

		try {
			x = Double.parseDouble(xywh[0]);
			y = Double.parseDouble(xywh[1]);
			
			w = Double.parseDouble(xywh[2]);
			if (w <= 0) throw new Exception("Bad W value: " + w);
			h = Double.parseDouble(xywh[3]);
			if (h <= 0) throw new Exception("Bad H value: " + h);
		} catch (Exception ex) {
//			//logger.debug("Invalid BBOX values: " + ex.getMessage());
			return defaultBBox;
		}

		return new Rectangle2D.Double(x, y, w, h);
	}

//	/**
//	 * Stores the current application settings into the persistence file.
//	 * 
//	 * @throws GvsigMobileException
//	 */
//	public static void store() throws GvsigMobileException {
//		
//		try {
//			File dir = new File(APP_SETTINGS_PERSISTENCE_FILE.getParent());
//			dir.mkdirs();
//			APP_SETTINGS_PERSISTENCE_FILE.createNewFile();
//		} catch (IOException e) {
//			throw new GvsigMobileException("While creating persistence file: " + e.getMessage());
//		}
//		
//		XmlBuilder xml = new XmlBuilder(true);
//		
//		try {
//			xml.writeHeader();
//			
//			xml.openTag(XmlProjectTags.PERSISTENCE);
//			
//			xml.writeTag(XmlProjectTags.DEFAULT_SRS, defaultSRS);
//			xml.writeTag(XmlProjectTags.DEFAULT_BBOX, toString(defaultBBox));
//			xml.writeTag(XmlProjectTags.DEFAULT_SAVE_PATH, defaultSavePath.toString());
//			xml.writeTag(XmlProjectTags.DEFAULT_BG_COLOR, toString(defaultBgColor));
//			xml.writeTag(XmlProjectTags.SIZE_FACTOR, sizeFactor);
//			
//			xml.writeTag(XmlProjectTags.DEFAULT_PROJECT_USED, toString(useDefaultProject));
//			xml.writeTag(XmlProjectTags.DEFAULT_PROJECT_FILE, toString(defaultProject));
//			
//			xml.writeTag(XmlProjectTags.APP_LANGUAGE, localeLanguage);
//
//			xml.writeTag(XmlProjectTags.GPS_BAUD_RATE, toString(GPSManager.getGPSManager().getBaudRate()));
//			xml.writeTag(XmlProjectTags.GPS_PORT, GPSManager.getGPSManager().getPortName());
//			
//			xml.writeTag(XmlProjectTags.DEFAULT_WAYPOINT_PATH, defaultWaypointPath.toString());
//			xml.writeTag(XmlProjectTags.DEFAULT_TRACKLOG_PATH, defaultTracklogPath.toString());
//			xml.writeTag(XmlProjectTags.DEFAULT_RAWDATA_PATH, defaultRawDataPath.toString());
//			
//			// wmsServers
//			xml.writeTag(XmlProjectTags.WMS_SERVERS, toString(wmsServers, ";"));
//			
//			
//			//TODO: COMPLETE GPS SETTINGS
//			xml.closeTag();
//
//			BufferedWriter bw = new BufferedWriter(new FileWriter(APP_SETTINGS_PERSISTENCE_FILE));
//			bw.write(xml.getXML());
//			bw.flush();
//			bw.close();
//			bw = null;
//		} catch (IOException e) {
//			logger.error("While writing persistence: " + e.getMessage());
//		} 		
//
//	}

	private static String toString(boolean b) {
		
		if (b) {
			return "1";
		} else {
			return "0";
		}
	}
	
	private static String toString(int i) {
		return "" + i;
	}

	private static String toString(File f) {
		if (f == null) return ""; 
		return f.getAbsolutePath();
	}

	private static String toString(Color c) {
		return c.getRed() + "," + c.getGreen() + "," + c.getBlue(); 
	}

	private static String toString(Rectangle2D r) {
		return r.getMinX() + "," + r.getMinY() + "," + r.getWidth() + "," + r.getHeight(); 
	}
	
	private static String toString(String[] ss, String sep) {

		if (ss == null) return "";
		if (ss.length == 0) return "";
		
		String resp = "";
		int sz = ss.length;
		if (sz > (WMS_SERVERS_PERSISTED_MAX_COUNT + 1)) {

			for (int i=0; i<WMS_SERVERS_PERSISTED_MAX_COUNT; i++) {
				resp = resp + ss[i] + sep;
			}
			resp = resp + ss[WMS_SERVERS_PERSISTED_MAX_COUNT];

		} else {
			
			int last_index = ss.length - 1;
			
			for (int i=0; i<last_index; i++) {
				resp = resp + ss[i] + sep;
			}
			resp = resp + ss[last_index];

		}
		
		return avoidAmpersand(resp);
		
	}
	
	private static String avoidAmpersand(String str) {
		return CompatLocator.getStringUtils().replaceAll(str, "&", AMPERSAND_REPLACED);
	}
	
	private static String restoreAmpersand(String str) {
		return CompatLocator.getStringUtils().replaceAll(str, AMPERSAND_REPLACED, "&");
	}
	
	/**
	 * Utility method to find out if a particular locale (language) is available in the
	 * current installation-
	 * 
	 * @param loc the locale of interest
	 * @return whether a particular locale (language) is available
	 */
    public static boolean isAvailableLocale(String loc) {
    	
    	String av[] = getAvailableLanguages();
    	for (int i=0; i<av.length; i++) {
    		if (loc.compareToIgnoreCase(av[i]) == 0) return true; 
    	}
    	return false;
    }

    /**
     * Gets an array with the available locales
     * 
     * @return an array with the available locales
     */
	public static String[] getAvailableLanguages() {
		
		File lang_dir = IconResourceReader.getLanguageFolder(ImageUtils.CORE_FOLDER, "languages");
		String[] files = lang_dir.list();
		if ((files == null) || (files.length == 0)) {
			return new String[0];
		}
		
		ArrayList resp = new ArrayList();
		for (int i=0; i<files.length; i++) {
			
			String item_file = files[i].toLowerCase();
			
			int pt_prop = item_file.indexOf(".properties"); 
			if ((item_file.indexOf("text_") == 0) 
				&& pt_prop != -1) {
				
				resp.add(item_file.substring(item_file.indexOf("_")+1, pt_prop));
			}
		}
		
		return (String[]) resp.toArray(new String[0]);
	}
	
	/**
	 * gets a subfolder from the resources folder
	 * @param dir the subfolder name
	 * @return the file object
	 */
	public static File getResourceFolder(String dir) {
		String filePath = ResourceReader.resourceDir + File.separator + dir;
		
		File resp = new File(filePath);
		return resp;
	}
	
	
	/**
	 * The WMS servers added to the WMS combo box
	 */
	public static String[] wmsServers = new String[]{"http://www.idee.es/wms/IDEE-Base/IDEE-Base",
		"http://ovc.catastro.meh.es/Cartografia/WMS/ServidorWMS.aspx"};

	public static String[] wfsServers = new String[]{"http://www.idee.es/IDEE-WFS/ogcwebservice",
		"http://ideg.xunta.es/wfs/request.aspx"};

	public static String[] sosServers = new String[]{"http://gis.prodevelop.es/sos/sos", 
		"http://localhost:8080/52nSOSv3_WAR/sos"};
	
	public void loadFromState(PersistentState state)
			throws PersistenceException {

		try {
			localeLanguage = state.getString("language");
		} catch (Exception ex) {
			logger.error("While reading app language settings : " + ex.getMessage());
		}

		try {
			size_Factor = state.getString("iconSize");
		} catch (Exception ex) {
			logger.error("While reading app icon size settings: " + ex.getMessage());
		}
		try {
			defaultSRS = state.getString("defaultSrs");
		} catch (Exception ex) {
			logger.error("While reading app default srs settings: " + ex.getMessage());
		}
		try {
			defaultSavePath = new File(state.getString("defSavePath"));
		} catch (Exception ex) {
			logger.error("While reading app default save path settings: " + ex.getMessage());
		}
		try {
			defaultBgColor = AppSettings.splitToColor(state
					.getString("defBgColor"));
		} catch (Exception ex) {
			logger.error("While reading app default background color settings: " + ex.getMessage());
		}
		try {
			double bboxminx = state.getDouble("bboxminx");
			double bboxminy = state.getDouble("bboxminy");
			double bboxw = state.getDouble("bboxw");
			double bboxh = state.getDouble("bboxh");
			defaultBBox = new Rectangle2D.Double(bboxminx, bboxminy, bboxw,
					bboxh);
		} catch (Exception ex) {
			logger.error("While reading app default bounding box settings: " + ex.getMessage());
		}
		try {
			useDefaultProject = state.getBoolean("useDefProject");
			if(useDefaultProject)
				defaultProject = new File(state.getString("defProject"));
		} catch (Exception ex) {
			logger.error("While reading app user default project settings: " + ex.getMessage());
		}
		
		
		try {
			maxMemShpSize = state.getLong("maxMemShpSize");
		} catch (Exception ex) {
			logger.warn("While reading app max memory shp size settings: " + ex.getMessage());
		}

	}
	
	public static void registerPersistent() {
		DynStruct definition =
			 ToolsLocator.getPersistenceManager().addDefinition(AppSettings.class,
					 "AppSettings_Persistent",
						"AppSettings Persistent definition",
	                null,
	                null);
	     
		definition.addDynFieldString("language").setMandatory(true);
		definition.addDynFieldString("iconSize").setMandatory(true);
		definition.addDynFieldString("defaultSrs").setMandatory(true);
		definition.addDynFieldString("defSavePath").setMandatory(true);
		definition.addDynFieldString("defBgColor").setMandatory(true);
		    
	    definition.addDynFieldDouble("bboxminx").setMandatory(true);
	    definition.addDynFieldDouble("bboxminy").setMandatory(true);
	    definition.addDynFieldDouble("bboxw").setMandatory(true);
	    definition.addDynFieldDouble("bboxh").setMandatory(true);
	    
	    definition.addDynFieldBoolean("useDefProject").setMandatory(true);
	    definition.addDynFieldString("defProject").setMandatory(true);
	    definition.addDynFieldLong("maxMemShpSize").setMandatory(true);
	   
	}

	public void saveToState(PersistentState state) throws PersistenceException {
		
		state.set("language", localeLanguage);
		state.set("iconSize", size_Factor);
		state.set("defaultSrs", defaultSRS);
		state.set("defSavePath", defaultSavePath.getAbsolutePath());
		String co =
			"" + defaultBgColor.getRed() +
			"," + defaultBgColor.getGreen() +
			"," + defaultBgColor.getBlue();  
		state.set("defBgColor", co);

		state.set("bboxminx", defaultBBox.getMinX());
		state.set("bboxminy", defaultBBox.getMinY());
		state.set("bboxw", defaultBBox.getWidth());
		state.set("bboxh", defaultBBox.getHeight());
		
		state.set("useDefProject", useDefaultProject);
		if (defaultProject != null) {
			state.set("defProject", defaultProject.getAbsolutePath());
		}
		state.set("maxMemShpSize", maxMemShpSize);
	}
	
	public static void store(AppSettings sett) {
		
		try {
			PersistenceManager pma = ToolsLocator.getPersistenceManager();
			File fout = new File(System.getProperty("user.home") + File.separator + APP_SETTINGS_FILE_NAME);
			if (fout.exists()) {
				fout.delete();
			}
			PersistentState psta = pma.getState(sett); 
			FileOutputStream fos = new FileOutputStream(fout);
			pma.saveState(psta, fos);
			fos.close();
		} catch (Exception ex) {
			logger.error("While storing app settings: " + ex.getMessage());
		}
		
	}
	
	
	public static void load(AppSettings sett) {
		
		try {
			PersistenceManager pma = ToolsLocator.getPersistenceManager();
			File fiin = new File(System.getProperty("user.home") + File.separator + APP_SETTINGS_FILE_NAME);
			if (!fiin.exists()) {
				logger.warn("App settings file not found. Creating...");
				store(sett);
				return;
			}
			FileInputStream fis = new FileInputStream(fiin);
			PersistentState psta = pma.loadState(fis);
			sett.loadFromState(psta);
			fis.close();
		} catch (Exception ex) {
			logger.error("While loading app settings: " + ex.getMessage());
		}
		
	}


}
