/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.ui.lw;

import java.awt.Color;
import java.awt.Font;
import java.awt.Image;
import java.awt.Insets;

import org.gvsig.mobile.ui.awt.ImageUtils;
import org.osgi.framework.Bundle;
import org.zaval.lw.LwAdvViewMan;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwImgRender;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.LwView;
import org.zaval.lw.event.LwActionListener;

/**
 * Some GUI constants regarding control sizes, margins, etc.
 * 
 * @author jldominguez
 * 
 */
public class DialogConstants {

	public static int COLOR_BUTTON_PREF_SIZE = 24;

	public static final int YSTART = 5 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static final int YSPACE = 3 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static final int XSPACE = 3 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static final int INSIDE_MARGIN = GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static final int CONTROL_HEIGHT = 22 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static final int TABLE_LABEL_HEIGHT = 16 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static final int CHOICE_FRIENDLY_HEIGHT = 20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static final int SMALL_CONTROL_HEIGHT = 16 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static final int LABELS_WIDTH = 60 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;

	public static final String DIALOG_FONT_NAME = "Dialog";
	public static final Font DIALOG_SMALL_ITALIC_FONT = new Font(
			DIALOG_FONT_NAME, 2,
			10 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font DIALOG_ITALIC_FONT = new Font(DIALOG_FONT_NAME, 2,
			10 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font DIALOG_SMALL_BOLD_FONT = new Font(
			DIALOG_FONT_NAME, 1,
			11 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font DIALOG_BIG_BOLD_FONT = new Font(DIALOG_FONT_NAME,
			1, 22 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font DIALOG_SMALL_FONT = new Font(DIALOG_FONT_NAME, 0,
			10 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font DIALOG_BIG_FONT = new Font(DIALOG_FONT_NAME, 0,
			20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font TABLE_SMALL_FONT = new Font(DIALOG_FONT_NAME, 0,
			9 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font TABLE_SMALL_ITALIC_FONT = new Font(
			DIALOG_FONT_NAME, 2,
			9 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font MENU_FONT = new Font(DIALOG_FONT_NAME, 0,
			9 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
	public static final Font BIG_FONT = new Font(DIALOG_FONT_NAME, 1,
			14 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);

	public static final Color DIALOG_TITLE_COLOR = new Color(255, 255, 255);

	public static final String DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME = "V.png";
	public static final String DIALOG_SMALL_ADD_BUTTON_IMAGE_FILE_NAME = "add.png";
	public static final String DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME = "X.png";

	private static Insets textFieldInsets = (Insets) LwToolkit
			.getStaticObj("tf.insets");
	private static Insets labelInsets = (Insets) LwToolkit
			.getStaticObj("lb.insets");

	public static Insets getTextFieldInsets() {
		return textFieldInsets;
	}

	public static Insets getLabelInsets() {
		return labelInsets;
	}

	public static LwConstraints getTextFieldConstraints() {
		final LwConstraints l = new LwConstraints();
		l.insets = getTextFieldInsets();
		l.fill = LwToolkit.HORIZONTAL;
		return l;
	}

	public static LwConstraints getLabelConstraints() {
		final LwConstraints l = new LwConstraints();
		l.insets = getLabelInsets();
		return l;
	}

	public static LwButton getButton(Bundle bundle, String onImg,
			String offImg, LwActionListener listener) {
		LwButton resp = new LwButton();
		if (listener != null)
			resp.addActionListener(listener);

		LwAdvViewMan man = new LwAdvViewMan();
		Image imgg = ImageUtils.getResourceImage(bundle,onImg);
		man.put("bt.on", new LwImgRender(imgg, LwView.ORIGINAL));
		man.put("bt.off", new LwImgRender(ImageUtils.getResourceImage(bundle,
				offImg), LwView.ORIGINAL));
		resp.setViewMan(man);

		resp.setInsets(0, 5, 0, 5);
		resp.setOpaque(false);
		return resp;
	}

	public static LwButton getButton(String onImg, String offImg,
			LwActionListener listener) {
		LwButton resp = new LwButton();
		if (listener != null)
			resp.addActionListener(listener);

		LwAdvViewMan man = new LwAdvViewMan();
		
		Image img = ImageUtils.getResourceImage(onImg);
		man.put("bt.on", new LwImgRender(img, LwView.ORIGINAL));
		man.put("bt.off", new LwImgRender(ImageUtils.getResourceImage(offImg),
				LwView.ORIGINAL));
		resp.setViewMan(man);

		resp.setInsets(0, 5, 0, 5);
		resp.setOpaque(false);
		return resp;
	}

	public static LwLabel getDefaultLabel(String title) {
		LwLabel l = new LwLabel(title);
		l.getTextRender().setFont(DialogConstants.getDefaultFont());
		return l;
	}

	public static LwLabel getDefaultTitleLabel(String title) {
		LwLabel l = new LwLabel(title);
		l.getTextRender().setFont(DialogConstants.getDefaultTitleFont());
		return l;
	}

	public static LwTextField getDefaultTextField() {
		LwTextField l = new LwTextField("");
		l.getTextRender().setFont(DialogConstants.getDefaultFont());
		return l;
	}

	public static boolean isResolution640() {
		// if (LwToolkit.getScreenSize().width > 240) {
		//			
		// }
		if (ImageUtils.IMG_SKIN_PREFIX.compareTo("img48") == 0)
			return true;
		return false;
	}

	public static Font getDefaultFont() {
		Font f = (Font) LwToolkit.getStaticObj("def.fn");
		return f;
	}

	public static Font getDefaultTitleFont() {
		return (Font) LwToolkit.getStaticObj("def.bfn");
	}

	public static int getTitleHeight() {
		if (isResolution640()) {
			return 36;
		}
		return 18;
	}
}
