/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 * 
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package org.gvsig.mobile.ui.lw;

import java.awt.Color;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Locale;

import org.gvsig.i18n.Messages;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Class that defines global constants to all the project
 *
 * @author mmontesinos
 *
 */
public class GvsigMobileGUIConstants {

	private static Logger logger = LoggerFactory.getLogger(GvsigMobileGUIConstants.class);

	/**
	 * Size factor to multiply by buttons and other widgets sizes due to 
	 * screen resolution (detected in phoneME and OpenMoko).
	 * - The default value is 1, but varies with screen resolutions
	 */
	public static int GENERAL_SIZE_FACTOR = 1; //IconResourceReader.getScreenSizeFactor();
	public static int DEFAULT_SIZE_FACTOR = 1; //IconResourceReader.getScreenSizeFactor();
	
	static {
		try {
//			AppSettings.load();
			configureLocales();
		} catch (Exception e) {
			logger.error("While loading appsettings: " + e.getMessage());
		}
		// updateGeneralSize();
	}

	
	// public static final int GENERAL_SIZE_FACTOR = 2;
	
	/**
	 * gvSIGMobile size of small icons or buttons
	 * . The value is  16
	 */
	public static final int ICON_SIZE_SMALL = 16 * GENERAL_SIZE_FACTOR;

	/**
	 * The default width of the buttons in a toolbar
	 * The value is  24
	 */
	public static final int DEFAULT_BUTTON_WIDTH = 24 * GENERAL_SIZE_FACTOR;

	/**
	 * The default height of the buttons in a toolbar
	 * - The value is  24
	 */
	public static final int DEFAULT_BUTTON_HEIGHT = 24 * GENERAL_SIZE_FACTOR;

	/**
	 * The height of a toolbar
	 * - The value is  24 (= DEFAULT_BUTTON_HEIGHT)
	 */
	public static final int TOOLBAR_HEIGHT = DEFAULT_BUTTON_HEIGHT;

	/**
	 * The default width of a group button
	 */
	public static final int DEFAULT_TB_GROUP_BUTTON_WIDTH = 32 * GENERAL_SIZE_FACTOR;

	/**
	 * The default height of a group button
	 */
	public static final int DEFAULT_TB_GROUP_BUTTON_HEIGHT = 24 * GENERAL_SIZE_FACTOR;

	/**
	 * The default width of the help button in a toolbar
	 */
	public static final int HELP_ICON_WIDTH = 8 * GENERAL_SIZE_FACTOR;

	/**
	 * The default width of the arrow button in a tool bar combo button
	 * - The value is  8
	 */
	public static final int ARROW_WIDTH = 8 * GENERAL_SIZE_FACTOR;

	/**
	 * The default width of the combo button in a tool bar including active
	 * button + drop down
	 * - The value is  32
	 */
	public static final int TB_COMBO_BUTTON_WIDTH = GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH + GvsigMobileGUIConstants.ARROW_WIDTH;
	/**
	 * Image of a drop-down combo button in the tool bar
	 * - The value is  image <i>menu_combo_down_black.png</i>
	 */
//	public static final Image DROP_DOWN_ICON =
//		ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "menu_combo_down_black.png"); //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Selected image of a drop-down combo button in the tool bar
	 * - The value is  image <i>menu_combo_down_orange.png</i>
	 */
//	public static final Image DROP_UP_ICON =
//		ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "menu_combo_up_orange.png"); //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Default width of drop-down menus
	 * - The value is  160
	 */
	public static final int MENU_WIDTH = 160 * GENERAL_SIZE_FACTOR;

	/**
	 * Default height of drop-down menus (1 element)
	 * - The value is  28
	 */
	public static final int MENU_HEIGHT = DEFAULT_BUTTON_HEIGHT + 4;

	/**
	 * Default border color of drop-down menus
	 * - The value is  Color(32, 32, 32)
	 */
	public static final Color MENU_BORDER_COLOR = new Color(32, 32, 32);

	/**
	 * Default border shadow color of drop-down menus
	 * - The value is  RGB (192, 192, 192)
	 */
	public static final Color MENU_BORDER_SHADOW_COLOR = new Color(64, 64, 64);

	/**
	 * Default fill color of drop-down menus
	 * At the moment the icons contain background color which is equal to
	 * this color, so for changing it, the icons should be redesigned.
	 * Possible enhance: To use transparent background in icons
	 * - The value is  RGBA (0, 0, 0, 153)
	 */
	// public static final Color MENU_FILL_COLOR = new Color(0, 0, 0, 153);

	/**
	 * Default font used in texts of drop-down menus
	 * - The value is  Font("Verdana", Font.BOLD, 10)
	 */
	public static final Font MENU_FONT = new Font("Verdana", Font.BOLD, 10); //$NON-NLS-1$
	
	public static final int LABEL_LEFT_MARGIN = 4 * GENERAL_SIZE_FACTOR;
	public static final int PROCESS_ICON_SIZE = 60 * GENERAL_SIZE_FACTOR;
	public static final int WMS_LAYER_ITEM_LEVEL_SKIP = 15 * GENERAL_SIZE_FACTOR;
	public static final int WMS_LAYER_ITEM_HEIGHT = 20 * GENERAL_SIZE_FACTOR;
	
	

	/*
	 * ------------------------------------------------------------------------
	 * 					TOOLBAR BUTTONS HELP TEXTS
	 * Toolbar button help texts should begin with "TB_" for code consistency
	 * TODO: To be replaced by i18n support
	 * ------------------------------------------------------------------------
	 */

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Zoom Rectangle"
	 */
	public static final String TB_NAV_ZOOM_RECT = Messages.getText("gvMGUI.5"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Zoom In"
	 */
	public static final String TB_NAV_ZOOM_IN = Messages.getText("gvMGUI.6"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Zoom Out"
	 */
	public static final String TB_NAV_ZOOM_OUT = Messages.getText("gvMGUI.7"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Zoom Previous"
	 */
	public static final String TB_NAV_ZOOM_PREVIOUS = Messages.getText("gvMGUI.8"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Zoom Next"
	 */
	public static final String TB_NAV_ZOOM_NEXT = Messages.getText("gvMGUI.9"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Zoom Extension"
	 */
	public static final String TB_NAV_ZOOM_EXTENSION = Messages.getText("gvMGUI.10"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Zoom to Selection"
	 */
	public static final String TB_NAV_ZOOM_SELECTION = Messages.getText("gvMGUI.11"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Clear Selection"
	 */
	public static final String TB_NAV_CLEAR_SELECTION = Messages.getText("gvMGUI.12"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Manual Pan"
	 */
	public static final String TB_NAV_PAN = Messages.getText("gvMGUI.13"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Recenter"
	 */
	public static final String TB_NAV_RECENTER = Messages.getText("gvMGUI.14"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Get Information"
	 */
	public static final String TB_INFO_IDENTIFY = Messages.getText("gvMGUI.15"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "View Coordinates"
	 */
	public static final String TB_INFO_MEASURE_COORDINATES = Messages.getText("gvMGUI.16"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Measure Distance"
	 */
	public static final String TB_INFO_MEASURE_DISTANCE = Messages.getText("gvMGUI.17"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Measure Ares"
	 */
	public static final String TB_INFO_MEASURE_AREA = Messages.getText("gvMGUI.18"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Search by Attributes..."
	 */
	public static final String TB_INFO_SEARCH_ATRRIBUTES = Messages.getText("gvMGUI.19"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Select by Point"
	 */
	public static final String TB_INFO_SELECT_POINT = Messages.getText("gvMGUI.20"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Select by Rectangle"
	 */
	public static final String TB_INFO_SELECT_RECTANGLE = Messages.getText("gvMGUI.21"); //$NON-NLS-1$
	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Add Vectorial File..."
	 */
	public static final String TB_LAYERS_ADD_VECT = Messages.getText("gvMGUI.22");
	
	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Add Raster Layer..."
	 */
	public static final String TB_LAYERS_ADD_RASTER = Messages.getText("gvMGUI.23"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Add Image..."
	 */
	public static final String TB_LAYERS_ADD_IMAGE = Messages.getText("gvMGUI.24"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Add WMS..."
	 */
	public static final String TB_LAYERS_ADD_WMS = Messages.getText("gvMGUI.25"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Add Layer..."
	 */
	public static final String TB_LAYERS_ADD_LAYER = Messages.getText("gvMGUI.26"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Table of Contents..."
	 */
	public static final String TB_LAYERS_TOC = Messages.getText("gvMGUI.27"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Export Screen..."
	 */
	public static final String TB_LAYERS_EXPORT_SCREEN = Messages.getText("gvMGUI.28"); //$NON-NLS-1$

	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Layer Properties..."
	 */
	public static final String TB_LAYERS_PROPERTIES = Messages.getText("gvMGUI.29"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Save Project"
	 */
	public static final String TB_PROJECT_SAVE = Messages.getText("gvMGUI.30"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Save Project As..."
	 */
	public static final String TB_PROJECT_SAVE_AS = Messages.getText("gvMGUI.31"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Application Setup..."
	 */
	public static final String TB_PROJECT_SETUP = Messages.getText("gvMGUI.32"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Exit"
	 */
	public static final String TB_PROJECT_EXIT = Messages.getText("gvMGUI.33"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "New Project..."
	 */
	public static final String TB_PROJECT_NEW = Messages.getText("gvMGUI.34"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Open Project..."
	 */
	public static final String TB_PROJECT_OPEN = Messages.getText("gvMGUI.35"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Project Properties..."
	 */
	public static final String TB_PROJECT_PROPERTIES = Messages.getText("gvMGUI.36"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "GPS Setup..."
	 */
	public static final String TB_GPS_SETUP = Messages.getText("gvMGUI.37"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Connect/Disconnect"
	 */
	public static final String TB_GPS_CONNECT = Messages.getText("gvMGUI.38"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Display Coordinates"
	 */
	public static final String TB_GPS_DISPLAY_COORDINATES = Messages.getText("gvMGUI.39"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Display Position"
	 */
	public static final String TB_GPS_DISPLAY_POSITION = Messages.getText("gvMGUI.40"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Start/Stop Tracklog"
	 */
	public static final String TB_GPS_START_TRACKLOG = Messages.getText("gvMGUI.41"); //$NON-NLS-1$

	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "View GPS Activity..."
	 */
	public static final String TB_GPS_ACTIVITY = Messages.getText("gvMGUI.42"); //$NON-NLS-1$
	/** 
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "New Waypoint..."
	 */
	public static final String TB_GPS_WAYPOINT = Messages.getText("gvMGUI.43"); //$NON-NLS-1$
	
	public static final String TB_GPS_FINISHPOINT = Messages.getText("GPSFinishPointDialog.3"); //$NON-NLS-1$

	public static final String TB_EDITING_ADD_POINT =Messages.getText("Editing.1");//$NON-NLS-1$

	public static final String TB_EDITING_ADD_GPS_VERTEX = Messages.getText("Editing.2");//$NON-NLS-1$

	public static final String TB_EDITING_ADD_LINE = Messages.getText("Editing.3");//$NON-NLS-1$

	public static final String TB_EDITING_ADD_POLYGON = Messages.getText("Editing.4");//$NON-NLS-1$

	public static final String TB_EDITING_ADD_AUTOMATIC_VERTEX = Messages.getText("Editing.5"); 
	/**
	 * gvSIGMobile toolbar button help text.
	 * - The value is  "Export to GML File..."
	 */
	public static final String TB_FILE_EXPORT = Messages.getText("gvMGUI.50");
		
	/*
	 * ------------------------------------------------------------------------
	 *                        ERROR CONSTANTS
	 * Error constants should begin with "ERROR_" for code consistency
	 * ------------------------------------------------------------------------
	 */
	
	/** 
	 * gvSIGMobile error constant. Unspecified (general) Error
	 * - The value is  1
	 */
	public static final int ERROR_UNSPECIFIED = 1;
	
	/** 
	 * gvSIGMobile error constant because the index is < 0
	 * - The value is  2
	 */
	public static final int ERROR_INDEX_UNDER_0 = 2;

	/** 
	 * gvSIGMobile error constant because the maximum button capacity in a
	 * toolbar has been reached and no more buttons can be added
	 * - The value is  3
	 */
	public static final int ERROR_TB_BUTTON_CAPACITY = 3;

	/** 
	 * gvSIGMobile error constant because a button can't be added to a toolbar
	 * leaving gaps among it and the existing buttons
	 * - The value is  4
	 */
	public static final int ERROR_TB_BUTTON_GAPS = 4;

	/** 
	 * gvSIGMobile error constant because an access to an array has been tried
	 * with an indez that is over the array's length
	 */
	public static final int ERROR_INDEX_OVERFLOW = 5;

	

	/**
	 * Gets the text description for a given error code.
	 * The constant error code must be declared and defined above in this class
	 * @param errorCode the error code of gvSIG Mobile.
	 * @return the text description of the error code.
	 *
	 */
	public static String getErrorText (int errorCode) {
		switch (errorCode) {
			case ERROR_UNSPECIFIED:			return Messages.getText("gvMGUI.44"); //$NON-NLS-1$
			case ERROR_INDEX_UNDER_0:		return Messages.getText("gvMGUI.45"); //$NON-NLS-1$
			case ERROR_TB_BUTTON_CAPACITY:	return Messages.getText("gvMGUI.46"); //$NON-NLS-1$
			case ERROR_TB_BUTTON_GAPS:		return Messages.getText("gvMGUI.47"); //$NON-NLS-1$
			case ERROR_INDEX_OVERFLOW:		return Messages.getText("gvMGUI.48"); //$NON-NLS-1$

		}
		return Messages.getText("gvMGUI.49"); //$NON-NLS-1$
	}



	


	private static int getGeneralFactorFromSize(String sizeFactor) {
		
		String size = sizeFactor.substring(3);
		
		int resp = DEFAULT_SIZE_FACTOR;
		
		int parsed = 0;
		try {
			parsed = Integer.parseInt(size);
			resp = parsed / 24;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return resp;
	}
	
	/**
	 * Configures the locales (languages and local resources) to be used
	 * by the application.
	 *
	 * First it tries to get the locale from the command line parameters,
	 * then the andami-config file is checked.
	 *
	 * The locale name is normalized to get a two characters language code
	 * as defined by ISO-639-1 (although ISO-639-2/T three characters codes
	 * are also accepted from the command line or the configuration file).
	 *
	 * Finally, the gvsig-i18n library and the default locales for Java and
	 * Swing are configured.
	 *
	 */
	public static void configureLocales() {
		//		 Configurar el locale
        String localeStr = null;

//        localeStr = (String) GvsigMobileApp.getParams().get(GvsigMobileApp.LANGUAGE_PARAM);
		if (localeStr == null) localeStr = AppSettings.localeLanguage;
		if (localeStr == null) localeStr = AppSettings.defaultLocaleLanguage;
		
		localeStr = normalizeLanguageCode(localeStr);
		Locale locale = new Locale(localeStr, "");
		Locale.setDefault(locale);
        org.gvsig.i18n.Messages.addLocale(locale);

		// add english and spanish as fallback languages
        if (localeStr.equals("ca")||localeStr.equals("gl")||localeStr.equals("eu")||localeStr.equals("va")) {
        	// prefer Spanish for languages spoken in Spain
        	org.gvsig.i18n.Messages.addLocale(new Locale("es", ""));
        	org.gvsig.i18n.Messages.addLocale(new Locale("en", ""));
        }
        else {
        	// prefer English for the rest
        	org.gvsig.i18n.Messages.addLocale(new Locale("en", ""));
        	// System.out.println("added locale english");
    		org.gvsig.i18n.Messages.addLocale(new Locale("es", ""));
        }
	}
	
	/**
	 * List of locales
	 */
	public static final File ISO_639_FILE =
		IconResourceReader.getWorkingDirFile("resources", "iso_639.tab");
	
	
	/**
	 * gets the ISO 839 two-characters-long language code matching the
	 * provided language code (which may be an ISO 839-2/T
	 * three-characters-long code or an ISO 839-1 two-characters-long
	 * code).
	 *
	 * If the provided parameter is already two characters long, it
	 * returns the parameter without any modification.
	 * 
	 * @param langCode
	 * @return the ISO 839 two-characters-long language code
	 */
	private static String normalizeLanguageCode(String langCode) {
		//final String fileName = "iso_639.tab";
		
		if (langCode.length()==2)
			return langCode;
		else if (langCode.length()==3) {
			if (langCode.equals("va") || langCode.equals("val")) { // special case for Valencian
				return "ca";
			}
			//URL isoCodes = Launcher.class.getClassLoader().getResource(fileName);
			if (ISO_639_FILE!=null) {
				try {
					BufferedReader reader =
						new BufferedReader(new FileReader(ISO_639_FILE));//, "ISO-8859-1"));
						String line;

						while ((line = reader.readLine()) != null) {
							String[] language = org.gvsig.compat.CompatLocator.getStringUtils().split(line, "\t");
							if (language[0].equals(langCode)) // first column is the three characters code
								return language[2]; // third column i the two characters code
						}
				}
				catch (IOException ex) {
					//logger.error(Messages.getString("Error_reading_isocodes_file"), ex);
					return "es";
				}
			}
			else {
				//logger.error(Messages.getString("Error_reading_isocodes_file"));
				return "es";
			}
		}
		return "es";
	}
	
	

}
