package org.gvsig.mobile.ui.lw;

import java.awt.Color;
import java.util.Timer;
import java.util.TimerTask;

import org.cresques.Messages;
import org.gvsig.mobile.core.extensions.interfaces.IFileProviderFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.tasks.WorkQueue;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.tools.observer.Observable;
import org.gvsig.tools.observer.Observer;
import org.zaval.lw.LwBorder;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwProgress;
import org.zaval.lw.LwTitledBorder;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.event.LwActionListener;

public class LongTaskDialog extends LwPanel implements
TaskStatusListener, LwActionListener, Observer  {
	
	public static int PROGRESS_BAR_MAX_VALUE = 100;
	
	public static final int START_TASK_DONT = 0;
	public static final int START_TASK_WITH_EXECUTE = 1;
	public static final int START_TASK_WITH_RUN = 2;

	public static final long UNKNOWN_TIMER_INTERVAL = 500;
	
	private LwProgress bar;
	private LwButton cancelb;
	private boolean cancel_task = false;
	private ITask theTask = null;
	
	private boolean progressKnown = false;
	private Timer unkTimer = null;
	// private String cancelmessage = "Task in progress...";
	
	public LongTaskDialog(String msg, ITask t, int parentwidth, int howstart, boolean progress_known) {
		
		theTask = t;
		progressKnown = progress_known;
		theTask.addObserver(this);
		
		int w = (4 * parentwidth) / 5;
		setPSSize(w, w);
		setSize(w, w);
		
		LwFlowLayout lout = new LwFlowLayout(
				LwToolkit.CENTER,
				LwToolkit.CENTER,
				LwToolkit.VERTICAL,
				35);
		
		bar = new LwProgress();
		
		if (progress_known) {
		      LwBorder bbo = new LwBorder(LwTitledBorder.PLAIN, Color.GRAY, Color.GRAY, Color.BLACK);
		      bar.setView(LwProgress.BUNDLE_VIEW, bbo);
		} else {
		      LwBorder bbo = new LwBorder(LwTitledBorder.PLAIN, Color.GRAY, Color.GRAY, Color.LIGHT_GRAY);
		      bar.setView(LwProgress.BUNDLE_VIEW, bbo);
		}
		
		
		bar.setMaxValue(PROGRESS_BAR_MAX_VALUE);
		bar.setPSSize((9*w)/10, w/6);
		bar.setSize((9*w)/10, w/6);
		cancelb = new LwButton("Cancel");
		cancelb.setPSSize(w/2, w/6);
		cancelb.setSize(w/2, w/6);
		cancelb.addActionListener(this);
		
		setLwLayout(lout);
		add(new LwLabel(msg + "   ")); // "   " avoids lwuit problem (last char cut)
		// add(new LwLabel(" "));
		add(bar);
		// add(new LwLabel(" "));
		add(cancelb);
		WindowManager.getInstance().show(this,
				Messages.getText("Task_in_progress") + "   ");
		
		
		if (!progress_known) {
			unkTimer = new Timer();
			unkTimer.schedule(new UnknownProgressBarUpdateTask(bar, 4), 0, UNKNOWN_TIMER_INTERVAL);
		}
		
		// start task or not
		switch (howstart) {
		
		case START_TASK_DONT:
			break;
		case START_TASK_WITH_EXECUTE:
			WorkQueue.getInstance().execute(theTask);
			break;
		case START_TASK_WITH_RUN:
			theTask.run();
			break;
		
		}
	}
	
	public void taskFinished(ITask task) {
		
		if (!cancel_task) {
			bar.setValue(PROGRESS_BAR_MAX_VALUE);
			if (!progressKnown) unkTimer.cancel();
			WindowManager.getInstance().closeActiveUnlessMain();
		}
		
		theTask.removeObserver(this);
		

	}

	public void taskInited(ITask task) {
		// TODO Auto-generated method stub

	}

	public void taskProgress(ITask task, int progress) {
		
		if (!progressKnown) {
			return;
		}
		
		bar.setValue(progress);
		bar.repaint();
	}

	public void actionPerformed(Object src, Object arg) {
		if (src == cancelb) {
			cancel_task = true;
			if (!progressKnown) unkTimer.cancel();
			if (theTask instanceof IFileProviderFunctionality) {
				((IFileProviderFunctionality) theTask).cancel();
			}
			theTask.stop();
			WindowManager.getInstance().closeActivePanel();
		}
	}

	public void update(Observable observable, Object notification) {
		
		if (notification instanceof Double) {
			Double val = (Double) notification;
			int newv = (int) Math.floor(val.doubleValue() * PROGRESS_BAR_MAX_VALUE);
			bar.setValue(newv);
			bar.repaint();
		}
		
	}
	
	private class UnknownProgressBarUpdateTask extends TimerTask {

		private LwProgress zerohunbar = null;
		private int prog = 0;
		private int inc = 20;
		
		public UnknownProgressBarUpdateTask(LwProgress b, int nparts) {
			zerohunbar = b;
			inc = 100 / nparts;
		}
		
		public void run() {
			
			prog = (prog + inc) % (100+inc);
			zerohunbar.setValue(prog);
			zerohunbar.repaint();
		}
		
	}
	


}
