package org.gvsig.mobile.ui.lw;

import java.awt.Dimension;

import org.cresques.Messages;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwWrappedText;
import org.zaval.lw.event.LwActionListener;

public class MessageBox extends OkCancelDialog {

	private static String message = "";
	private static LwActionListener _listener;

	public static final int MESSAGE_OK = 1;
	public static final int MESSAGE_CANCEL = 2;

	public static final int MESSAGE_TYPE_ERROR = 3;
	public static final int MESSAGE_TYPE_WARNING = 4;
	public static final int MESSAGE_TYPE_INFO = 5;

	public static final int MESSAGE_ONLY_OK = 6;
	public static final int MESSAGE_OK_CANCEL = 7;
	public static final int MESSAGE_ONLY_CANCEL = 8;

	private static int _type = 3;
	private static int button = 6;

	private MessageBox(boolean scrollable, BundleContext bc, boolean init) {
		super(scrollable, bc, init);
		// TODO Auto-generated constructor stub
	}

	/**
	 * Shows a modal dialog to inform the user about something. A
	 * LwActionListener must be registered to the MessageBox so it can be
	 * informed about the action selected.
	 * 
	 * @param scrollable
	 *            If the dialog shows a scroll bar or not
	 * @param bc
	 *            The bundle context
	 * @param text
	 *            The text to be shown
	 * @param listener
	 * 
	 *            A LwActionListener to be informed about button pressed. If OK Button
	 *            is pressed {@link #MESSAGE_OK} tag will be sent as the data on
	 *            the actionPerformed method to the listener, else
	 *            {@link #MESSAGE_CANCEL} tag will be sent
	 * @param type
	 *            The type of the dialog. It is used to set the appropriate
	 *            title {@link #MESSAGE_TYPE_ERROR} Error
	 *            {@link #MESSAGE_TYPE_INFO} Info {@link #MESSAGE_TYPE_WARNING}
	 *            Warning
	 * @param buttons
	 *            The number and kind of buttons of the dialog
	 *            {@link #MESSAGE_OK_CANCEL} Both ok and cancel buttons
	 *            {@link #MESSAGE_ONLY_CANCEL} Only cancel button
	 *            {@link #MESSAGE_ONLY_OK} Only ok button
	 */
	public static void showDialog(boolean scrollable, BundleContext bc,
			String text, LwActionListener listener, int type, int buttons) {
		button = buttons;
		_type = type;
		message = MyTextWrapper.wrap(text, 35, null, true);
		_listener = listener;
		// MessageBox m = new MessageBox(scrollable, bc, true);
		MessageBox m = new MessageBox(true, bc, true);
		m.showDialog();
	}

	public Dimension getDesiredSize() {
		// TODO Auto-generated method stub
		if (mapControl != null) {
			int w = mapControl.getWidth() - 25;
			return new Dimension(w,w);
		} else {
			return new Dimension(220, 220);
		}
	}

	public boolean isMaximized() {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 * Extend this method to send whatever Message to the listener
	 */
	public void actionPerformed(Object arg0, Object arg1) {
		
		if (_listener != null) {
			if ((arg0 == this.getOkButton())) {
				_listener.actionPerformed(arg0, new Integer(MESSAGE_OK));
			} else {
				_listener.actionPerformed(arg0, new Integer(MESSAGE_CANCEL));
			}
		}
		close();
	}

	public LwPanel buildMainPanel() {
		LwPanel panel = new LwPanel();
		panel.setLwLayout(new LwBorderLayout());

		// LwLabel label = DialogConstants.getDefaultLabel(message);
		
		LwWrappedText wtext  = new LwWrappedText(message);
	
		LwLabel label = new LwLabel(wtext);
		
		
		panel.add(LwBorderLayout.CENTER, label);
		switch (this.button) {
		case MessageBox.MESSAGE_OK_CANCEL:
			break;
		case MessageBox.MESSAGE_ONLY_CANCEL:
			this.getOkButton().setVisible(false);
			break;
		case MessageBox.MESSAGE_ONLY_OK:
			this.getCancelButton().setVisible(false);
			break;
		default:
			break;
		}
		return panel;
	}

	public String getTitle() {
		switch (_type) {
		case MessageBox.MESSAGE_TYPE_ERROR:
			return Messages.getText("Error");
		case MessageBox.MESSAGE_TYPE_INFO:
			return Messages.getText("Info");
		case MessageBox.MESSAGE_TYPE_WARNING:
			return Messages.getText("Warning");
		default:
			return "";
		}
	}

}
