/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.ui.lw;

import java.awt.Color;
import java.awt.Image;

import org.gvsig.mobile.ui.awt.ImageUtils;
import org.zaval.lw.LwAdvViewMan;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwImgRender;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwView;
import org.zaval.lw.event.LwActionListener;

/**
 * A FlowLayout Panel with two buttons, to accept or decline. 
 * @author aromeu
 *
 */
public class OkCancelComposite extends LwPanel {
	
	private LwButton okButton;
	private LwButton cancelButton;

	/**
	 * The constructor
	 * @param listener - The listener to perform the actions associated to the ok and cancel
	 * events
	 */
	public OkCancelComposite(final LwActionListener listener) {
		this.initialize(listener);
		this.setBackground(Color.white);		
//		this.setOpaque(false);
//		super.setOpaque(false);
	}
	
	/**
	 * Initializes the panel
	 * @param listener - The listener
	 */
	public void initialize(final LwActionListener listener) {
		
		setLwLayout(new LwFlowLayout());
		okButton = new DisableableButton("V", "Vdisabled", listener);
		// cancelButton = DialogConstants.getButton("X", "X", listener);		
		cancelButton = new DisableableButton("X", "X", listener);
		
		add(okButton);
		add(cancelButton);
		
		this.setInsets(5, 0, 5, 0);
		
//		setBackground(Color.WHITE);
//		okButton.setBackground(Color.WHITE);
//		cancelButton.setBackground(Color.WHITE);		
	}
	
	public LwButton getOkButton() {
		return okButton;
	}

	public LwButton getCancelButton() {
		return cancelButton;
	}
	
	private class DisableableButton extends LwButton {

		private Image enaimg = null;
		private Image disimg = null;
		private LwAdvViewMan enaman = new LwAdvViewMan();
		private LwAdvViewMan disman = new LwAdvViewMan();
		
		public DisableableButton(String enabledimage, String disabledimage, LwActionListener listenr) {
			
			if (listenr != null) addActionListener(listenr);

			enaimg = ImageUtils.getResourceImage(enabledimage);
			enaman.put("bt.on", new LwImgRender(enaimg, LwView.ORIGINAL));
			enaman.put("bt.off", new LwImgRender(enaimg, LwView.ORIGINAL));
			disimg = ImageUtils.getResourceImage(disabledimage);
			disman.put("bt.on", new LwImgRender(disimg, LwView.ORIGINAL));
			disman.put("bt.off", new LwImgRender(disimg, LwView.ORIGINAL));
			
			setInsets(0, 5, 0, 5);
			setEnabled(true);
			setOpaque(false);
		}
		
		public void setEnabled(boolean ena) {
			super.setEnabled(ena);
			if (ena) {
				setViewMan(enaman);
			} else {
				setViewMan(disman);
			}
		}
		
	}
}
