/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.ui.lw;

import java.awt.Color;
import java.awt.Dimension;

import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.ui.awt.IWindowInfo;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwComposite;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.event.LwActionListener;

public abstract class OkCancelDialog extends LwPanel implements IWindowInfo,
LwComposite, LwActionListener {
	
	protected LwPanel mainPanel;
	protected OkCancelComposite okCancelComp;
	protected static Logger logger = LoggerFactory.getLogger(OkCancelDialog.class);

	
	protected BundleContext bc;
	protected MapControl mapControl=null;

	public OkCancelDialog(final boolean scrollable, BundleContext bc) {
		this(scrollable, bc, true);
	}
	
	public OkCancelDialog(final boolean scrollable, BundleContext bc, boolean init) {
		
		this.bc = bc;
		if ((this.mapControl == null) && (this.bc!=null)){
			ServiceReference cont=null;  
			cont=bc.getServiceReference(IGvSigContext.class.getName());
			IGvSigContext gv = (IGvSigContext)this.bc.getService(cont);
			this.mapControl = (MapControl)gv.getMapControl();
		}
		if (init) {
			this.initialize(scrollable);
		}
		
		this.setBackground(Color.white);		
		// this.setOpaque(false);
	}

	
	public Bundle getBundle() {
		return this.bc.getBundle();
	}
	
	public BundleContext getBundleContext() {
		return this.bc;
	}
	
	protected void initialize(final boolean scrollable) {		
		setLwLayout(new LwBorderLayout());
		add(LwBorderLayout.SOUTH, this.getOkCancelComposite());

		mainPanel = buildMainPanel();
//		mainPanel.setOpaque(false);
		
		if (scrollable) {
			final LwScrollPan l = new LwScrollPan(mainPanel);	
			l.setOpaque(false);
			add(LwBorderLayout.CENTER, l);
		} else {
			add(LwBorderLayout.CENTER, mainPanel);
		}
	}

	/**
	 * Shows the modal dialog
	 */
	public void showDialog() {
		WindowManager.getInstance().show(this, this.getTitle());
	}
	
	public abstract LwPanel buildMainPanel();
	
	public LwPanel getMainPanel() {
		return this.mainPanel;
	}
	

	private LwPanel getOkCancelComposite() {
		okCancelComp = new OkCancelComposite(this);
//		okCancelComp.setOpaque(false);
		return okCancelComp;
	}

	public Dimension getDesiredSize() {
		// TODO Auto-generated method stub
//		return new Dimension(240, 260);
		return null;
	}

	public boolean isMaximized() {
		// TODO Auto-generated method stub
		return true;
	}

	public boolean catchInput(LwComponent arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	public abstract void actionPerformed(Object arg0, Object arg1);
	
	public abstract String getTitle();

	public LwButton getOkButton() {
		return okCancelComp.getOkButton();
	}

	public LwButton getCancelButton() {
		return okCancelComp.getCancelButton();
	}
	
	public void close() {
		WindowManager.getInstance().closeActivePanel();
	}
}
