/**
 *     Caption: J2SE/J2ME(Personal Profile) Light-Weight Visual Components Library
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    4.50
 *
 * The latest product version is always available from the product's homepage:
 * http://www.lwvcl.com/
 *
 * Copyright (C) 2001-2005  Zaval Creative Engineering Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.lw.misc;

import java.awt.Color;
import java.awt.Font;

import org.gvsig.mobile.ui.lw.DialogConstants;
import org.zaval.data.Item;
import org.zaval.lw.LwCompRender;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwImage;
import org.zaval.lw.LwImgRender;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwTextRender;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.LwView;

public class LwGenericView
{
   private LwTextRender render;
   private LwCompRender crender;
   private LwPanel      panel;
   private Font         defFont;
   private Color        defColor;

   public LwGenericView()
   {
//     defFont  = (Font )LwToolkit.getStaticObj("def.fn");	   
	   defFont = DialogConstants.getDefaultFont();
	   
     defColor = (Color)LwToolkit.getStaticObj("def.col");
     render = new LwTextRender("");
     panel = new LwPanel ();
     panel.setLwLayout (new LwFlowLayout (LwToolkit.LEFT, LwToolkit.CENTER, LwToolkit.HORIZONTAL, 2));
     panel.add (new LwLabel (render));
     crender = new LwCompRender(panel);
   }

   public void setDefFont (Font f) {
     if (f == null) throw new IllegalArgumentException ();
     if (!f.equals (defFont)) defFont = f;
   }

   public void setDefColor (Color c) {
     if (c == null) throw new IllegalArgumentException ();
     if (!c.equals (defColor)) defColor = c;
   }

   protected LwView getView (Object item)
   {

	   if (item instanceof LwView) {
		   return (LwView) item;
	   }

	   Object initem = item;
	   if (item instanceof Item && (!(item instanceof CItem))) {
		   initem = ((Item) item).getValue();
	   }
	   if (initem instanceof String) {
		   render.setFont(defFont);
		   render.setForeground(defColor);
		   render.getTextModel().setText((String) initem);
		   return render;
	   } else {
		   if (initem instanceof LwView) {
			   return (LwView) initem;
		   }
	   }
	   
	   // ==================================
	   
     Object v = (item instanceof Item)?((Item)item).getValue():item;
     String s = (v == null)?"":LwUtils.fetchRenderedValue(v.toString());
     render.getTextModel().setText(s);

     render.setFont       (defFont);
     render.setForeground (defColor);
     if (item instanceof CItem)
     {
       CItem ci = (CItem)item;
       if (ci.getFont () != null) render.setFont(defFont);
       if (ci.getColor() != null) render.setForeground (ci.getColor());

       if (ci.getIcon () != null)
       {
         if (panel.count() > 1) panel.remove(0);
         panel.insert (0, null, new LwImage(ci.getIcon()));
         panel.setOpaque (false);
         return crender;
       }
     }

     if (s.length() > 0 && s.charAt(0) == '@')
     {
       int index = s.indexOf (':', 1);
       if (index > 0)
       {
         LwPanel pan = new LwPanel ();
         pan.setLwLayout (new LwFlowLayout (LwToolkit.LEFT, LwToolkit.CENTER, LwToolkit.HORIZONTAL, 2));
         pan.add(new LwImage((LwImgRender)LwToolkit.getView (s.substring(1, index))));
         LwLabel l = new LwLabel(s.substring(index + 1));
         l.getTextRender().setFont(defFont);
         pan.add(l);
         pan.setOpaque (false);
         return new LwCompRender(pan);
       }
       return LwToolkit.getView (s.substring(1));
     }

     return render;
   }
}
